@@ -223,10 +223,9 @@ int SG_Compare_Char_Ptr(const void *a, const void *b)
 //---------------------------------------------------------
 double	SG_Degree_To_Decimal(double Deg, double Min, double Sec)
 {
-	return( Deg > 0.
-		? (Deg + Min / 60. + Sec / 3600.)
-		: (Deg - Min / 60. - Sec / 3600.)
-	);
+	if( Deg < 0. ) { Min = -fabs(Min); Sec = -fabs(Sec); }
+
+	return( Deg + Min / 60. + Sec / 3600. );
 }
 
 //---------------------------------------------------------
