@@ -66,9 +66,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -107,7 +107,7 @@ END_EVENT_TABLE()
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -143,9 +143,9 @@ public:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -153,16 +153,19 @@ IMPLEMENT_CLASS(CParameters_Control, wxPanel)
 
 //---------------------------------------------------------
 BEGIN_EVENT_TABLE(CParameters_Control, wxPanel)
-	EVT_SIZE			(CParameters_Control::On_Size)
-	EVT_KEY_DOWN		(CParameters_Control::On_Key)
+	EVT_SIZE     (CParameters_Control::On_Size)
+	EVT_KEY_DOWN (CParameters_Control::On_Key)
 
-	EVT_PG_SELECTED		(ID_WND_PARM, CParameters_Control::On_PG_Selected)
-	EVT_PG_CHANGED		(ID_WND_PARM, CParameters_Control::On_PG_Changed)
+	EVT_PG_SELECTED      (ID_WND_PARM, CParameters_Control::On_PG_Selected)
+	EVT_PG_CHANGED       (ID_WND_PARM, CParameters_Control::On_PG_Changed)
+
+	EVT_PG_ITEM_COLLAPSED(ID_WND_PARM, CParameters_Control::On_PG_Collapsed)
+	EVT_PG_ITEM_EXPANDED (ID_WND_PARM, CParameters_Control::On_PG_Expanded)
 END_EVENT_TABLE()
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -200,7 +203,7 @@ CParameters_Control::~CParameters_Control(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -246,9 +249,35 @@ void CParameters_Control::On_PG_Changed(wxPropertyGridEvent &event)
 	event.Skip();
 }
 
+//---------------------------------------------------------
+void CParameters_Control::On_PG_Collapsed(wxPropertyGridEvent &event)
+{
+	CSG_Parameter *pParameter = _Get_Parameter(event.GetProperty());
+
+	if( pParameter && (pParameter = m_pOriginal->Get_Parameter(pParameter->Get_Identifier())) )
+	{
+		pParameter->Set_Collapsed(true);
+	}
+
+	event.Skip();
+}
+
+//---------------------------------------------------------
+void CParameters_Control::On_PG_Expanded(wxPropertyGridEvent &event)
+{
+	CSG_Parameter *pParameter = _Get_Parameter(event.GetProperty());
+
+	if( pParameter && (pParameter = m_pOriginal->Get_Parameter(pParameter->Get_Identifier())) )
+	{
+		pParameter->Set_Collapsed(false);
+	}
+
+	event.Skip();
+}
+
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -306,7 +335,7 @@ bool CParameters_Control::Load(void)
 
 	if( DLG_Open(File_Path, ID_DLG_PARAMETERS_OPEN) )
 	{
-		CSG_File	File(&File_Path);
+		CSG_File	File(File_Path);
 
 		m_pParameters->Set_Callback(false);
 
@@ -357,7 +386,7 @@ bool CParameters_Control::Save(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -419,7 +448,7 @@ bool CParameters_Control::Set_Parameters(CSG_Parameters *pParameters)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -551,12 +580,14 @@ void CParameters_Control::_Add_Property(wxPGProperty *pParent, CSG_Parameter *pP
 wxPGProperty * CParameters_Control::_Get_Property(wxPGProperty *pParent, CSG_Parameter *pParameter)
 {
 	#define ADD_PROPERTY(p, limit) if( pParent ) { m_pPG->Insert(pParent, -1, pProperty = p); } else { m_pPG->Append(pProperty = p); }\
-		if( pParameter->is_Information() ) { m_pPG->LimitPropertyEditing(pProperty); m_pPG->EnableProperty(pProperty, false); } else\
-		if( limit ) { m_pPG->LimitPropertyEditing(pProperty); } { CSG_String s, sDesc;\
-		sDesc = pParameter->Get_Description(PARAMETER_DESCRIPTION_TYPE);\
-		s     = pParameter->Get_Description(PARAMETER_DESCRIPTION_TEXT      ); if( !s.is_Empty() ) { sDesc.Append("\n___\n"); sDesc.Append(s); }\
-		s     = pParameter->Get_Description(PARAMETER_DESCRIPTION_PROPERTIES); if( !s.is_Empty() ) { sDesc.Append("\n___\n"); sDesc.Append(s); }\
-		m_pPG->SetPropertyHelpString(pProperty, sDesc.c_str()); }
+		if( pParameter->is_Information() ) { m_pPG->LimitPropertyEditing(pProperty); m_pPG->EnableProperty(pProperty, false); }\
+		else if( limit ) { m_pPG->LimitPropertyEditing(pProperty); }\
+		{   CSG_String s, sDesc;\
+		    sDesc = pParameter->Get_Description(PARAMETER_DESCRIPTION_TYPE);\
+		    s     = pParameter->Get_Description(PARAMETER_DESCRIPTION_TEXT      ); if( !s.is_Empty() ) { sDesc.Append("\n___\n"); sDesc.Append(s); }\
+		    s     = pParameter->Get_Description(PARAMETER_DESCRIPTION_PROPERTIES); if( !s.is_Empty() ) { sDesc.Append("\n___\n"); sDesc.Append(s); }\
+		    m_pPG->SetPropertyHelpString(pProperty, sDesc.c_str());\
+		}
 
 	wxPGProperty *pProperty = NULL; wxString Name(pParameter->Get_Name()), ID(pParameter->Get_Identifier());
 
@@ -577,34 +608,29 @@ wxPGProperty * CParameters_Control::_Get_Property(wxPGProperty *pParent, CSG_Par
 		}
 		break;
 
-	case PARAMETER_TYPE_Bool            :
+	case PARAMETER_TYPE_Bool           :
 		ADD_PROPERTY(new wxBoolProperty       (Name, ID, pParameter->asBool()  ), false);
 		pProperty->SetAttribute(wxPG_BOOL_USE_CHECKBOX, true);
 		break;
 
-	case PARAMETER_TYPE_Int             :
+	case PARAMETER_TYPE_Int            :
 		ADD_PROPERTY(new wxIntProperty        (Name, ID, pParameter->asInt()   ), false);
 		break;
 
-	case PARAMETER_TYPE_Double          :
+	case PARAMETER_TYPE_Double         :
 		ADD_PROPERTY(new wxFloatProperty      (Name, ID, pParameter->asDouble()), false);
 		pProperty->SetAttribute(wxPG_FLOAT_PRECISION, m_Precision);
 		break;
 
-	case PARAMETER_TYPE_Range           :
+	case PARAMETER_TYPE_Range          :
 		ADD_PROPERTY(new CParameters_PG_Range (Name, ID, pParameter, m_Precision), false);
 		break;
 
-	case PARAMETER_TYPE_Degree          :
-		ADD_PROPERTY(new CParameters_PG_Degree(Name, ID, pParameter            ), false);
+	case PARAMETER_TYPE_Degree         :
+		ADD_PROPERTY(new CParameters_PG_Degree(Name, ID, pParameter             ), false);
 		break;
 
-	case PARAMETER_TYPE_Date            :
-		ADD_PROPERTY(new wxDateProperty       (Name, ID, pParameter->asDouble()), false);	// from JDN
-		pProperty->SetAttribute(wxPG_DATE_PICKER_STYLE, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
-		break;
-
-	case PARAMETER_TYPE_String          :
+	case PARAMETER_TYPE_String         :
 		if( ((CSG_Parameter_String *)pParameter)->is_Password() )
 		{
 			ADD_PROPERTY(new wxStringProperty    (Name, ID, pParameter->asString()), false);
@@ -625,6 +651,13 @@ wxPGProperty * CParameters_Control::_Get_Property(wxPGProperty *pParent, CSG_Par
 		ADD_PROPERTY(new CParameters_PG_Colors(Name, ID, pParameter), true);
 		break;
 
+	case PARAMETER_TYPE_Date           :
+		#ifdef WITH_WXDATECTRL
+		ADD_PROPERTY(new wxDateProperty       (Name, ID, pParameter->asDouble() ), false);	// from JDN
+		pProperty->SetAttribute(wxPG_DATE_PICKER_STYLE, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
+		break;
+		#endif
+
 	case PARAMETER_TYPE_FilePath       :
 	case PARAMETER_TYPE_Choices        :
 	case PARAMETER_TYPE_Table_Fields   :
@@ -670,7 +703,7 @@ wxPGProperty * CParameters_Control::_Get_Property(wxPGProperty *pParent, CSG_Par
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -696,7 +729,7 @@ bool CParameters_Control::_Get_Enabled(CSG_Parameter *pParameter)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -743,8 +776,12 @@ void CParameters_Control::_Set_Parameter(wxPGProperty *pProperty)
 			break;
 
 		case PARAMETER_TYPE_Date    :
+			#ifdef WITH_WXDATECTRL
 			pParameter->Set_Value(((wxDateProperty *)pProperty)->GetDateValue().GetJDN());
 			pParameter = NULL; // no further processing
+			#else
+			pParameter->Set_Value(m_pPG->GetPropertyValueAsString(pProperty).wx_str());
+			#endif
 			break;
 		}
 
@@ -810,17 +847,6 @@ void CParameters_Control::_Update_Parameter(CSG_Parameter *pParameter)
 			((CParameters_PG_Degree *)pProperty)->Update();
 			break;
 
-		case PARAMETER_TYPE_Date:
-			{
-				wxDateTime Date(pParameter->asDouble());
-
-				if( ((wxDateProperty *)pProperty)->GetDateValue() != Date )
-				{
-					((wxDateProperty *)pProperty)->SetDateValue(Date);
-				}
-			}
-			break;
-
         case PARAMETER_TYPE_Color:
             if( ((wxColourProperty *)pProperty)->GetVal().m_colour != pParameter->asColor() )
             {
@@ -842,6 +868,9 @@ void CParameters_Control::_Update_Parameter(CSG_Parameter *pParameter)
 			((CParameters_PG_Choice *)pProperty)->Update();
 		break;
 
+		#ifndef WITH_WXDATECTRL
+		case PARAMETER_TYPE_Date           :
+		#endif
 		case PARAMETER_TYPE_Text           :
 		case PARAMETER_TYPE_FilePath       :
 		case PARAMETER_TYPE_Font           :
@@ -977,16 +1006,32 @@ void CParameters_Control::_Init_Pararameters(void)
 	{
 		for(int i=0; i<m_pParameters->Get_Count(); i++)
 		{
-			m_pParameters->Get_Parameter(i)->has_Changed(PARAMETER_CHECK_ENABLE);
+			CSG_Parameter *pParameter = m_pParameters->Get_Parameter(i);
+
+			pParameter->has_Changed(PARAMETER_CHECK_ENABLE);
+
+			wxPGProperty *pProperty = m_pPG->GetProperty(_Get_Identifier(pParameter));
+
+			if( pProperty )
+			{
+				if( pParameter->is_Collapsed() )
+				{
+					m_pPG->Collapse(pProperty);
+				}
+				else
+				{
+					m_pPG->Expand  (pProperty);
+				}
+			}
 		}
 	}
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
