@@ -71,17 +71,17 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define IS_BAND_WISE_FIT(pLayer)	(m_pLayer->Get_Type() == WKSP_ITEM_Grids && m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_OVERLAY && m_pLayer->Get_Parameter("OVERLAY_FIT")->asInt() != 0)
+#define IS_BAND_WISE_FIT (m_pLayer->Get_Type() == WKSP_ITEM_Grids && m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_OVERLAY && m_pLayer->Get_Parameter("OVERLAY_FIT")->asInt() != 0)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -102,12 +102,12 @@ BEGIN_EVENT_TABLE(CVIEW_Histogram, CVIEW_Base)
 	EVT_MENU      (ID_CMD_HISTOGRAM_GAUSSIAN    , CVIEW_Histogram::On_Gaussian)
 	EVT_MENU      (ID_CMD_HISTOGRAM_SET_MINMAX  , CVIEW_Histogram::On_Set_MinMax)
 	EVT_MENU      (ID_CMD_HISTOGRAM_AS_TABLE    , CVIEW_Histogram::On_AsTable)
-	EVT_MENU      (ID_CMD_HISTOGRAM_TO_CLIPBOARD, CVIEW_Histogram::On_ToClipboard)
+	EVT_MENU      (ID_CMD_VIEW_CLIPBOARD_COPY   , CVIEW_Histogram::On_ToClipboard)
 END_EVENT_TABLE()
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -134,7 +134,7 @@ CVIEW_Histogram::CVIEW_Histogram(CWKSP_Layer *pLayer)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -148,7 +148,7 @@ wxMenu * CVIEW_Histogram::_Create_Menu(void)
 	CMD_Menu_Add_Item(pMenu, true , ID_CMD_HISTOGRAM_GAUSSIAN);
 	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_HISTOGRAM_AS_TABLE);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_HISTOGRAM_TO_CLIPBOARD);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_VIEW_CLIPBOARD_COPY);
 
 	return( pMenu );
 }
@@ -162,7 +162,7 @@ wxToolBarBase * CVIEW_Histogram::_Create_ToolBar(void)
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_HISTOGRAM_CUMULATIVE);
 	CMD_ToolBar_Add_Item(pToolBar, true , ID_CMD_HISTOGRAM_GAUSSIAN);
 	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_HISTOGRAM_AS_TABLE);
-	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_HISTOGRAM_TO_CLIPBOARD);
+	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_VIEW_CLIPBOARD_COPY);
 
 	CMD_ToolBar_Add(pToolBar, _TL("Histogram"));
 
@@ -171,15 +171,15 @@ wxToolBarBase * CVIEW_Histogram::_Create_ToolBar(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CVIEW_Histogram::Do_Update(void)
 {
-	if( IS_BAND_WISE_FIT(pLayer) )
+	if( IS_BAND_WISE_FIT )
 	{
-		m_pLayer->Get_Classifier()->Set_Metric(0, 1, m_pLayer->Get_Value_Minimum(), m_pLayer->Get_Value_Maximum());
+		m_pLayer->Get_Classifier()->Get_Stretch().Set_Range(m_pLayer->Get_Stretch_Minimum(), m_pLayer->Get_Stretch_Maximum());
 	}
 
 	if( m_pLayer->Get_Classifier()->Histogram_Update() )
@@ -192,7 +192,7 @@ void CVIEW_Histogram::Do_Update(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -212,7 +212,7 @@ void CVIEW_Histogram::On_Paint(wxPaintEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -286,7 +286,7 @@ void CVIEW_Histogram::Draw_Histogram(wxDC &dc)
 {
 	wxRect r(Get_Histogram_Box(dc.GetSize()));
 
-	const CSG_Histogram	&Histogram	= m_pLayer->Get_Classifier()->Histogram_Get();
+	const CSG_Histogram &Histogram = m_pLayer->Get_Classifier()->Histogram_Get();
 
 	if( Histogram.Get_Class_Count() < 1 || Histogram.Get_Element_Count() < 1 )
 	{
@@ -298,14 +298,17 @@ void CVIEW_Histogram::Draw_Histogram(wxDC &dc)
 	//-----------------------------------------------------
 	int StdDev[2]; StdDev[0] = StdDev[1] = -1; CSG_Simple_Statistics s;
 
-	if( m_bGaussian && m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_LUT )
+	if( m_bGaussian
+	&&  m_pLayer->Get_Classifier()->Get_Stretch().Get_Mode() == CSG_Scaler::TMode::LINEAR
+	&&  m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_SINGLE
+	&&  m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_LUT )
 	{
 		s.Create(m_pLayer->Get_Classifier()->Statistics_Get());
 
 		if( s.Get_Count() > 0 )
 		{
-			double	Minimum = m_pLayer->Get_Classifier()->Get_RelativeToMetric(0.);
-			double	Maximum = m_pLayer->Get_Classifier()->Get_RelativeToMetric(1.);
+			double Minimum = m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(0.);
+			double Maximum = m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(1.);
 
 			double dx = (Maximum - Minimum) / (double)r.GetWidth();
 
@@ -340,7 +343,7 @@ void CVIEW_Histogram::Draw_Histogram(wxDC &dc)
 		x[0] = x[1]; x[1] = r.GetLeft() + (int)(dx * (iClass + 1.));
 		y[1] = y[0] - (int)((r.GetHeight() - 1) * Value);
 
-		if( m_bColored && m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_OVERLAY && m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_SHADE )
+		if( m_bColored && m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_OVERLAY )
 		{
 			Color = m_pLayer->Get_Classifier()->Get_Class_Color(iClass);
 		}
@@ -361,8 +364,8 @@ void CVIEW_Histogram::Draw_Histogram(wxDC &dc)
 			dc.DrawLine(ix, r.GetBottom(), ix, r.GetBottom() - (int)(dy * FUNC_NORMAL(X)));\
 		}
 
-		double Minimum = m_pLayer->Get_Classifier()->Get_RelativeToMetric(0.);
-		double Maximum = m_pLayer->Get_Classifier()->Get_RelativeToMetric(1.);
+		double Minimum = m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(0.);
+		double Maximum = m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(1.);
 
 		double dx = (Maximum - Minimum) / (double)r.GetWidth(), dy = r.GetHeight() - 1;
 
@@ -430,15 +433,15 @@ void CVIEW_Histogram::Draw_Frame(wxDC &dc)
 	//-----------------------------------------------------
 	if( m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_LUT )
 	{
-		double	dx	= r.GetWidth() / (double)Histogram.Get_Class_Count(), n;
+		double dx = r.GetWidth() / (double)Histogram.Get_Class_Count(), n = 0.;
 
 		for(size_t iClass=0; iClass<Histogram.Get_Class_Count(); iClass++, n+=dx)
 		{
 			if( iClass == 0 || n > (FontSize + 5) )
 			{
-				n	= 0.;
+				n = 0.;
 
-				int	ix	= r.GetLeft() + (int)(dx * (0.5 + iClass));
+				int ix = r.GetLeft() + (int)(dx * (0.5 + iClass));
 				dc.DrawLine(ix, r.GetBottom(), ix, r.GetBottom() + 5);
 				Draw_Text(dc, TEXTALIGN_TOPRIGHT, ix, r.GetBottom() + 7, 45.,
 					m_pLayer->Get_Classifier()->Get_Class_Name(iClass)
@@ -451,11 +454,49 @@ void CVIEW_Histogram::Draw_Frame(wxDC &dc)
 	else switch( m_XLabeling )
 	{
 	case  0: {
-		Draw_Scale(dc, wxRect(r.GetLeft(), r.GetBottom(), r.GetWidth(), 20),
-			m_pLayer->Get_Classifier()->Get_RelativeToMetric(0.),
-			m_pLayer->Get_Classifier()->Get_RelativeToMetric(1.),
-			true , true , true
-		);
+		switch( m_pLayer->Get_Classifier()->Get_Stretch_Mode() )
+		{
+		case CSG_Scaler::LINEAR:
+			Draw_Scale(dc, wxRect(r.GetLeft(), r.GetBottom(), r.GetWidth(), 20),
+				m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(0.),
+				m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(1.),
+				true , true , true
+			);
+			break;
+
+		default:
+			{
+				#define METRIC_POS_H(Value) (r.GetLeft() + (int)((double)r.GetWidth() * m_pLayer->Get_Classifier()->Get_Stretch_to_Relative(Value)))
+
+				double zMin = m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(0.);
+				double zMax = m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(1.);
+
+				wxSize sText = dc.GetTextExtent(wxString::Format("01234567"));
+
+				double zToDC = r.GetWidth() / (zMax - zMin), dz = pow(10., floor(log10(zMax - zMin)) - 1.);
+
+				while( zToDC * dz < sText.y ) { dz *= 2.; }
+
+				int x0 = METRIC_POS_H(zMin) - r.GetWidth();
+
+				double z = dz * floor(zMin / dz); if( z < zMin ) { z += dz; }
+
+				for(; z<=zMax; z+=dz)
+				{
+					int x = METRIC_POS_H(z); dc.DrawLine(x, r.GetBottom(), x, r.GetBottom() + 5);
+
+					#define METRIC_GET_STRING(z, dz) wxString::Format("%.*f", dz >= 1. ? 0 : 1 + (int)fabs(log10(dz)), z)
+
+					wxString Label(METRIC_GET_STRING(z, dz)); wxSize s(dc.GetTextExtent(Label));
+
+					if( abs(x0 - x) >= (int)(1.25 * s.x) )
+					{
+						x0 = x; Draw_Text(dc, TEXTALIGN_TOPCENTER, x, r.GetBottom() + 7, Label);
+					}
+				}
+			}
+			break;
+		}
 		break; }
 
 	default:
@@ -474,7 +515,7 @@ void CVIEW_Histogram::Draw_Frame(wxDC &dc)
 				int	ix	= r.GetLeft() + (int)(dx * i);
 				dc.DrawLine(ix, r.GetBottom(), ix, r.GetBottom() + 5);
 				Draw_Text(dc, TEXTALIGN_CENTERRIGHT, ix, r.GetBottom() + 7, 45.,
-					SG_Get_String(m_pLayer->Get_Classifier()->Get_RelativeToMetric(dz * i), -2).c_str()
+					SG_Get_String(m_pLayer->Get_Classifier()->Get_Stretch_from_Relative(dz * i), -2).c_str()
 				);
 			}
 		break; }
@@ -510,7 +551,7 @@ wxRect CVIEW_Histogram::Get_Histogram_Box(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -527,7 +568,7 @@ void CVIEW_Histogram::On_Mouse_Motion(wxMouseEvent &event)
 //---------------------------------------------------------
 void CVIEW_Histogram::On_Mouse_LDown(wxMouseEvent &event)
 {
-	if( IS_BAND_WISE_FIT(pLayer) )
+	if( IS_BAND_WISE_FIT )
 	{
 		return;
 	}
@@ -536,7 +577,6 @@ void CVIEW_Histogram::On_Mouse_LDown(wxMouseEvent &event)
 	{
 	case CLASSIFY_GRADUATED:
 	case CLASSIFY_DISCRETE :
-	case CLASSIFY_SHADE    :
 	case CLASSIFY_OVERLAY  :
 		m_bMouse_Down = true;
 		m_Mouse_Move = m_Mouse_Down = event.GetPosition();
@@ -554,14 +594,11 @@ void CVIEW_Histogram::On_Mouse_LUp(wxMouseEvent &event)
 	{
 		ReleaseMouse();
 
-		m_bMouse_Down = false;
-		m_Mouse_Move  = event.GetPosition();
+		m_bMouse_Down = false; m_Mouse_Move = event.GetPosition(); wxRect r(Get_Histogram_Box());
 
-		wxRect r(Get_Histogram_Box());
-
-		m_pLayer->Set_Color_Range(
-			m_pLayer->Get_Classifier()->Get_RelativeToMetric((double)(m_Mouse_Down.x - r.GetLeft()) / (double)r.GetWidth()),
-			m_pLayer->Get_Classifier()->Get_RelativeToMetric((double)(m_Mouse_Move.x - r.GetLeft()) / (double)r.GetWidth())
+		m_pLayer->Set_Stretch_Range(
+			m_pLayer->Get_Classifier()->Get_Stretch_from_Relative((double)(m_Mouse_Down.x - r.GetLeft()) / (double)r.GetWidth()),
+			m_pLayer->Get_Classifier()->Get_Stretch_from_Relative((double)(m_Mouse_Move.x - r.GetLeft()) / (double)r.GetWidth())
 		);
 	}
 }
@@ -569,9 +606,9 @@ void CVIEW_Histogram::On_Mouse_LUp(wxMouseEvent &event)
 //---------------------------------------------------------
 void CVIEW_Histogram::On_Mouse_RDown(wxMouseEvent &event)
 {
-	wxMenu	Menu;
+	wxMenu Menu;
 
-	if( !IS_BAND_WISE_FIT(pLayer) )
+	if( !IS_BAND_WISE_FIT )
 	{
 		CMD_Menu_Add_Item(&Menu, false, ID_CMD_HISTOGRAM_SET_MINMAX);
 		Menu.AppendSeparator();
@@ -585,7 +622,7 @@ void CVIEW_Histogram::On_Mouse_RDown(wxMouseEvent &event)
 
 	Menu.AppendSeparator();
 	CMD_Menu_Add_Item(&Menu, false, ID_CMD_HISTOGRAM_AS_TABLE);
-	CMD_Menu_Add_Item(&Menu, false, ID_CMD_HISTOGRAM_TO_CLIPBOARD);
+	CMD_Menu_Add_Item(&Menu, false, ID_CMD_VIEW_CLIPBOARD_COPY);
 
 	Menu.AppendSeparator();
 	CMD_Menu_Add_Item(&Menu, false, ID_CMD_HISTOGRAM_PARAMETERS);
@@ -595,7 +632,7 @@ void CVIEW_Histogram::On_Mouse_RDown(wxMouseEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -604,33 +641,19 @@ void CVIEW_Histogram::On_Command_UI(wxUpdateUIEvent &event)
 	switch( event.GetId() )
 	{
 	case ID_CMD_HISTOGRAM_CUMULATIVE:
-		if( m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_SINGLE
-		||  m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_RGB )
-		{
-			event.Enable(false);
-			event.Check (false);
-		}
-		else
-		{
-			event.Enable(true);
-			event.Check (m_bCumulative);
-		}
+		event.Check (m_bCumulative);
+		event.Enable(
+		    m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_SINGLE
+		);
 		break;
 
 	case ID_CMD_HISTOGRAM_GAUSSIAN:
-		if( m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_DISCRETE
-		||  m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_GRADUATED
-		||  m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_SHADE
-		||  m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_OVERLAY )
-		{
-			event.Enable(true);
-			event.Check (m_bGaussian);
-		}
-		else
-		{
-			event.Enable(false);
-			event.Check (false);
-		}
+		event.Check (m_bGaussian);
+		event.Enable(
+		    m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_SINGLE
+		 && m_pLayer->Get_Classifier()->Get_Mode() != CLASSIFY_LUT
+		 && m_pLayer->Get_Classifier()->Get_Stretch().Get_Mode() == CSG_Scaler::TMode::LINEAR
+		);
 		break;
 
 	default: break;
@@ -665,7 +688,6 @@ void CVIEW_Histogram::On_Properties(wxCommandEvent &event)
 	//-----------------------------------------------------
 	P.Set_Enabled("NCLASSES",
 		m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_GRADUATED
-	||	m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_SHADE
 	||	m_pLayer->Get_Classifier()->Get_Mode() == CLASSIFY_OVERLAY
 	);
 
@@ -696,7 +718,7 @@ void CVIEW_Histogram::On_Properties(wxCommandEvent &event)
 //---------------------------------------------------------
 void CVIEW_Histogram::On_Cumulative(wxCommandEvent &event)
 {
-	m_bCumulative	= !m_bCumulative;
+	m_bCumulative = !m_bCumulative;
 
 	Refresh();
 }
@@ -704,7 +726,7 @@ void CVIEW_Histogram::On_Cumulative(wxCommandEvent &event)
 //---------------------------------------------------------
 void CVIEW_Histogram::On_Gaussian(wxCommandEvent &event)
 {
-	m_bGaussian	= !m_bGaussian;
+	m_bGaussian = !m_bGaussian;
 
 	Refresh();
 }
@@ -712,10 +734,7 @@ void CVIEW_Histogram::On_Gaussian(wxCommandEvent &event)
 //---------------------------------------------------------
 void CVIEW_Histogram::On_Set_MinMax(wxCommandEvent &event)
 {
-	m_pLayer->Set_Color_Range(
-		m_pLayer->Get_Value_Minimum(),
-		m_pLayer->Get_Value_Maximum()
-	);
+	m_pLayer->Set_Stretch_FullRange();
 }
 
 //---------------------------------------------------------
@@ -784,9 +803,9 @@ void CVIEW_Histogram::On_ToClipboard(wxCommandEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
