@@ -66,9 +66,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -146,7 +146,7 @@ private:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -154,7 +154,7 @@ private:
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -171,7 +171,7 @@ END_EVENT_TABLE()
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -191,7 +191,7 @@ CVIEW_Table_Diagram_Control::~CVIEW_Table_Diagram_Control(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -231,7 +231,7 @@ bool CVIEW_Table_Diagram_Control::Update_Diagram(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -324,7 +324,7 @@ bool CVIEW_Table_Diagram_Control::Set_Zoom(double Zoom, wxPoint Center)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -347,7 +347,7 @@ void CVIEW_Table_Diagram_Control::On_Mouse_RDown(wxMouseEvent &event)
 {
 	wxMenu Menu;
 
-	CMD_Menu_Add_Item(&Menu, false, ID_CMD_DIAGRAM_TO_CLIPBOARD);
+	CMD_Menu_Add_Item(&Menu, false, ID_CMD_VIEW_CLIPBOARD_COPY);
 	Menu.AppendSeparator();
 	CMD_Menu_Add_Item(&Menu, false, ID_CMD_DIAGRAM_SIZE_FIT);
 	CMD_Menu_Add_Item(&Menu, false, ID_CMD_DIAGRAM_SIZE_INC);
@@ -392,7 +392,7 @@ void CVIEW_Table_Diagram_Control::On_Key_Down(wxKeyEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -414,7 +414,7 @@ void CVIEW_Table_Diagram_Control::OnDraw(wxDC &dc)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -445,7 +445,7 @@ void CVIEW_Table_Diagram_Control::SaveToClipboard(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -545,7 +545,7 @@ bool CVIEW_Table_Diagram_Control::_Create(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -644,7 +644,10 @@ bool CVIEW_Table_Diagram_Control::_Initialize(void)
 
 	m_Parameters.Add_Choice("NODE_CHART"  , "TYPE"              , _TL("Chart Type"        ), _TL(""), CSG_String::Format("%s|%s|%s|%s|%s", CHART_TYPES), 4);
 
-//	m_Parameters.Add_Node  ("NODE_CHART"  , "NODE_BARS"         , _TL("Bars"              ), _TL(""));
+	m_Parameters.Add_Node  ("NODE_CHART"  , "NODE_BARS"         , _TL("Bars"              ), _TL(""));
+	m_Parameters.Add_Bool  ("NODE_BARS"   , "BARS_OUTLINE"      , _TL("Outline"           ), _TL(""), false);
+	m_Parameters.Add_Color ("BARS_OUTLINE", "BARS_OUTLINE_COLOR", _TL("Color"             ), _TL(""), SG_COLOR_BLACK);
+	m_Parameters.Add_Int   ("BARS_OUTLINE", "BARS_OUTLINE_SIZE" , _TL("Size"              ), _TL(""), 1, 1, true, 20, true);
 //	m_Parameters.Add_Choice("NODE_BARS"   , "BARS_OFFSET"       , _TL("Offset"            ), _TL(""), CSG_String::Format("%s|%s|%s", _TL("origin"), _TL("bottom"), _TL("top")), 0);
 
 	m_Parameters.Add_Node  ("NODE_CHART"  , "NODE_LINES"        , _TL("Lines"             ), _TL(""));
@@ -676,7 +679,7 @@ bool CVIEW_Table_Diagram_Control::_Initialize(void)
 //---------------------------------------------------------
 int CVIEW_Table_Diagram_Control::_On_Parameter_Changed(CSG_Parameter *pParameter, int Flags)
 {
-	CSG_Parameters	*pParameters	= pParameter ? pParameter->Get_Parameters() : NULL;
+	CSG_Parameters *pParameters = pParameter ? pParameter->Get_Parameters() : NULL;
 
 	if( !pParameters )
 	{
@@ -704,7 +707,7 @@ int CVIEW_Table_Diagram_Control::_On_Parameter_Changed(CSG_Parameter *pParameter
 	{
 		if( pParameter->Cmp_Identifier("TYPE") )
 		{
-		//	pParameters->Set_Enabled("NODE_BARS"    , pParameter->asInt() >= 4 || pParameter->asInt() == 0                            );
+			pParameters->Set_Enabled("NODE_BARS"    , pParameter->asInt() >= 4 || pParameter->asInt() == 0                            );
 			pParameters->Set_Enabled("NODE_LINES"   , pParameter->asInt() >= 4 || pParameter->asInt() == 1 || pParameter->asInt() == 3);
 			pParameters->Set_Enabled("NODE_POINTS"  , pParameter->asInt() >= 4 || pParameter->asInt() == 2 || pParameter->asInt() == 3);
 
@@ -731,6 +734,11 @@ int CVIEW_Table_Diagram_Control::_On_Parameter_Changed(CSG_Parameter *pParameter
 			}
 		}
 
+		if( pParameter->Cmp_Identifier("BARS_OUTLINE") )
+		{
+			pParameter->Set_Children_Enabled(pParameter->asBool());
+		}
+
 		if( pParameter->Cmp_Identifier("POINTS_COLOR_FIELD") )
 		{
 			pParameters->Set_Enabled("POINTS_COLORS", pParameter->asInt() < pParameter->asChoice()->Get_Count() - 1);
@@ -794,7 +802,7 @@ int CVIEW_Table_Diagram_Control::_On_Parameter_Changed(CSG_Parameter *pParameter
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -825,7 +833,7 @@ inline CSG_Table_Record * CVIEW_Table_Diagram_Control::_Get_Record(int iRecord)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1065,7 +1073,7 @@ void CVIEW_Table_Diagram_Control::_Draw_Legend(wxDC &dc, wxRect r)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1189,13 +1197,33 @@ void CVIEW_Table_Diagram_Control::_Draw_Bars(wxDC &dc, wxRect r, double dx, doub
 			}
 		}
 	}
+
+	if( m_Parameters["BARS_OUTLINE"].asBool() && dxb > 2 * m_Parameters["BARS_OUTLINE_SIZE"].asInt() )
+	{
+		dc.SetPen(wxPen(Get_Color_asWX(m_Parameters["BARS_OUTLINE_COLOR"].asColor()), m_Parameters["BARS_OUTLINE_SIZE"].asInt(), wxPENSTYLE_SOLID));
+
+		for(sLong iRecord=0; iRecord<m_pTable->Get_Count(); iRecord++)
+		{
+			CSG_Table_Record *pRecord = _Get_Record(iRecord);
+
+			if( !pRecord->is_NoData(iField) && (m_pTable->Get_Selection_Count() < 1 || pRecord->is_Selected()) )
+			{
+				int xa = DRAW_GET_XPOS + dxa, y = DRAW_GET_YPOS; int xb = xa + dxb;
+
+				dc.DrawLine(xa, yOffset, xa, y);
+				dc.DrawLine(xb, yOffset, xb, y);
+				dc.DrawLine(xa, yOffset, xb, yOffset);
+				dc.DrawLine(xa, y      , xb, y);
+			}
+		}
+	}
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1206,17 +1234,17 @@ BEGIN_EVENT_TABLE(CVIEW_Table_Diagram, CVIEW_Base)
 	EVT_SIZE    (CVIEW_Table_Diagram::On_Size)
 	EVT_KEY_DOWN(CVIEW_Table_Diagram::On_Key_Down)
 
-	EVT_MENU(ID_CMD_DIAGRAM_PARAMETERS  , CVIEW_Table_Diagram::On_Parameters)
-	EVT_MENU(ID_CMD_DIAGRAM_SIZE_FIT    , CVIEW_Table_Diagram::On_Size_Fit)
-	EVT_MENU(ID_CMD_DIAGRAM_SIZE_INC    , CVIEW_Table_Diagram::On_Size_Inc)
-	EVT_MENU(ID_CMD_DIAGRAM_SIZE_DEC    , CVIEW_Table_Diagram::On_Size_Dec)
-	EVT_MENU(ID_CMD_DIAGRAM_LEGEND      , CVIEW_Table_Diagram::On_Legend)
-	EVT_MENU(ID_CMD_DIAGRAM_TO_CLIPBOARD, CVIEW_Table_Diagram::On_SaveToClipboard)
+	EVT_MENU(ID_CMD_DIAGRAM_PARAMETERS , CVIEW_Table_Diagram::On_Parameters)
+	EVT_MENU(ID_CMD_DIAGRAM_SIZE_FIT   , CVIEW_Table_Diagram::On_Size_Fit)
+	EVT_MENU(ID_CMD_DIAGRAM_SIZE_INC   , CVIEW_Table_Diagram::On_Size_Inc)
+	EVT_MENU(ID_CMD_DIAGRAM_SIZE_DEC   , CVIEW_Table_Diagram::On_Size_Dec)
+	EVT_MENU(ID_CMD_DIAGRAM_LEGEND     , CVIEW_Table_Diagram::On_Legend)
+	EVT_MENU(ID_CMD_VIEW_CLIPBOARD_COPY, CVIEW_Table_Diagram::On_SaveToClipboard)
 END_EVENT_TABLE()
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1242,7 +1270,7 @@ CVIEW_Table_Diagram::CVIEW_Table_Diagram(CWKSP_Table *pTable, CSG_Parameters *pP
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1257,7 +1285,7 @@ wxMenu * CVIEW_Table_Diagram::_Create_Menu(void)
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DIAGRAM_SIZE_DEC);
 	CMD_Menu_Add_Item(pMenu,  true, ID_CMD_DIAGRAM_LEGEND);
 	pMenu->AppendSeparator();
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DIAGRAM_TO_CLIPBOARD);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_VIEW_CLIPBOARD_COPY);
 
 	return( pMenu );
 }
@@ -1271,6 +1299,7 @@ wxToolBarBase * CVIEW_Table_Diagram::_Create_ToolBar(void)
 	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_DIAGRAM_SIZE_FIT);
 	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_DIAGRAM_SIZE_INC);
 	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_DIAGRAM_SIZE_DEC);
+	CMD_ToolBar_Add_Item(pToolBar, false, ID_CMD_VIEW_CLIPBOARD_COPY);
 
 	CMD_ToolBar_Add(pToolBar, _TL("Diagram"));
 
@@ -1279,7 +1308,7 @@ wxToolBarBase * CVIEW_Table_Diagram::_Create_ToolBar(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1296,7 +1325,7 @@ void CVIEW_Table_Diagram::Do_Update(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1313,7 +1342,7 @@ void CVIEW_Table_Diagram::On_Key_Down(wxKeyEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1371,9 +1400,9 @@ void CVIEW_Table_Diagram::On_Command_UI(wxUpdateUIEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
