@@ -53,6 +53,8 @@
 #endif
 
 #include <wx/filename.h>
+#include <wx/clipbrd.h>
+#include <wx/fs_mem.h>
 
 #include <saga_api/saga_api.h>
 
@@ -82,6 +84,7 @@
 
 #include "wksp_map_manager.h"
 
+#include "wksp_tool_manager.h"
 #include "wksp_tool.h"
 
 #include "active.h"
@@ -95,7 +98,7 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CWKSP_Data_Manager	*g_pData	= NULL;
+CWKSP_Data_Manager *g_pData = NULL;
 
 
 ///////////////////////////////////////////////////////////
@@ -107,25 +110,38 @@ CWKSP_Data_Manager	*g_pData	= NULL;
 //---------------------------------------------------------
 CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 {
-	g_pData			= this;
-
-	m_pTables		= NULL;
-	m_pShapes		= NULL;
-	m_pTINs			= NULL;
-	m_pPointClouds	= NULL;
-	m_pGrids		= NULL;
-
-	m_pProject		= new CWKSP_Project;
-	m_pMenu_Files	= new CWKSP_Data_Menu_Files;
+	wxMemoryFSHandler::AddFile("preview_data.png", "dummy");
 
 	//-----------------------------------------------------
-	m_Parameters.Add_Bool (""          , "THUMBNAILS"        , _TL("Thumbnails"     ), _TL(""), true);
-	m_Parameters.Add_Int  ("THUMBNAILS", "THUMBNAIL_SIZE"    , _TL("Size"           ), _TL(""), 50, 10, true);
-	m_Parameters.Add_Color("THUMBNAILS", "THUMBNAIL_SELCOLOR", _TL("Selection Color"), _TL(""), Get_Color_asInt(SYS_Get_Color(wxSYS_COLOUR_BTNSHADOW)));
-	m_Parameters.Add_Bool ("THUMBNAILS", "THUMBNAIL_CATEGORY", _TL("Show Categories"), _TL(""), true);
+	g_pData        = this;
+
+	m_pTables      = NULL;
+	m_pShapes      = NULL;
+	m_pTINs        = NULL;
+	m_pPointClouds = NULL;
+	m_pGrids       = NULL;
+
+	m_pProject     = new CWKSP_Project;
+	m_pMenu_Files  = new CWKSP_Data_Menu_Files;
+
+	//-----------------------------------------------------
+	m_Parameters.Add_Bool (""          , "THUMBNAILS"        , _TL("Thumbnails"         ), _TL(""), true);
+	m_Parameters.Add_Int  ("THUMBNAILS", "THUMBNAIL_SIZE"    , _TL("Size"               ), _TL(""), 50, 10, true);
+	m_Parameters.Add_Color("THUMBNAILS", "THUMBNAIL_SELCOLOR", _TL("Selection Color"    ), _TL(""), Get_Color_asInt(SYS_Get_Color(wxSYS_COLOUR_BTNSHADOW)));
+	m_Parameters.Add_Bool ("THUMBNAILS", "THUMBNAIL_CATEGORY", _TL("Show Categories"    ), _TL(""), true);
+
+	m_Parameters.Add_Node (""          , "NODE_DESC"         , _TL("Description"        ), _TL(""));
+	m_Parameters.Add_Bool ("NODE_DESC" , "PREVIEW"           , _TL("Preview"            ), _TL("Show a preview in description."), false);
+	m_Parameters.Add_Int  ("PREVIEW"   , "PREVIEW_SIZE"      , _TL("Size"               ), _TL(""), 200, 10, true);
 
 	m_Parameters.Add_Bool("",
-		"SHOW_FILE_SOURCES"		, _TL("Show Data File Source Browser"),
+		"SHOW_TOOLBAR"          , _TL("Show Toolbar"),
+		_TL("Show toolbar with data layer related commands."),
+		true
+	);
+
+	m_Parameters.Add_Bool("",
+		"SHOW_FILE_SOURCES"     , _TL("Show Data File Source Browser"),
 		_TL("Show data sources tab for file system. Disabling might speed up start-up. Changes take effect after restart."),
 		true
 	);
@@ -134,19 +150,19 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	m_Parameters.Add_Node("", "NODE_PROJECT", _TL("Projects"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_PROJECT",
-		"PROJECT_START"			, _TL("Startup Project"),
+		"PROJECT_START"         , _TL("Startup Project"),
 		_TL(""),
 		CSG_String::Format("%s|%s|%s",
 			_TL("empty"),
 			_TL("last state"),
-			_TL("always ask what to do")
+			_TL("ask what to do")
 		), 2
 	);
 
 	m_Parameters.Add_Choice("NODE_PROJECT",
-		"PROJECT_MAP_ARRANGE", _TL("Map Window Arrangement"),
+		"PROJECT_MAP_ARRANGE"   , _TL("Map Window Arrangement"),
 		_TL("Initial map window arrangement after a project has been loaded."),
-		#ifdef _SAGA_MSW
+		#ifndef MDI_TABBED
 		CSG_String::Format("%s|%s|%s|%s|%s",
 			_TL("Cascade"),
 			_TL("Tile Horizontally"),
@@ -166,7 +182,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	m_Parameters.Add_Node("", "NODE_DB", _TL("Databases"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_DB",
-		"PROJECT_DB_REOPEN"		, _TL("Reopen Database Connections"),
+		"PROJECT_DB_REOPEN"     , _TL("Reopen Database Connections"),
 		_TL("Reopen PostgreSQL database connections. Warning: if set to true account information including unencrypted passwords for automatic connection will be stored."),
 		CSG_String::Format("%s|%s",
 			_TL("no"),
@@ -175,7 +191,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	);
 
 	m_Parameters.Add_Int("PROJECT_DB_REOPEN",
-		"PROJECT_DB_WAIT"		, _TL("Response Time"),
+		"PROJECT_DB_WAIT"       , _TL("Response Time"),
 		_TL("Maximum time (seconds) to wait for server response. If zero it waits until the PostgreSQL connection is established or refused."),
 		2, 0, true
 	);
@@ -184,53 +200,65 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	m_Parameters.Add_Node("", "NODE_DATA", _TL("Data"), _TL(""));
 
 	m_Parameters.Add_Int("NODE_DATA",
-		"NUMBERING"				, _TL("Data Set Numeration"),
+		"NUMBERING"             , _TL("Data Set Numeration"),
 		_TL("Minimum width of data set numbering. If set to 0 no numbering is applied at all. Negative values will prepend zeros. Numbering helps in data set sorting and selection."),
 		3
 	);
 
 	m_Parameters.Add_Bool("NODE_DATA",
-		"NAME_BY_FILE"			, _TL("Use File Name for Naming"),
+		"NAME_BY_FILE"          , _TL("Use File Name for Naming"),
 		_TL("Use file name to name a data set, when it has been loaded from file."),
 		false
 	);
 
 	m_Parameters.Add_Bool("NODE_DATA",
-		"COLORS_FROM_TOOL"		, _TL("Tool Set Colors"),
+		"COLORS_FROM_TOOL"      , _TL("Tool Set Colors"),
 		_TL("Allow tools to change data set colors programmatically."),
 		true
 	);
 
 	m_Parameters.Add_Colors("NODE_DATA",
-		"COLORS_DEFAULT"		, _TL("Default Colors"),
+		"COLORS_DEFAULT"        , _TL("Default Color Ramp"),
 		_TL("")
 	);
 
+	m_Parameters.Add_FilePath("NODE_DATA",
+		"COLORS_FOLDER"         , _TL("Additional Color Ramps"),
+		_TL(""),
+		#if defined(_SAGA_MSW)
+		NULL, SG_UI_Get_Application_Path(true) + "\\resources\\color_ramps", false, true
+		#elif defined(SHARE_PATH)
+		NULL, SG_File_Make_Path(SHARE_PATH, "resources/color_ramps"), false, true
+		#endif
+	);
+
 	m_Parameters.Add_Choice("NODE_DATA"      , "STRETCH_DEFAULT", _TL("Histogram Stretch Defaults"), _TL("Histogram stretch applied by default to new data."), CSG_String::Format("%s|%s|%s",
 		_TL("Linear"), _TL("Standard Deviation"), _TL("Percent Clip")), 1
-	);
+	)->Set_Collapsed();
 
-	m_Parameters.Add_Range ("STRETCH_DEFAULT", "STRETCH_LINEAR" , _TL("Linear"                    ), _TL("Linear percent stretch allows you to trim extreme values from both ends of the histogram using the percentage specified here."), 5., 95., 0., true, 100., true);
+	m_Parameters.Add_Range ("STRETCH_DEFAULT", "STRETCH_LINEAR" , _TL("Linear"                    ), _TL("Linear percent stretch allows you to trim extreme values from both ends of the histogram using the percentage specified here."), 0., 100., 0., true, 100., true);
 	m_Parameters.Add_Double("STRETCH_DEFAULT", "STRETCH_STDDEV" , _TL("Standard Deviation"        ), _TL(""), 2., 0., true);
-	m_Parameters.Add_Range ("STRETCH_DEFAULT", "STRETCH_PCTL"   , _TL("Percent Clip"              ), _TL(""), 2., 98., 0., true, 100., true);
+	m_Parameters.Add_Bool  ("STRETCH_STDDEV" , "STRETCH_INRANGE", _TL("Keep in Range"             ), _TL(""), true);
+	m_Parameters.Add_Range ("STRETCH_DEFAULT", "STRETCH_PCTL"   , _TL("Percent Clip"              ), _TL(""), 1., 99., 0., true, 100., true);
+	m_Parameters.Add_Choice("STRETCH_DEFAULT", "STRETCH_NORANGE", _TL("Invalid Range"             ), _TL(""), CSG_String::Format("%s|%s|%s", _TL("ignore"), _TL("ask what to do"), _TL("adjust to full range")), 2);
 
-	SG_DataObject_Set_Max_Samples(1000000);	// default to one million
+	SG_DataObject_Set_Max_Samples(1e06); // defaults to 1 million
 
 	m_Parameters.Add_Int("NODE_DATA",
-		"DATA_SAMPLE_MAX"		, _TL("Maximum Samples"),
+		"DATA_SAMPLE_MAX"       , _TL("Maximum Samples"),
 		_TL("Default maximum number of samples used to build statistics and histograms. Set to zero to use all data records."),
 		(int)SG_DataObject_Get_Max_Samples(), 0, true
 	);
 
 	//-----------------------------------------------------
 	m_Parameters.Add_Int("NODE_DATA",
-		"HISTORY_DEPTH"			, _TL("History Depth"),
+		"HISTORY_DEPTH"         , _TL("History Depth"),
 		_TL("Depth to which data history is stored. Set -1 keeps all history entries (default), 0 switches history option off."),
 		SG_Get_History_Depth(), -1, true
 	);
 
 	m_Parameters.Add_Bool("HISTORY_DEPTH",
-		"HISTORY_LISTS"			, _TL("Ignore Input Lists"),
+		"HISTORY_LISTS"         , _TL("Ignore Input Lists"),
 		_TL(""),
 		SG_Get_History_Ignore_Lists() != 0
 	);
@@ -239,7 +267,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	m_Parameters.Add_Node("", "NODE_GRID", _TL("Grids"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_GRID",
-		"GRID_FMT_DEFAULT"		, _TL("Default Output Format"),
+		"GRID_FMT_DEFAULT"      , _TL("Default Output Format"),
 		_TL(""),
 		CSG_String::Format("%s (*.sg-grd-z)|%s (*.sg-grd)|%s (*.tif)",
 			_TL("SAGA Compressed Grid Files"),
@@ -249,19 +277,19 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	);
 
 	m_Parameters.Add_Int("NODE_GRID",
-		"GRID_COORD_PRECISION"	, _TL("Coordinate Precision"),
-		_TL("Precision used to store coordinates and cell sizes (i.e. number of decimals)."),
-		10, 0, true
+		"GRID_COORD_PRECISION"  , _TL("Coordinate Precision"),
+		_TL("Precision used to store coordinates and cell sizes (i.e. number of decimals). Ignored if negative."),
+		10, -1, true
 	);
 
 	m_Parameters.Add_Int("NODE_GRID",
-		"GRID_SELECT_MAX"		, _TL("Maximum Selection"),
+		"GRID_SELECT_MAX"       , _TL("Maximum Selection"),
 		_TL("Maximum number of rows/columns in selection of grid cells."),
 		100, 1, true
 	);
 
 	m_Parameters.Add_Choice("NODE_GRID",
-		"GRID_CACHE_MODE"		, _TL("File Cache"),
+		"GRID_CACHE_MODE"       , _TL("File Cache"),
 		_TL("Activate file caching automatically, if memory size exceeds the threshold value."),
 		CSG_String::Format("%s|%s|%s",
 			_TL("no"),
@@ -271,13 +299,13 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	);
 
 	m_Parameters.Add_Double("GRID_CACHE_MODE",
-		"GRID_CACHE_THRSHLD"	, _TL("Threshold for automatic mode [MB]"),
+		"GRID_CACHE_THRSHLD"    , _TL("Threshold for automatic mode [MB]"),
 		_TL(""),
 		SG_Grid_Cache_Get_Threshold_MB(), 0., true
 	);
 
 	m_Parameters.Add_FilePath("GRID_CACHE_MODE",
-		"GRID_CACHE_TMPDIR"		, _TL("Temporary files"),
+		"GRID_CACHE_TMPDIR"     , _TL("Temporary files"),
 		_TL("Directory, where temporary cache files shall be saved."),
 		NULL, SG_Grid_Cache_Get_Directory(), true, true
 	);
@@ -286,7 +314,7 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	m_Parameters.Add_Node("", "NODE_TABLE", _TL("Tables"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_TABLE",
-		"TABLE_FLT_STYLE"		, _TL("Floating Point Numbers"),
+		"TABLE_FLT_STYLE"       , _TL("Floating Point Numbers"),
 		_TL(""),
 		CSG_String::Format("%s|%s|%s",
 			_TL("system default"),
@@ -296,16 +324,30 @@ CWKSP_Data_Manager::CWKSP_Data_Manager(void)
 	);
 
 	m_Parameters.Add_Int("NODE_TABLE",
-		"TABLE_FLT_DECIMALS"	, _TL("Decimals"),
+		"TABLE_FLT_DECIMALS"    , _TL("Decimals"),
 		_TL(""),
 		6, 0, true
 	);
 
+	m_Parameters.Add_Choice("NODE_TABLE",
+		"TABLE_ENCODING"        , _TL("Default Encoding"),
+		_TL("Default encoding used when loading a table."),
+		CSG_String::Format("%s|%s|%s|%s|%s|%s|%s",
+			SG_T("ANSI"),
+			SG_T("UTF-7"),
+			SG_T("UTF-8"),
+			SG_T("UTF-16 (Little Endian)"),
+			SG_T("UTF-16 (Big Endian)"),
+			SG_T("UTF-32 (Little Endian)"),
+			SG_T("UTF-32 (Big Endian)")
+		), 2
+	);
+
 	//-----------------------------------------------------
 	m_Parameters.Add_Node("", "NODE_SHAPES", _TL("Shapes"), _TL(""));
 
 	m_Parameters.Add_Choice("NODE_SHAPES",
-		"SHAPES_FMT_DEFAULT"	, _TL("Default Output Format"),
+		"SHAPES_FMT_DEFAULT"    , _TL("Default Output Format"),
 		_TL(""),
 		CSG_String::Format("%s|%s|%s",
 			_TL("ESRI Shape File (*.shp)"),
@@ -363,16 +405,16 @@ CWKSP_Data_Manager::~CWKSP_Data_Manager(void)
 wxFileName Get_SAGA_GUI_CFG(void)
 {
 #ifdef _SAGA_LINUX
-//	wxFileName	fProject(wxString(getenv( "HOME"), wxConvFile ), wxT("saga_gui"), wxT("cfg"));
+//	wxFileName	fProject(wxString(getenv( "HOME"), wxConvFile ), wxT("saga_gui"), wxT("sg-project"));
 	CSG_String	sHome(getenv("HOME"));
-	wxFileName	fProject(sHome.c_str(), "saga_gui", "cfg");
+	wxFileName	fProject(sHome.c_str(), "saga_gui", "sg-project");
 #else
-	wxFileName	fProject(g_pSAGA->Get_App_Path(), "saga_gui", "cfg");
+	wxFileName	fProject(g_pSAGA->Get_App_Path(), "saga_gui", "sg-project");
 
 	if(	( fProject.FileExists() && (!fProject.IsFileReadable() || !fProject.IsFileWritable()))
 	||	(!fProject.FileExists() && (!fProject.IsDirReadable () || !fProject.IsDirWritable ())) )
 	{
-		fProject.Assign(wxGetHomeDir(), "saga_gui", "cfg");
+		fProject.Assign(wxGetHomeDir(), "saga_gui", "sg-project");
 	}
 #endif
 
@@ -385,63 +427,66 @@ wxFileName Get_SAGA_GUI_CFG(void)
 bool CWKSP_Data_Manager::Initialise(void)
 {
 	//-----------------------------------------------------
-	if( m_pProject->Has_File_Name() )	{	return( m_pProject->Load(false) );	}
+	if( m_pProject->Has_File_Name() ) { return( m_pProject->Load(false) ); }
 	//-----------------------------------------------------
 
 	//-----------------------------------------------------
-	wxFileName	fProject, fLastState	= Get_SAGA_GUI_CFG();
+	wxFileName fProject, fLastState = Get_SAGA_GUI_CFG();
 
 	switch( m_Parameters("PROJECT_START")->asInt() )
 	{
-	case 0:	// empty
-		return( true );
-
 	case 1:	// last state
-		return( m_pProject->Load(fLastState.GetFullPath(), false, false) );
+		fProject = fLastState;
+		break;
 
-	case 2:	// always ask what to do
+	case 2:	// ask what to do
 		{
-			wxArrayString	Projects;
+			wxArrayString Projects;
 
 			Projects.Add(wxString::Format("[%s]", _TL("empty")));
 
-			if( fLastState.FileExists() )
+			if( fLastState.FileExists() && g_pTools->Get_Parameter("SAVE_CONFIG")->asBool() )
 			{
 				Projects.Add(wxString::Format("[%s]", _TL("last state")));
 			}
 
-			m_pMenu_Files->Recent_Get(SG_DATAOBJECT_TYPE_Undefined, Projects, true);
+			m_pMenu_Files->Recent_Get(CWKSP_Data_Menu_File::Recent_Type::Project, Projects, true);
 
-			wxSingleChoiceDialog	dlg(MDI_Get_Top_Window(), _TL("Startup Project"), _TL("Select Startup Project"), Projects);
+			if( Projects.Count() > 1 )
+			{
+				wxSingleChoiceDialog dlg(MDI_Get_Top_Window(), _TL("Startup Project"), _TL("Select Startup Project"), Projects);
 
-			if( Projects.Count() <= 1 || dlg.ShowModal() != wxID_OK || dlg.GetSelection() == 0 )
-			{	// empty
-				return( true );
-			}
-
-			if( fLastState.FileExists() && dlg.GetSelection() == 1 )
-			{	// last state
-				return( m_pProject->Load(fLastState.GetFullPath(), false, false) );
-			}
-			else
-			{	// recently opened project
-				return( m_pProject->Load(dlg.GetStringSelection(), false, true) );
+				if( dlg.ShowModal() == wxID_OK && dlg.GetSelection() > 0 )
+				{
+					if( fLastState.FileExists() && dlg.GetSelection() == 1 )
+					{	// last state
+						fProject = fLastState;
+					}
+					else
+					{	// recently opened project
+						fProject = dlg.GetStringSelection();
+					}
+				}
 			}
 		}
+		break;
 	}
 
 	//-----------------------------------------------------
-	//wxString	FileName	= Get_FilePath_Absolute(g_pSAGA->Get_App_Path(), fProject.GetFullPath());
+	if( fProject.FileExists() )
+	{
+		return( m_pProject->Load(fProject.GetFullPath(), false, fProject != fLastState) );
+	}
 
-	return( false );
+	return( true );	// empty
 }
 
 //---------------------------------------------------------
 bool CWKSP_Data_Manager::Finalise(void)
 {
-	wxFileName	fProject	= Get_SAGA_GUI_CFG();
+	wxFileName fProject = Get_SAGA_GUI_CFG();
 
-	if( Get_Count() > 0 )
+	if( Get_Count() > 0 && g_pTools->Get_Parameter("SAVE_CONFIG")->asBool() )
 	{	// last state
 		m_pProject->Save(fProject.GetFullPath(), false);
 
@@ -459,6 +504,35 @@ bool CWKSP_Data_Manager::Finalise(void)
 }
 
 
+///////////////////////////////////////////////////////////
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+wxString CWKSP_Data_Manager::Set_Description_Image(CWKSP_Layer *pLayer)
+{
+	wxString s;
+
+	if( m_Parameters["PREVIEW"].asBool() && pLayer->Get_Extent().Get_Area() > 0. )
+	{
+		int nx, ny; double x2y = pLayer->Get_Extent().Get_XRange() / pLayer->Get_Extent().Get_YRange();
+
+		if( x2y > 1. ) { nx = m_Parameters["PREVIEW_SIZE"].asInt(); ny = (int)(nx / x2y); }
+		else           { ny = m_Parameters["PREVIEW_SIZE"].asInt(); nx = (int)(ny * x2y); }
+
+		wxMemoryFSHandler::RemoveFile("preview_data.png");
+
+		wxMemoryFSHandler::AddFile("preview_data.png", pLayer->Get_Thumbnail(nx, ny), wxBITMAP_TYPE_PNG);
+	
+		s = "<img src=\"memory:preview_data.png\">";
+
+		s = "<table border=\"1\"><th><tr>" + s + "</tr></th></table><hr>";
+	}
+
+	return( s );
+}
+
+
 ///////////////////////////////////////////////////////////
 //                                                       //
 ///////////////////////////////////////////////////////////
@@ -501,9 +575,10 @@ wxString CWKSP_Data_Manager::Get_Description(void)
 //---------------------------------------------------------
 wxMenu * CWKSP_Data_Manager::Get_Menu(void)
 {
-	if( m_Sel_Items.Count() > 0 )
+//	if( m_Sel_Items.Count() > 1 )
+	if( Get_Control()->Get_Selection_Count() > 1 )
 	{
-		wxMenu	*pMenu	= new wxMenu;
+		wxMenu *pMenu = new wxMenu;
 
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_SHOW);
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_CLOSE);
@@ -516,7 +591,7 @@ wxMenu * CWKSP_Data_Manager::Get_Menu(void)
 	}
 
 	//-----------------------------------------------------
-	wxMenu	*pMenu	= new wxMenu(_TL("Data"));
+	wxMenu *pMenu = new wxMenu(_TL("Data"));
 
 	if( wxGetKeyState(WXK_CONTROL) ) // add advanced/hidden commands
 	{
@@ -543,6 +618,26 @@ wxMenu * CWKSP_Data_Manager::Get_Menu(void)
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_SEARCH);
 	}
 
+	//-----------------------------------------------------
+	if( wxTheClipboard->Open() )
+	{
+		bool bSeparator = false;
+
+		if( wxTheClipboard->IsSupported(wxDF_TEXT) )
+		{
+			if( !bSeparator ) { bSeparator = true; pMenu->AppendSeparator(); }
+			CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_CLIPBOARD_PASTE_TABLE);
+		}
+
+		if( wxTheClipboard->IsSupported(wxDF_BITMAP) )
+		{
+			if( !bSeparator ) { bSeparator = true; pMenu->AppendSeparator(); }
+			CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_CLIPBOARD_PASTE_IMAGE);
+		}
+
+		wxTheClipboard->Close();
+	}
+
 	//-----------------------------------------------------
 	return( pMenu );
 }
@@ -560,41 +655,10 @@ bool CWKSP_Data_Manager::On_Command(int Cmd_ID)
 		return( true );
 	}
 
-	if( m_pTables && Cmd_ID >= ID_CMD_TABLE_FIRST  && Cmd_ID <= ID_CMD_TABLE_LAST  && m_pTables->On_Command(Cmd_ID) )
-	{
-		return( true );
-	}
-
-	if( m_pShapes && Cmd_ID >= ID_CMD_SHAPES_FIRST && Cmd_ID <= ID_CMD_SHAPES_LAST && m_pShapes->On_Command(Cmd_ID) )
-	{
-		return( true );
-	}
-
-	if( m_pTINs   && Cmd_ID >= ID_CMD_TIN_FIRST    && Cmd_ID <= ID_CMD_TIN_LAST    && m_pTINs  ->On_Command(Cmd_ID) )
-	{
-		return( true );
-	}
-
-	if( m_pPointClouds && Cmd_ID >= ID_CMD_POINTCLOUD_FIRST && Cmd_ID <= ID_CMD_POINTCLOUD_LAST && m_pPointClouds->On_Command(Cmd_ID) )
-	{
-		return( true );
-	}
-
-	if( m_pGrids  && Cmd_ID >= ID_CMD_GRID_FIRST   && Cmd_ID <= ID_CMD_GRID_LAST   && m_pGrids ->On_Command(Cmd_ID) )
-	{
-		return( true );
-	}
-
-	if( m_pGrids  && Cmd_ID >= ID_CMD_GRIDS_FIRST  && Cmd_ID <= ID_CMD_GRIDS_LAST  && m_pGrids ->On_Command(Cmd_ID) )
-	{
-		return( true );
-	}
-
 	//-----------------------------------------------------
 	switch( Cmd_ID )
 	{
-	default:
-		return( CWKSP_Base_Manager::On_Command(Cmd_ID) );
+	default: return( CWKSP_Base_Manager::On_Command(Cmd_ID) );
 
 	//-----------------------------------------------------
 	case ID_CMD_WKSP_ITEM_RETURN     :                                      break;
@@ -618,6 +682,10 @@ bool CWKSP_Data_Manager::On_Command(int Cmd_ID)
 	case ID_CMD_GRID_OPEN            : Open(SG_DATAOBJECT_TYPE_Grid      ); break;
 	case ID_CMD_GRIDS_OPEN           : Open(SG_DATAOBJECT_TYPE_Grids     ); break;
 
+	//-----------------------------------------------------
+	case ID_CMD_DATA_CLIPBOARD_PASTE_TABLE: Clipboard_Paste_Table(); break;
+	case ID_CMD_DATA_CLIPBOARD_PASTE_IMAGE: Clipboard_Paste_Image(); break;
+
 	//-----------------------------------------------------
 	case ID_CMD_DATA_MANAGER_LIST    : {
 		CSG_String s(SG_Get_Data_Manager().Get_Summary());
@@ -670,19 +738,24 @@ bool CWKSP_Data_Manager::On_Command_UI(wxUpdateUIEvent &event)
 {
 	switch( event.GetId() )
 	{
-	default:
-		return( CWKSP_Base_Manager::On_Command_UI(event) );
+	default: return( CWKSP_Base_Manager::On_Command_UI(event) );
 
-	case ID_CMD_WKSP_ITEM_CLOSE:
-		event.Enable(Get_Count() > 0 && g_pTool == NULL);
+	case ID_CMD_WKSP_ITEM_CLOSE     : event.Enable(Get_Count() > 0 && g_pTool == NULL); break;
+
+	case ID_CMD_DATA_PROJECT_SAVE   : event.Enable(Get_Count() > 0 && m_pProject->Has_File_Name() ); break;
+	case ID_CMD_DATA_PROJECT_SAVE_AS: event.Enable(Get_Count() > 0); break;
+	case ID_CMD_DATA_PROJECT_COPY   : event.Enable(g_pData->Get_Count() > 0); break;
+	case ID_CMD_DATA_PROJECT_COPY_DB: event.Enable(g_pData->Get_Count() > 0); break;
+
+	case ID_CMD_DATA_FILE_RECENT    : event.Enable(m_pMenu_Files->Recent_Count(CWKSP_Data_Menu_File::Recent_Type::Data  ) > 0); break;
+	case ID_CMD_DATA_FOLDER_RECENT  : event.Enable(m_pMenu_Files->Recent_Count(CWKSP_Data_Menu_File::Recent_Type::Folder) > 0); break;
+
+	case ID_CMD_DATA_CLIPBOARD_PASTE_TABLE:
+		if( wxTheClipboard->Open() ) { event.Enable(wxTheClipboard->IsSupported(wxDF_TEXT  )); wxTheClipboard->Close(); } else { event.Enable(false); }
 		break;
 
-	case ID_CMD_DATA_PROJECT_SAVE:
-		event.Enable(Get_Count() > 0 && m_pProject->Has_File_Name() );
-		break;
-
-	case ID_CMD_DATA_PROJECT_SAVE_AS:
-		event.Enable(Get_Count() > 0);
+	case ID_CMD_DATA_CLIPBOARD_PASTE_IMAGE:
+		if( wxTheClipboard->Open() ) { event.Enable(wxTheClipboard->IsSupported(wxDF_BITMAP)); wxTheClipboard->Close(); } else { event.Enable(false); }
 		break;
 	}
 
@@ -779,6 +852,11 @@ int CWKSP_Data_Manager::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Pa
 			pParameter->Set_Children_Enabled(pParameter->asBool());
 		}
 
+		if(	pParameter->Cmp_Identifier("PREVIEW") )
+		{
+			pParameter->Set_Children_Enabled(pParameter->asBool());
+		}
+
 		if(	pParameter->Cmp_Identifier("GRID_CACHE_MODE") )
 		{
 			pParameters->Set_Enabled("GRID_CACHE_THRSHLD", pParameter->asInt() != 0);
@@ -805,11 +883,26 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
 {
 	CSG_Data_Object *pObject = NULL;
 
-//	SG_Get_Data_Manager().Add(File, DataType);
-
 	switch( DataType )
 	{
-	case SG_DATAOBJECT_TYPE_Table     : pObject = SG_Create_Table     (&File); break;
+	case SG_DATAOBJECT_TYPE_Table     : {
+		int Encoding;
+
+		switch( m_Parameters("TABLE_ENCODING")->asInt() )
+		{
+		case  0: Encoding = SG_FILE_ENCODING_ANSI     ; break;
+		case  1: Encoding = SG_FILE_ENCODING_UTF7     ; break;
+		case  2: Encoding = SG_FILE_ENCODING_UTF8     ; break;
+		case  3: Encoding = SG_FILE_ENCODING_UTF16LE  ; break;
+		case  4: Encoding = SG_FILE_ENCODING_UTF16BE  ; break;
+		case  5: Encoding = SG_FILE_ENCODING_UTF32LE  ; break;
+		case  6: Encoding = SG_FILE_ENCODING_UTF32BE  ; break;
+		default: Encoding = SG_FILE_ENCODING_UNDEFINED; break;
+		}
+
+		pObject = SG_Create_Table(&File, TABLE_FILETYPE_Undefined, Encoding);
+		break; }
+
 	case SG_DATAOBJECT_TYPE_Shapes    : pObject = SG_Create_Shapes    (&File); break;
 	case SG_DATAOBJECT_TYPE_TIN       : pObject = SG_Create_TIN       (&File); break;
 	case SG_DATAOBJECT_TYPE_PointCloud: pObject = SG_Create_PointCloud(&File); break;
@@ -819,11 +912,11 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
 
 	if( pObject )
 	{
-		CWKSP_Data_Item *pItem = Add(pObject);
+		CWKSP_Data_Item *pItem = pObject->is_Valid() ? Add(pObject) : NULL;
 
 		if( pItem )
 		{
-			m_pMenu_Files->Recent_Add(pObject->Get_ObjectType(), File);
+			m_pMenu_Files->Recent_Add(CWKSP_Data_Menu_File::Recent_Type::Data, File);
 
 			SG_Get_Data_Manager().Add(pObject);
 
@@ -833,7 +926,7 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Open(const wxString &File, int DataType)
 		delete(pObject);
 	}
 
-	m_pMenu_Files->Recent_Del(DataType, File);
+	m_pMenu_Files->Recent_Del(CWKSP_Data_Menu_File::Recent_Type::Data, File);
 
 	return( NULL );
 }
@@ -863,7 +956,8 @@ bool CWKSP_Data_Manager::Open(const wxString &File)
 	}
 
 	//-----------------------------------------------------
-	if( SG_File_Cmp_Extension(&File, "sprj") )
+	if( SG_File_Cmp_Extension(&File, "sg-project")
+	||  SG_File_Cmp_Extension(&File, "sprj"      ) )
 	{
 		return( m_pProject->Load(File, false, true) );
 	}
@@ -902,7 +996,14 @@ bool CWKSP_Data_Manager::Open(const wxString &File)
 		return( Open(File, SG_DATAOBJECT_TYPE_Grids     ) != NULL );
 	}
 
-	return( SG_Get_Data_Manager().Add(&File) != NULL );
+	if( SG_Get_Data_Manager().Add(&File) != NULL )
+	{
+		m_pMenu_Files->Recent_Add(CWKSP_Data_Menu_File::Recent_Type::Data, File);
+
+		return( true );
+	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -946,6 +1047,26 @@ bool CWKSP_Data_Manager::Open_CMD(int Cmd_ID)
 	return( m_pMenu_Files->Recent_Open(Cmd_ID) );
 }
 
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Open_Directory(const wxString &Directory)
+{
+	wxArrayString Files;
+
+	if( DLG_Open(Files, ID_DLG_FILE_OPEN, Directory) )
+	{
+		MSG_General_Add_Line();
+
+		for(size_t i=0; i<Files.GetCount(); i++)
+		{
+			Open(Files[i]);
+		}
+
+		return( true );
+	}
+
+	return( false );
+}
+
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -1275,11 +1396,11 @@ bool CWKSP_Data_Manager::Close(bool bSilent)
 		{
 			if( bSilent )
 			{
-				MSG_General_Add(_TL("Stop running tool or wait until its exection has been finished!"), true, false);
+				MSG_General_Add(_TL("Stop running tool or wait until its execution has been finished!"), true, false);
 			}
 			else
 			{
-				DLG_Message_Show(_TL("Stop running tool or wait until its exection has been finished!"), g_pTool->Get_Name());
+				DLG_Message_Show(_TL("Stop running tool or wait until its execution has been finished!"), g_pTool->Get_Name());
 			}
 
 			return( false );
@@ -1328,6 +1449,46 @@ CWKSP_Base_Manager * CWKSP_Data_Manager::Get_Manager(TSG_Data_Object_Type Type,
 	return( NULL );
 }
 
+//---------------------------------------------------------
+void CWKSP_Data_Manager::Del_Manager(CWKSP_Base_Item *pItem)
+{
+	if( pItem == m_pTables      ) { m_pTables      = NULL; }
+	if( pItem == m_pTINs        ) { m_pTINs        = NULL; }
+	if( pItem == m_pPointClouds ) { m_pPointClouds = NULL; }
+	if( pItem == m_pShapes      ) { m_pShapes      = NULL; }
+	if( pItem == m_pGrids       ) { m_pGrids       = NULL; }
+
+	if( Get_Count() == 0 )
+	{
+		m_pProject->Clr_File_Name();
+	}
+}
+
+
+///////////////////////////////////////////////////////////
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CWKSP_Data_Item * CWKSP_Data_Manager::Add(CSG_Data_Object *pObject)
+{
+	if( SG_Get_Data_Manager().Add(pObject) && Get_Manager(pObject->Get_ObjectType(), true) )
+	{
+		switch( pObject->Get_ObjectType() )
+		{
+		case SG_DATAOBJECT_TYPE_Table     : return( (CWKSP_Data_Item *)m_pTables     ->Add_Data((CSG_Table      *)pObject) );
+		case SG_DATAOBJECT_TYPE_TIN       : return( (CWKSP_Data_Item *)m_pTINs       ->Add_Data((CSG_TIN        *)pObject) );
+		case SG_DATAOBJECT_TYPE_PointCloud: return( (CWKSP_Data_Item *)m_pPointClouds->Add_Data((CSG_PointCloud *)pObject) );
+		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Data_Item *)m_pShapes     ->Add_Data((CSG_Shapes     *)pObject) );
+		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grid       *)pObject) );
+		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grids      *)pObject) );
+		default: break;
+		}
+	}
+
+	return( NULL );
+}
+
 //---------------------------------------------------------
 CWKSP_Data_Item * CWKSP_Data_Manager::Get(CSG_Data_Object *pObject)
 {
@@ -1348,6 +1509,25 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Get(CSG_Data_Object *pObject)
 	return( NULL );
 }
 
+//---------------------------------------------------------
+CWKSP_Layer * CWKSP_Data_Manager::Get_Layer(CSG_Data_Object *pObject)
+{
+	if( pObject && pObject != DATAOBJECT_CREATE && Get_Manager(pObject->Get_ObjectType()) )
+	{
+		switch( pObject->Get_ObjectType() )
+		{
+		case SG_DATAOBJECT_TYPE_TIN       : return( (CWKSP_Layer *)m_pTINs       ->Get_Data((CSG_TIN        *)pObject) );
+		case SG_DATAOBJECT_TYPE_PointCloud: return( (CWKSP_Layer *)m_pPointClouds->Get_Data((CSG_PointCloud *)pObject) );
+		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Layer *)m_pShapes     ->Get_Data((CSG_Shapes     *)pObject) );
+		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grid       *)pObject) );
+		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grids      *)pObject) );
+		default: break;
+		}
+	}
+
+	return( NULL );
+}
+
 //---------------------------------------------------------
 CWKSP_Data_Item * CWKSP_Data_Manager::Get_byFile(const wxString &File)
 {
@@ -1404,57 +1584,109 @@ CWKSP_Data_Item * CWKSP_Data_Manager::Get_byID_or_File(const SG_Char *ID, const
 }
 
 //---------------------------------------------------------
-CWKSP_Data_Item * CWKSP_Data_Manager::Add(CSG_Data_Object *pObject)
+bool CWKSP_Data_Manager::Del(CSG_Data_Object *pObject, bool bConfirm)
 {
-	if( SG_Get_Data_Manager().Add(pObject) && Get_Manager(pObject->Get_ObjectType(), true) )
+	return( Get_Control()->Del_Item(Get(pObject), bConfirm == false) ); // bConfirm == true/false => bSilent == false/true
+}
+
+
+///////////////////////////////////////////////////////////
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Clipboard_Paste_Table(void)
+{
+	if( wxTheClipboard->Open() )
 	{
-		switch( pObject->Get_ObjectType() )
+		if( wxTheClipboard->IsSupported(wxDF_TEXT) )
 		{
-		case SG_DATAOBJECT_TYPE_Table     : return( (CWKSP_Data_Item *)m_pTables     ->Add_Data((CSG_Table      *)pObject) );
-		case SG_DATAOBJECT_TYPE_TIN       : return( (CWKSP_Data_Item *)m_pTINs       ->Add_Data((CSG_TIN        *)pObject) );
-		case SG_DATAOBJECT_TYPE_PointCloud: return( (CWKSP_Data_Item *)m_pPointClouds->Add_Data((CSG_PointCloud *)pObject) );
-		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Data_Item *)m_pShapes     ->Add_Data((CSG_Shapes     *)pObject) );
-		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grid       *)pObject) );
-		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Data_Item *)m_pGrids      ->Add_Data((CSG_Grids      *)pObject) );
-		default: break;
+			wxTextDataObject Data;
+
+			if( wxTheClipboard->GetData(Data) )
+			{
+				wxString Text(Data.GetText()); CSG_Table *pTable = SG_Create_Table();
+
+				if( pTable->from_Text(&Text) )
+				{
+					CWKSP_Data_Item *pItem = Add(pTable);
+					
+					if( pItem )
+					{
+						pItem->Show();
+
+						wxTheClipboard->Close();
+
+						return( true );
+					}
+				}
+
+				delete(pTable);
+			}
 		}
+
+		wxTheClipboard->Close();
 	}
 
-	return( NULL );
+	DLG_Message_Show_Error(_TL("Failed to paste clipboard content as table!"), _TL("Paste from Clipboard"));
+
+	return( false );
 }
 
 //---------------------------------------------------------
-CWKSP_Layer * CWKSP_Data_Manager::Get_Layer(CSG_Data_Object *pObject)
+bool CWKSP_Data_Manager::Clipboard_Paste_Image(void)
 {
-	if( pObject && pObject != DATAOBJECT_CREATE && Get_Manager(pObject->Get_ObjectType()) )
+	if( wxTheClipboard->Open() )
 	{
-		switch( pObject->Get_ObjectType() )
+		if( wxTheClipboard->IsSupported(wxDF_BITMAP) )
 		{
-		case SG_DATAOBJECT_TYPE_TIN       : return( (CWKSP_Layer *)m_pTINs       ->Get_Data((CSG_TIN        *)pObject) );
-		case SG_DATAOBJECT_TYPE_PointCloud: return( (CWKSP_Layer *)m_pPointClouds->Get_Data((CSG_PointCloud *)pObject) );
-		case SG_DATAOBJECT_TYPE_Shapes    : return( (CWKSP_Layer *)m_pShapes     ->Get_Data((CSG_Shapes     *)pObject) );
-		case SG_DATAOBJECT_TYPE_Grid      : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grid       *)pObject) );
-		case SG_DATAOBJECT_TYPE_Grids     : return( (CWKSP_Layer *)m_pGrids      ->Get_Data((CSG_Grids      *)pObject) );
-		default: break;
+			wxBitmapDataObject Data;
+
+			if( wxTheClipboard->GetData(Data) && Data.GetBitmap().IsOk() )
+			{
+				wxImage Image(Data.GetBitmap().ConvertToImage());
+
+				CSG_Grid *pGrid = SG_Create_Grid(SG_DATATYPE_Int, Image.GetWidth(), Image.GetHeight());
+
+				if( pGrid )
+				{
+					pGrid->Set_Name(_TL("New Image"));
+					pGrid->Set_NoData_Value(-1.);
+
+					for(int y=0, yy=pGrid->Get_NY()-1; y<pGrid->Get_NY(); y++, yy--) for(int x=0; x<pGrid->Get_NX(); x++)
+					{
+						BYTE rgb[4] = {
+							Image.GetRed(x, y), Image.GetGreen(x, y), Image.GetBlue(x, y), Image.HasAlpha() ? Image.GetAlpha(x, y) : (BYTE)0
+						};
+
+						pGrid->Set_Value(x, yy, *(int *)rgb);
+					}
+
+					CWKSP_Data_Item *pItem = Add(pGrid);
+
+					if( pItem )
+					{
+						pItem->Get_Parameters()->Set_Parameter("COLORS_TYPE", 5               ); // RGB Coded Values
+						pItem->Get_Parameters()->Set_Parameter("RGB_ALPHA"  , Image.HasAlpha()); // Alpha Channel
+						pItem->Parameters_Changed();
+						pItem->Show(SG_UI_DATAOBJECT_SHOW_MAP);
+
+						wxTheClipboard->Close();
+
+						return( true );
+					}
+
+					delete(pGrid);
+				}
+			}
 		}
+
+		wxTheClipboard->Close();
 	}
 
-	return( NULL );
-}
+	DLG_Message_Show_Error(_TL("Failed to paste clipboard content as image!"), _TL("Paste from Clipboard"));
 
-//---------------------------------------------------------
-void CWKSP_Data_Manager::Del_Manager(CWKSP_Base_Item *pItem)
-{
-	if( pItem == m_pTables      )	m_pTables		= NULL;
-	if( pItem == m_pTINs        )	m_pTINs			= NULL;
-	if( pItem == m_pPointClouds )	m_pPointClouds	= NULL;
-	if( pItem == m_pShapes      )	m_pShapes		= NULL;
-	if( pItem == m_pGrids       )	m_pGrids		= NULL;
-
-	if( Get_Count() == 0 )
-	{
-		m_pProject->Clr_File_Name();
-	}
+	return( false );
 }
 
 
@@ -1529,10 +1761,18 @@ bool CWKSP_Data_Manager::Set_Colors(CSG_Data_Object *pObject, CSG_Colors *pColor
 	return( pLayer && pLayer->Set_Colors(pColors) );
 }
 
+//---------------------------------------------------------
+bool CWKSP_Data_Manager::Classify(CSG_Data_Object *pObject, const CSG_MetaData &Options)
+{
+	CWKSP_Layer	*pLayer	= Get_Layer(pObject);
+
+	return( pLayer && pLayer->Classify(Options) );
+}
+
 //---------------------------------------------------------
 bool CWKSP_Data_Manager::Get_Parameters(CSG_Data_Object *pObject, CSG_Parameters *pParameters)
 {
-	CWKSP_Data_Item	*pItem	= pParameters ? Get(pObject) : NULL;
+	CWKSP_Data_Item *pItem = pParameters ? Get(pObject) : NULL;
 
 	return( pItem && pParameters->Assign(pItem->Get_Parameters()) );
 }
@@ -1540,7 +1780,7 @@ bool CWKSP_Data_Manager::Get_Parameters(CSG_Data_Object *pObject, CSG_Parameters
 //---------------------------------------------------------
 bool CWKSP_Data_Manager::Set_Parameters(CSG_Data_Object *pObject, CSG_Parameters *pParameters)
 {
-	CWKSP_Data_Item	*pItem	= pParameters ? Get(pObject) : NULL;
+	CWKSP_Data_Item *pItem = pParameters ? Get(pObject) : NULL;
 
 	if( pItem && pItem->Get_Parameters()->Assign_Values(pParameters) )
 	{
