@@ -270,11 +270,11 @@ wxMenu * CWKSP_Grid::Get_Menu(void)
 	if( m_pObject->is_File_Native() )
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_DEL_FILES);
 
-	pMenu->AppendSeparator();
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECTION);
-
 	if( m_pObject->Get_MetaData().Get_Children_Count() > 0 )
+	{
+		pMenu->AppendSeparator();
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_METADATA);
+	}
 
 	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_CLASSIFY);
@@ -284,7 +284,7 @@ wxMenu * CWKSP_Grid::Get_Menu(void)
 
 	pMenu->AppendSeparator();
 
-	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Charts"   ), pSubMenu = new wxMenu());
+	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Charts"  ), pSubMenu = new wxMenu());
 	CMD_Menu_Add_Item(pSubMenu, true , ID_CMD_DATA_HISTOGRAM);
 	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_SCATTERPLOT);
 	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_SCATTERPLOT_3D);
@@ -297,6 +297,9 @@ wxMenu * CWKSP_Grid::Get_Menu(void)
 		CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_DATA_SELECTION_CLEAR);
 	}
 
+	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Advanced"), pSubMenu = new wxMenu());
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_PROJECTION);
+
 	return( pMenu );
 }
 
