@@ -230,6 +230,7 @@ void CWKSP_Layer::On_Create_Parameters(void)
 	//-----------------------------------------------------
 	// Nodes...
 
+	m_Parameters.Add_Node("", "NODE_LEGEND"   , _TL("Legend"   ), _TL(""), true);
 	m_Parameters.Add_Node("", "NODE_DISPLAY"  , _TL("Display"  ), _TL(""));
 	m_Parameters.Add_Node("", "NODE_COLORS"   , _TL("Colors"   ), _TL(""));
 	m_Parameters.Add_Node("", "NODE_SIZE"     , _TL("Size"     ), _TL(""));
@@ -249,8 +250,19 @@ void CWKSP_Layer::On_Create_Parameters(void)
 		);
 	}
 
-	m_Parameters.Add_Bool  ("NODE_GENERAL", "LEGEND_SHOW" , _TL("Show Legend"), _TL(""), true)->Set_Collapsed();
-	m_Parameters.Add_Choice("LEGEND_SHOW" , "LEGEND_STYLE", _TL("Style"      ), _TL(""), CSG_String::Format("%s|%s", _TL("vertical"), _TL("horizontal")), 0);
+	//-----------------------------------------------------
+	// Legend...
+
+	m_Parameters.Add_Bool  ("NODE_LEGEND" , "LEGEND_SHOW"       , _TL("Map Legends" ), _TL("Include in map legends."), true);
+	m_Parameters.Add_Choice("NODE_LEGEND" , "LEGEND_ORIENTATION", _TL("Orientation" ), _TL(""), CSG_String::Format("%s|%s", _TL("vertical"), _TL("horizontal")), 0);
+	m_Parameters.Add_Choice("NODE_LEGEND" , "LEGEND_TITLE"      , _TL("Title"       ), _TL(""), CSG_String::Format("%s|%s|%s", _TL("data set"), _TL("user defined"), _TL("none")), 0);
+	m_Parameters.Add_String("LEGEND_TITLE", "LEGEND_TITLE_TEXT" , _TL("User Defined"), _TL(""), _TL("Title"));
+
+	if( m_pObject->asTable(true) )
+	{
+		m_Parameters.Add_Choice("NODE_LEGEND" , "LEGEND_FIELD"     , _TL("Field Labeling"), _TL(""), CSG_String::Format("%s|%s|%s", _TL("automatic"), _TL("user defined"), _TL("none")), 0);
+		m_Parameters.Add_String("LEGEND_FIELD", "LEGEND_FIELD_TEXT", _TL("User Defined"  ), _TL(""), _TL("Label"));
+	}
 
 	//-----------------------------------------------------
 	// Display...
@@ -712,12 +724,22 @@ int CWKSP_Layer::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter
 		if(	pParameter->Cmp_Identifier("LEGEND_SHOW")
 		||	pParameter->Cmp_Identifier("COLORS_TYPE") )
 		{
-			pParameters->Set_Enabled("LEGEND_STYLE",
+			pParameters->Set_Enabled("LEGEND_ORIENTATION",
 				   (*pParameters)("LEGEND_SHOW")->asBool()
 				&& (*pParameters)("COLORS_TYPE")->asInt() == CLASSIFY_GRADUATED
 			);
 		}
 
+		if(	pParameter->Cmp_Identifier("LEGEND_TITLE") )
+		{
+			pParameters->Set_Enabled("LEGEND_TITLE_TEXT", pParameter->asInt() == 1);
+		}
+
+		if(	pParameter->Cmp_Identifier("LEGEND_FIELD") )
+		{
+			pParameters->Set_Enabled("LEGEND_FIELD_TEXT", pParameter->asInt() == 1);
+		}
+
 		if(	pParameter->Cmp_Identifier("SHOW_ALWAYS") )
 		{
 			pParameters->Set_Enabled("SHOW_RANGE", pParameter->asBool() == false);
@@ -822,7 +844,9 @@ void CWKSP_Layer::On_Parameters_Changed(void)
 
 	m_pClassify->Update();
 
-	m_pLegend->Set_Orientation(m_Parameters("LEGEND_STYLE")->asInt() == LEGEND_VERTICAL ? LEGEND_VERTICAL : LEGEND_HORIZONTAL);
+	m_pLegend->m_Title    = Get_Legend_Title   ();
+	m_pLegend->m_Subtitle = Get_Legend_Subtitle();
+	m_pLegend->Set_Orientation(m_Parameters("LEGEND_ORIENTATION")->asInt() == 0 ? LEGEND_VERTICAL : LEGEND_HORIZONTAL);
 
 	//-----------------------------------------------------
 	CWKSP_Data_Item::On_Parameters_Changed();
@@ -1019,41 +1043,57 @@ CSG_Table * CWKSP_Layer::_Get_Field_Table(int &Field, const CSG_Parameters &Para
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-wxString CWKSP_Layer::Get_Field_Naming(void) const
+wxString CWKSP_Layer::Get_Legend_Title(void) const
 {
-	wxString s;
-
-	CSG_Table *pTable = m_pObject->asTable(true);
-
-	if( pTable )
+	switch( m_Parameters["LEGEND_TITLE"].asInt() )
 	{
-		int Value = -1, Normal = -1;
+	case  0: return( m_Parameters["OBJECT_NAME"      ].asString() ); // data set
+	case  1: return( m_Parameters["LEGEND_TITLE_TEXT"].asString() ); // user defined
+	default: return( "" ); // none
+	}
+}
 
-		switch( m_Parameters["COLORS_TYPE"].asInt() )
+//---------------------------------------------------------
+wxString CWKSP_Layer::Get_Legend_Subtitle(void) const
+{
+	wxString s; CSG_Table *pTable = m_pObject->asTable(true);
+
+	if( pTable && m_Parameters["LEGEND_FIELD"].asInt() != 2 ) // no labeling?
+	{
+		if( m_Parameters["LEGEND_FIELD"].asInt() == 1 ) // user defined
 		{
-		case  1: // CLASSIFY_LUT
-			Value  = Get_Fields_Choice(m_Parameters("LUT_FIELD"    ));
-			Normal = Get_Fields_Choice(m_Parameters("LUT_NORMAL"   ));
-			break;
-
-		case  2: // CLASSIFY_DISCRETE
-		case  3: // CLASSIFY_GRADUATED
-			Value  = Get_Fields_Choice(m_Parameters("METRIC_FIELD" ));
-			Normal = Get_Fields_Choice(m_Parameters("METRIC_NORMAL"));
-			break;
-
-		case  4: // CLASSIFY_RGB
-			Value  = Get_Fields_Choice(m_Parameters("RGB_FIELD"    )); // point clouds only!
-			break;
+			s = m_Parameters["LEGEND_FIELD_TEXT"].asString();
 		}
-
-		if( Value >= 0 )
+		else // automatic
 		{
-			s = pTable->Get_Field_Name(Value);
+			int Value = -1, Normal = -1;
 
-			if( Normal >= 0 )
+			switch( m_Parameters["COLORS_TYPE"].asInt() )
 			{
-				s += " / "; s += pTable->Get_Field_Name(Normal);
+			case  1: // CLASSIFY_LUT
+				Value  = Get_Fields_Choice(m_Parameters("LUT_FIELD"    ));
+				Normal = Get_Fields_Choice(m_Parameters("LUT_NORMAL"   ));
+				break;
+
+			case  2: // CLASSIFY_DISCRETE
+			case  3: // CLASSIFY_GRADUATED
+				Value  = Get_Fields_Choice(m_Parameters("METRIC_FIELD" ));
+				Normal = Get_Fields_Choice(m_Parameters("METRIC_NORMAL"));
+				break;
+
+			case  4: // CLASSIFY_RGB
+				Value  = Get_Fields_Choice(m_Parameters("RGB_FIELD"    )); // point clouds only!
+				break;
+			}
+
+			if( Value >= 0 )
+			{
+				s = pTable->Get_Field_Name(Value);
+
+				if( Normal >= 0 )
+				{
+					s += " / "; s += pTable->Get_Field_Name(Normal);
+				}
 			}
 		}
 	}
@@ -1516,6 +1556,8 @@ int CWKSP_Layer::_Classify_Callback(CSG_Parameter *pParameter, int Flags)
 			pParameters->Set_Enabled("ALL_FIELDS"  , pParameter->asInt() == 0);
 			pParameters->Set_Enabled("NUM_FIELDS"  , pParameter->asInt() != 0);
 			pParameters->Set_Enabled("NUM_NORMAL"  , pParameter->asInt() != 0);
+			pParameters->Set_Enabled("NUM_FIXED"   , pParameter->asInt() != 0);
+			pParameters->Set_Enabled("LABELING"    , pParameter->asInt() != 0);
 			pParameters->Set_Enabled("CLASSES_MAX" , pParameter->asInt() == 0);
 			pParameters->Set_Enabled("CLASSES"     , pParameter->asInt() != 0 && pParameter->asInt() != 2 && pParameter->asInt() != 6);
 			pParameters->Set_Enabled("INTERVAL"    , pParameter->asInt() == 2);
@@ -1529,6 +1571,16 @@ int CWKSP_Layer::_Classify_Callback(CSG_Parameter *pParameter, int Flags)
 		{
 			pParameters->Set_Enabled("OFFSET_VALUE", pParameter->asInt() == 1);
 		}
+
+		if( pParameter->Cmp_Identifier("NUM_NORMAL") )
+		{
+			pParameters->Set_Enabled("NORMAL_UNIT", Get_Fields_Choice(pParameter) >= 0);
+		}
+
+		if( pParameter->Cmp_Identifier("NUM_FIXED") )
+		{
+			pParameters->Set_Enabled("DECIMALS", pParameter->asBool());
+		}
 	}
 
 	return( 1 );
@@ -1566,9 +1618,10 @@ bool CWKSP_Layer::_Classify(void)
 		{
 			m_Classify.Add_Table ("", "TABLE", _TL("Table"), _TL(""), PARAMETER_INPUT)->Set_Enabled(false); m_Classify["TABLE"].Set_Value(m_pObject);
 
-			m_Classify.Add_Choice("", "ALL_FIELDS", _TL("Field"        ), _TL(""), "");
-			m_Classify.Add_Choice("", "NUM_FIELDS", _TL("Field"        ), _TL(""), "");
-			m_Classify.Add_Choice("", "NUM_NORMAL", _TL("Normalization"), _TL(""), "");
+			m_Classify.Add_Choice(""          , "ALL_FIELDS" , _TL("Field"        ), _TL(""), "");
+			m_Classify.Add_Choice(""          , "NUM_FIELDS" , _TL("Field"        ), _TL(""), "");
+			m_Classify.Add_Choice(""          , "NUM_NORMAL" , _TL("Normalization"), _TL(""), "");
+			m_Classify.Add_Choice("NUM_NORMAL", "NORMAL_UNIT", _TL("Display as..."), _TL(""), CSG_String::Format("%s|%s", _TL("fraction"), _TL("percentage")));
 		}
 
 		if( m_pObject->asGrids() )
@@ -1582,21 +1635,23 @@ bool CWKSP_Layer::_Classify(void)
 
 			Choices += _TL("all bands");
 
-			m_Classify.Add_Choice("", "BAND"      , _TL("Apply to..."  ), _TL(""), Choices, m_pObject->asGrids()->Get_Grid_Count());
+			m_Classify.Add_Choice("", "BAND", _TL("Apply to..."  ), _TL(""), Choices, m_pObject->asGrids()->Get_Grid_Count());
 		}
 
-		m_Classify.Add_Colors(""      , "COLORS"      , _TL("Colors"                    ), _TL(""), m_Parameters["METRIC_COLORS"].asColors());
-		m_Classify.Add_Int   (""      , "CLASSES_MAX" , _TL("Maximum Number of Classes" ), _TL(""), 1000, 10, true);
-		m_Classify.Add_Int   (""      , "CLASSES"     , _TL("Number of Classes"         ), _TL(""),   10,  1, true);
-		m_Classify.Add_Choice(""      , "METHOD"      , _TL("Classification"            ), _TL(""), Methods, 1);
-		m_Classify.Add_Choice("METHOD", "LABELING"    , _TL("Labeling"                  ), _TL(""), CSG_String::Format("%s|%s", _TL("range"), _TL("less than...")), 1);
-		m_Classify.Add_Double(""      , "INTERVAL"    , _TL("Defined Interval"          ), _TL(""), 1., 0., true);
-		m_Classify.Add_Choice(""      , "OFFSET"      , _TL("Offset"                    ), _TL(""), CSG_String::Format("%s|%s", _TL("minimum value"), _TL("user defined")));
-		m_Classify.Add_Double("OFFSET", "OFFSET_VALUE", _TL("Value"                     ), _TL(""), 1.);
-		m_Classify.Add_Choice(""      , "HISTOGRAM"   , _TL("Percentile Estimation"     ), _TL(""), CSG_String::Format("%s|%s", _TL("index"), _TL("histogram")), 1);
-		m_Classify.Add_Bool  (""      , "INCREASING"  , _TL("Increasing Intervals"      ), _TL(""), true);
-		m_Classify.Add_Double(""      , "STDDEV"      , _TL("Standard Deviation"        ), _TL(""), 0.5, 0.01, true);
-		m_Classify.Add_Double("STDDEV", "STDDEV_MAX"  , _TL("Maximum Standard Deviation"), _TL(""), 4.5, 0.5 , true);
+		m_Classify.Add_Colors(""         , "COLORS"      , _TL("Colors"                    ), _TL(""), m_Parameters["METRIC_COLORS"].asColors());
+		m_Classify.Add_Int   (""         , "CLASSES_MAX" , _TL("Maximum Number of Classes" ), _TL(""), 1000, 10, true);
+		m_Classify.Add_Int   (""         , "CLASSES"     , _TL("Number of Classes"         ), _TL(""),   10,  1, true);
+		m_Classify.Add_Choice(""         , "METHOD"      , _TL("Classification"            ), _TL(""), Methods, 1);
+		m_Classify.Add_Choice("METHOD"   , "LABELING"    , _TL("Labeling"                  ), _TL(""), CSG_String::Format("%s|%s", _TL("range"), _TL("less than...")), 1);
+		m_Classify.Add_Bool  ("METHOD"   , "NUM_FIXED"   , _TL("Fixed Number of Decimals"  ), _TL(""), false);
+		m_Classify.Add_Int   ("NUM_FIXED", "DECIMALS"    , _TL("Decimals"                  ), _TL(""), 2, 0, true);
+		m_Classify.Add_Double(""         , "INTERVAL"    , _TL("Defined Interval"          ), _TL(""), 1., 0., true);
+		m_Classify.Add_Choice(""         , "OFFSET"      , _TL("Offset"                    ), _TL(""), CSG_String::Format("%s|%s", _TL("minimum value"), _TL("user defined")));
+		m_Classify.Add_Double("OFFSET"   , "OFFSET_VALUE", _TL("Value"                     ), _TL(""), 1.);
+		m_Classify.Add_Choice(""         , "HISTOGRAM"   , _TL("Percentile Estimation"     ), _TL(""), CSG_String::Format("%s|%s", _TL("index"), _TL("histogram")), 1);
+		m_Classify.Add_Bool  (""         , "INCREASING"  , _TL("Increasing Intervals"      ), _TL(""), true);
+		m_Classify.Add_Double(""         , "STDDEV"      , _TL("Standard Deviation"        ), _TL(""), 0.5, 0.01, true);
+		m_Classify.Add_Double("STDDEV"   , "STDDEV_MAX"  , _TL("Maximum Standard Deviation"), _TL(""), 4.5, 0.5 , true);
 
 		if( m_pObject->asGrid () ) { m_Classify["INTERVAL"].Set_Value(m_pObject->asGrid ()->Get_Range() / 10); m_Classify["OFFSET_VALUE"].Set_Value(m_pObject->asGrid ()->Get_Min()); }
 		if( m_pObject->asGrids() ) { m_Classify["INTERVAL"].Set_Value(m_pObject->asGrids()->Get_Range() / 10); m_Classify["OFFSET_VALUE"].Set_Value(m_pObject->asGrids()->Get_Min()); }
@@ -1654,7 +1709,9 @@ bool CWKSP_Layer::_Classify(void)
 		}
 		else // if( Method > 0 ) // quantities!
 		{
-			Classify.Create(m_pObject->asTable(true), Get_Fields_Choice(m_Classify("NUM_FIELDS")), Get_Fields_Choice(m_Classify("NUM_NORMAL")));
+			Classify.Create(m_pObject->asTable(true), Get_Fields_Choice(m_Classify("NUM_FIELDS")),
+				Get_Fields_Choice(m_Classify("NUM_NORMAL")), m_Classify["NORMAL_UNIT"].asInt() == 1
+			);
 		}
 	}
 	else if( m_pObject->asGrids() && m_Classify["BAND"].asInt() < m_pObject->asGrids()->Get_Grid_Count() )
@@ -1680,7 +1737,7 @@ bool CWKSP_Layer::_Classify(void)
 	}
 
 	//-----------------------------------------------------
-	if( Classify.Set_LUT(*m_Parameters["LUT"].asTable(), *m_Classify["COLORS"].asColors(), m_Classify["LABELING"].asInt()) )
+	if( Classify.Set_LUT(*m_Parameters["LUT"].asTable(), *m_Classify["COLORS"].asColors(), m_Classify["LABELING"].asInt(), m_Classify["NUM_FIXED"].asBool() ? m_Classify["DECIMALS"].asInt() : -2) )
 	{
 		if( m_Parameters["LUT"].asTable()->Get_MetaData().Get_Child("SAGA_GUI_LUT_TYPE") == NULL )
 		{
