@@ -71,27 +71,27 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define ZOOMED(x)			((int)(m_Zoom * (double)x))
+#define ZOOMED(x) ((int)(m_Zoom * (double)x))
 
 //---------------------------------------------------------
-#define SPACE_VERTICAL		ZOOMED(5)
-#define BOX_WIDTH			ZOOMED(30)
-#define BOX_HEIGHT			ZOOMED(12)
-#define BOX_SPACE			ZOOMED(2)
-#define TICK_WIDTH			ZOOMED(5)
-#define SIZE_HEIGHT			ZOOMED(100)
+#define SPACE_VERTICAL     ZOOMED(5)
+#define BOX_WIDTH          ZOOMED(30)
+#define BOX_HEIGHT         ZOOMED(12)
+#define BOX_SPACE          ZOOMED(2)
+#define TICK_WIDTH         ZOOMED(5)
+#define SIZE_HEIGHT        ZOOMED(100)
 
 //---------------------------------------------------------
-#define FONT_SIZE_TITLE		ZOOMED(10)
-#define FONT_SIZE_SUBTITLE	ZOOMED(8)
-#define FONT_SIZE_LABEL		ZOOMED(8)
+#define FONT_SIZE_TITLE    ZOOMED(10)
+#define FONT_SIZE_SUBTITLE ZOOMED(8)
+#define FONT_SIZE_LABEL    ZOOMED(8)
 
 //---------------------------------------------------------
-#define BOX_STYLE_RECT		0x01
-#define BOX_STYLE_LINE		0x02
-#define BOX_STYLE_SYMB		0x04
-#define BOX_STYLE_OUTL		0x08
-#define BOX_STYLE_FILL		0x10
+#define BOX_STYLE_RECT     0x01
+#define BOX_STYLE_LINE     0x02
+#define BOX_STYLE_SYMB     0x04
+#define BOX_STYLE_OUTL     0x08
+#define BOX_STYLE_FILL     0x10
 
 
 ///////////////////////////////////////////////////////////
@@ -103,7 +103,7 @@
 //---------------------------------------------------------
 enum
 {
-	FONT_TITLE		= 0,
+	FONT_TITLE = 0,
 	FONT_SUBTITLE,
 	FONT_LABEL
 };
@@ -118,29 +118,27 @@ enum
 //---------------------------------------------------------
 CWKSP_Layer_Legend::CWKSP_Layer_Legend(CWKSP_Layer *pLayer)
 {
-	m_pLayer		= pLayer;
-	m_pClassify		= pLayer->Get_Classifier();
+	m_pLayer      = pLayer;
+	m_pClassify   = pLayer->Get_Classifier();
 
-	m_Orientation	= LEGEND_VERTICAL;
+	m_Orientation = LEGEND_VERTICAL;
 }
 
 
 ///////////////////////////////////////////////////////////
 //                                                       //
-//                                                       //
-//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CWKSP_Layer_Legend::Set_Orientation(int Orientation)
 {
-	m_Orientation	= Orientation == LEGEND_VERTICAL ? LEGEND_VERTICAL : LEGEND_HORIZONTAL;
+	m_Orientation = Orientation == LEGEND_VERTICAL ? LEGEND_VERTICAL : LEGEND_HORIZONTAL;
 }
 
 //---------------------------------------------------------
 wxSize CWKSP_Layer_Legend::Get_Size(double Zoom, double Zoom_Map)
 {
-	wxMemoryDC	dc;
+	wxMemoryDC dc;
 
 	Draw(dc, Zoom, Zoom_Map, wxPoint(0, 0), NULL);
 
@@ -151,26 +149,26 @@ wxSize CWKSP_Layer_Legend::Get_Size(double Zoom, double Zoom_Map)
 void CWKSP_Layer_Legend::Draw(wxDC &dc, double Zoom, double Zoom_Map, wxPoint Position, wxSize *pSize)
 {
 	//-----------------------------------------------------
-	m_Zoom		= Zoom > 0.0 ? Zoom : 1.0;
-	m_Position	= Position;
-	m_Size		= wxSize(BOX_WIDTH, 0);
+	m_Zoom     = Zoom > 0. ? Zoom : 1.;
+	m_Position = Position;
+	m_Size     = wxSize(BOX_WIDTH, 0);
 
-	m_Zoom_Map	= Zoom_Map;
+	m_Zoom_Map = Zoom_Map;
 
 	//-----------------------------------------------------
-	m_oldPen	= dc.GetPen  ();
-	m_oldBrush	= dc.GetBrush();
-	m_oldFont	= dc.GetFont ();
+	m_oldPen   = dc.GetPen  ();
+	m_oldBrush = dc.GetBrush();
+	m_oldFont  = dc.GetFont ();
 
 	//-----------------------------------------------------
-	m_xBox		= m_Position.x;
-	m_dxBox		= BOX_WIDTH;
-	m_xTick		= m_xBox	+ m_dxBox;
-	m_dxTick	= TICK_WIDTH;
-	m_xText		= m_xTick	+ m_dxTick;
+	m_xBox     = m_Position.x;
+	m_dxBox    = BOX_WIDTH;
+	m_xTick    = m_xBox  + m_dxBox;
+	m_dxTick   = TICK_WIDTH;
+	m_xText    = m_xTick + m_dxTick;
 
 	//-----------------------------------------------------
-	_Draw_Title(dc, FONT_TITLE, m_pLayer->Get_Object()->Get_Name());
+	_Draw_Title(dc, FONT_TITLE, m_Title);
 
 	//-----------------------------------------------------
 	switch( m_pLayer->Get_Type() )
@@ -201,15 +199,13 @@ void CWKSP_Layer_Legend::Draw(wxDC &dc, double Zoom, double Zoom_Map, wxPoint Po
 	//-----------------------------------------------------
 	if( pSize )
 	{
-		*pSize	= m_Size;
+		*pSize = m_Size;
 	}
 }
 
 
 ///////////////////////////////////////////////////////////
 //                                                       //
-//                                                       //
-//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -242,7 +238,7 @@ inline void CWKSP_Layer_Legend::_Set_Font(wxDC &dc, int Style)
 //---------------------------------------------------------
 inline void CWKSP_Layer_Legend::_Draw_Title(wxDC &dc, int Style, wxString Text)
 {
-	if( Text.Length() > 0 )
+	if( !Text.IsEmpty() )
 	{
 		wxCoord	dx_Text, dy_Text;
 
@@ -265,17 +261,10 @@ inline void CWKSP_Layer_Legend::_Draw_Label(wxDC &dc, int y, wxString Text, int
 
 	switch( yAlign )
 	{
-	case TEXTALIGN_TOP:	default:
-		yAlign	= TEXTALIGN_TOPLEFT;
-		break;
-
-	case TEXTALIGN_YCENTER:
-		yAlign	= TEXTALIGN_CENTERLEFT;
-		break;
-
-	case TEXTALIGN_BOTTOM:
-		yAlign	= TEXTALIGN_BOTTOMLEFT;
-		break;
+	default:
+	case TEXTALIGN_TOP    : yAlign = TEXTALIGN_TOPLEFT   ; break;
+	case TEXTALIGN_YCENTER: yAlign = TEXTALIGN_CENTERLEFT; break;
+	case TEXTALIGN_BOTTOM : yAlign = TEXTALIGN_BOTTOMLEFT; break;
 	}
 
 	_Set_Font(dc, FONT_LABEL);
@@ -290,19 +279,8 @@ inline void CWKSP_Layer_Legend::_Draw_Label(wxDC &dc, int y, wxString Text, int
 //---------------------------------------------------------
 inline void CWKSP_Layer_Legend::_Draw_Box(wxDC &dc, int y, int dy, int Style, int LineColor, int FillColor, wxString Text)
 {
-	if( (Style & BOX_STYLE_OUTL) != 0 )
-	{
-		wxPen	Pen		= dc.GetPen();
-		Pen		.SetColour(Get_Color_asWX(LineColor));
-		dc		.SetPen   (Pen);
-	}
-
-	if( (Style & BOX_STYLE_FILL) != 0 )
-	{
-		wxBrush	Brush	= dc.GetBrush();
-		Brush	.SetColour(Get_Color_asWX(FillColor));
-		dc		.SetBrush (Brush);
-	}
+	if( (Style & BOX_STYLE_OUTL) != 0 ) { wxPen   Pen  (dc.GetPen  ()); Pen  .SetColour(Get_Color_asWX(LineColor)); dc.SetPen  (Pen  ); }
+	if( (Style & BOX_STYLE_FILL) != 0 ) { wxBrush Brush(dc.GetBrush()); Brush.SetColour(Get_Color_asWX(FillColor)); dc.SetBrush(Brush); }
 
 	if( !Text.IsEmpty() )
 	{
@@ -311,7 +289,7 @@ inline void CWKSP_Layer_Legend::_Draw_Box(wxDC &dc, int y, int dy, int Style, in
 
 	_Set_Size(0, dy);
 
-	dy	-= BOX_SPACE;
+	dy -= BOX_SPACE;
 
 	//-----------------------------------------------------
 	if( (Style & BOX_STYLE_RECT) != 0 )
@@ -338,21 +316,21 @@ inline void CWKSP_Layer_Legend::_Draw_Box(wxDC &dc, int y, int dy, int Style, in
 	int	LineColor	= Get_Color_asInt(dc.GetTextForeground());
 	int	FillColor	= Get_Color_asInt(dc.GetBrush().IsOk() ? dc.GetBrush().GetColour() : dc.GetTextBackground());
 
-	wxString	Text;
+	wxString Text;
 
 	if( iClass >= 0 )
 	{
 		if( (Style & BOX_STYLE_OUTL) == 0 )
 		{
-			LineColor	= m_pClassify->Get_Class_Color(iClass);
+			LineColor = m_pClassify->Get_Class_Color(iClass);
 		}
 
 		if( (Style & BOX_STYLE_FILL) != 0 )
 		{
-			FillColor	= m_pClassify->Get_Class_Color(iClass);
+			FillColor = m_pClassify->Get_Class_Color(iClass);
 		}
 
-		Text	= m_pClassify->Get_Class_Name(iClass);
+		Text = m_pClassify->Get_Class_Name(iClass);
 	}
 
 	_Draw_Box(dc, y, dy, Style, LineColor, FillColor, Text);
@@ -379,18 +357,13 @@ void CWKSP_Layer_Legend::_Draw_Boxes(wxDC &dc, int y, int Style, double zFactor)
 
 ///////////////////////////////////////////////////////////
 //                                                       //
-//                                                       //
-//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CWKSP_Layer_Legend::_Draw_Point(wxDC &dc, CWKSP_Shapes_Point *pLayer)
 {
-	int			size_Min, size_Max;
-	double		size_Min_Value, size_dValue;
-	wxString	size_Name;
+	int size_Min, size_Max; double size_Min_Value, size_dValue; wxString size_Name;
 
-	//-----------------------------------------------------
 	dc.SetBrush(pLayer->Get_Def_Brush());
 	dc.SetPen  (pLayer->Get_Def_Pen  ());
 
@@ -398,20 +371,17 @@ void CWKSP_Layer_Legend::_Draw_Point(wxDC &dc, CWKSP_Shapes_Point *pLayer)
 	{
 		_Draw_Title(dc, FONT_SUBTITLE, size_Name);
 
-		int		iClass, nClasses, dy, y;
-		double	iSize, dySize;
+		int  nClasses = (int)((double)SIZE_HEIGHT / (double)BOX_HEIGHT);
+		double dySize = (double)(size_Max - size_Min) / (double)nClasses;
 
-		nClasses	= (int)((double)SIZE_HEIGHT / (double)BOX_HEIGHT);
-		dySize		= (double)(size_Max - size_Min) / (double)nClasses;
-
-		for(iClass=nClasses, iSize=size_Max; iClass>=0; iClass--, iSize-=dySize)
+		for(int iClass=nClasses, iSize=size_Max; iClass>=0; iClass--, iSize-=dySize)
 		{
-			dy	= (int)(2.0 * m_Zoom_Map * iSize) + BOX_SPACE;
-			y	= m_Position.y + SIZE_HEIGHT - iClass * BOX_HEIGHT;
+			int dy = (int)(2.0 * m_Zoom_Map * iSize) + BOX_SPACE;
+			int  y = m_Position.y + SIZE_HEIGHT - iClass * BOX_HEIGHT;
 
 			pLayer->Draw_Symbol(dc, m_xBox + m_dxBox / 2, y + dy / 2, dy / 2);
 
-			_Draw_Label(dc, y, wxString::Format(wxT("%f"), size_Min_Value + (iSize - size_Min) / size_dValue), TEXTALIGN_TOP);
+			_Draw_Label(dc, y, wxString::Format("%f", size_Min_Value + (iSize - size_Min) / size_dValue), TEXTALIGN_TOP);
 		}
 
 		_Set_Size(0, SIZE_HEIGHT + dc.GetFont().GetPointSize());
@@ -423,33 +393,29 @@ void CWKSP_Layer_Legend::_Draw_Point(wxDC &dc, CWKSP_Shapes_Point *pLayer)
 	}
 
 	//-----------------------------------------------------
-	_Draw_Title(dc, FONT_SUBTITLE, pLayer->Get_Field_Naming());
+	_Draw_Title(dc, FONT_SUBTITLE, m_Subtitle);
 	_Draw_Boxes(dc, m_Position.y, pLayer->Get_Outline() ? BOX_STYLE_RECT|BOX_STYLE_FILL|BOX_STYLE_OUTL : BOX_STYLE_RECT|BOX_STYLE_FILL);
 }
 
 //---------------------------------------------------------
 void CWKSP_Layer_Legend::_Draw_Line(wxDC &dc, CWKSP_Shapes_Line *pLayer)
 {
-	//-----------------------------------------------------
-	int			size_Min, size_Max;
-	double		size_Min_Value, size_dValue;
-	wxString	size_Name;
+	int size_Min, size_Max; double size_Min_Value, size_dValue; wxString size_Name;
 
 	if( pLayer->Get_Style_Size(size_Min, size_Max, size_Min_Value, size_dValue, &size_Name) )
 	{
 		_Draw_Title(dc, FONT_SUBTITLE, size_Name);
 
-		wxPen Pen	= pLayer->Get_Def_Pen();
+		wxPen Pen = pLayer->Get_Def_Pen();
 
 		for(int iSize=size_Min; iSize<=size_Max; iSize++)
 		{
-			Pen.SetWidth(iSize);
-			dc.SetPen(Pen);
+			Pen.SetWidth(iSize); dc.SetPen(Pen);
 
-			int	y	= m_Position.y;
+			int y = m_Position.y;
 
-			_Draw_Box	(dc, y, BOX_HEIGHT, BOX_STYLE_LINE|BOX_STYLE_OUTL, -1);
-			_Draw_Label	(dc, y, wxString::Format(wxT("%f"), size_Min_Value + (iSize - size_Min) / size_dValue), TEXTALIGN_TOP);
+			_Draw_Box  (dc, y, BOX_HEIGHT, BOX_STYLE_LINE|BOX_STYLE_OUTL, -1);
+			_Draw_Label(dc, y, wxString::Format("%f", size_Min_Value + (iSize - size_Min) / size_dValue), TEXTALIGN_TOP);
 		}
 
 		if( m_pClassify->Get_Mode() == CLASSIFY_SINGLE )
@@ -461,28 +427,28 @@ void CWKSP_Layer_Legend::_Draw_Line(wxDC &dc, CWKSP_Shapes_Line *pLayer)
 	//-----------------------------------------------------
 	dc.SetPen  (pLayer->Get_Def_Pen());
 
-	_Draw_Title(dc, FONT_SUBTITLE, pLayer->Get_Field_Naming());
+	_Draw_Title(dc, FONT_SUBTITLE, m_Subtitle);
 	_Draw_Boxes(dc, m_Position.y, BOX_STYLE_LINE);
 }
 
 //---------------------------------------------------------
 void CWKSP_Layer_Legend::_Draw_Polygon(wxDC &dc, CWKSP_Shapes_Polygon *pLayer)
 {
-	_Draw_Title(dc, FONT_SUBTITLE, pLayer->Get_Field_Naming());
+	_Draw_Title(dc, FONT_SUBTITLE, m_Subtitle);
 	_Draw_Boxes(dc, m_Position.y, pLayer->Get_Outline() ? BOX_STYLE_RECT|BOX_STYLE_FILL|BOX_STYLE_OUTL : BOX_STYLE_RECT|BOX_STYLE_FILL);
 }
 
 //---------------------------------------------------------
 void CWKSP_Layer_Legend::_Draw_TIN(wxDC &dc, CWKSP_TIN *pLayer)
 {
-	_Draw_Title(dc, FONT_SUBTITLE, pLayer->Get_Field_Naming());
+	_Draw_Title(dc, FONT_SUBTITLE, m_Subtitle);
 	_Draw_Boxes(dc, m_Position.y, BOX_STYLE_RECT|BOX_STYLE_FILL|BOX_STYLE_OUTL);
 }
 
 //---------------------------------------------------------
 void CWKSP_Layer_Legend::_Draw_PointCloud(wxDC &dc, CWKSP_PointCloud *pLayer)
 {
-	_Draw_Title(dc, FONT_SUBTITLE, pLayer->Get_Field_Naming());
+	_Draw_Title(dc, FONT_SUBTITLE, m_Subtitle);
 	_Draw_Boxes(dc, m_Position.y, BOX_STYLE_RECT|BOX_STYLE_FILL|BOX_STYLE_OUTL);
 }
 
@@ -538,17 +504,17 @@ void CWKSP_Layer_Legend::_Draw_Grids(wxDC &dc, CWKSP_Grids *pLayer)
 //---------------------------------------------------------
 void CWKSP_Layer_Legend::_Draw_Layer_Image(wxDC &dc, int ay, CWKSP_Layer *pLayer)
 {
-	int		nx, ny;
+	int nx, ny;
 
 	if( pLayer->Get_Extent().Get_XRange() > pLayer->Get_Extent().Get_YRange() )
 	{
-		nx	= m_dxBox;
-		ny	= (int)(pLayer->Get_Extent().Get_YRange() / (pLayer->Get_Extent().Get_XRange() / nx));
+		nx = m_dxBox;
+		ny = (int)(pLayer->Get_Extent().Get_YRange() / (pLayer->Get_Extent().Get_XRange() / nx));
 	}
 	else
 	{
-		ny	= m_dxBox;
-		nx	= (int)(pLayer->Get_Extent().Get_XRange() / (pLayer->Get_Extent().Get_YRange() / ny));
+		ny = m_dxBox;
+		nx = (int)(pLayer->Get_Extent().Get_XRange() / (pLayer->Get_Extent().Get_YRange() / ny));
 	}
 
 	dc.DrawBitmap(pLayer->Get_Thumbnail(nx, ny), m_xBox, ay, false);
@@ -558,14 +524,12 @@ void CWKSP_Layer_Legend::_Draw_Layer_Image(wxDC &dc, int ay, CWKSP_Layer *pLayer
 	//-----------------------------------------------------
 	_Set_Size(0, ny);
 
-	ny	+= BOX_SPACE;
+	ny += BOX_SPACE;
 }
 
 
 ///////////////////////////////////////////////////////////
 //                                                       //
-//                                                       //
-//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
