@@ -837,10 +837,7 @@ bool CWKSP_Map::Update_Map(CWKSP_Layer *pLayer, bool bMapOnly)
 
 	if( bRefresh )
 	{
-		if( m_pView )
-		{
-			View_Refresh(bMapOnly);
-		}
+		View_Refresh(bMapOnly);
 
 		_Img_Save_On_Change();
 
@@ -1007,31 +1004,52 @@ CWKSP_Map_Graticule * CWKSP_Map::Add_Graticule(CSG_MetaData *pEntry)
 //---------------------------------------------------------
 CWKSP_Map_BaseMap * CWKSP_Map::Add_BaseMap(CSG_MetaData *pEntry)
 {
-	if( (Get_Count() > 0 && m_Projection.is_Okay()) || pEntry )
+	if( pEntry == NULL )
 	{
-		g_pMaps->Add(this);
-
-		CWKSP_Map_BaseMap	*pItem	= new CWKSP_Map_BaseMap(pEntry);
-
-		Add_Item(pItem);
-
-		if( !pEntry )
+		if( Get_Count() < 1 || m_Projection.is_Okay() == false )
 		{
-			pItem->Dlg_Parameters();
+			return( NULL );
 		}
 
-		switch( pItem->Get_Parameter("POSITION")->asInt() )
+		int nBaseMaps = 0;
+
+		for(int i=0; i<Get_Count(); i++)
 		{
-		default: Move_Bottom(pItem); break;
-		case  0: Move_Top   (pItem); break;
+			if( Get_Item(i)->Get_Type() == WKSP_ITEM_Map_BaseMap )
+			{
+				nBaseMaps++;
+			}
 		}
 
-		View_Refresh(true);
-
-		return( pItem );
+		if( nBaseMaps && !DLG_Message_Confirm(wxString::Format("%s\n%s", _TL("This map has already a base map."), _TL("Do you still want to add another one?")), _TL("Add Base Map")) )
+		{
+			return( NULL );
+		}
 	}
 
-	return( NULL );
+	//-----------------------------------------------------
+	CWKSP_Map_BaseMap *pItem = new CWKSP_Map_BaseMap(pEntry);
+
+	Add_Item(pItem);
+
+	if( !pEntry && !pItem->Dlg_Parameters() )
+	{
+		Get_Control()->Del_Item(pItem, true);
+
+		return( NULL );
+	}
+
+	g_pMaps->Add(this);
+
+	switch( pItem->Get_Parameter("POSITION")->asInt() )
+	{
+	default: Move_Bottom(pItem); break;
+	case  0: Move_Top   (pItem); break;
+	}
+
+	View_Refresh(true);
+
+	return( pItem );
 }
 
 //---------------------------------------------------------
@@ -1046,7 +1064,7 @@ CWKSP_Base_Item * CWKSP_Map::Add_Copy(CWKSP_Base_Item *pItem)
 
 		if( pItem->Get_Type() == WKSP_ITEM_Map_Graticule )
 		{
-			CWKSP_Map_Graticule	*pCopy	= Add_Graticule();
+			CWKSP_Map_Graticule *pCopy = Add_Graticule();
 
 			pCopy->Get_Parameters()->Assign_Values(pItem->Get_Parameters());
 
@@ -1055,11 +1073,11 @@ CWKSP_Base_Item * CWKSP_Map::Add_Copy(CWKSP_Base_Item *pItem)
 
 		if( pItem->Get_Type() == WKSP_ITEM_Map_BaseMap )
 		{
-			CSG_MetaData	Settings;
+			CSG_MetaData Settings;
 
 			pItem->Get_Parameters()->Serialize(Settings, true);
 
-			CWKSP_Map_BaseMap	*pCopy	= Add_BaseMap(&Settings);
+			CWKSP_Map_BaseMap *pCopy = Add_BaseMap(&Settings);
 
 		//	pItem->Get_Parameters()->Assign_Values(pItem->Get_Parameters());
 
@@ -1596,47 +1614,6 @@ CSG_Point CWKSP_Map::Get_World(wxRect rClient, wxPoint Point)
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-#define MASK_R 254
-#define MASK_G 255
-#define MASK_B 255
-
-//---------------------------------------------------------
-bool CWKSP_Map::Get_Image(wxImage &Image, const CSG_Grid_System &System)
-{
-	if( System.is_Valid() && Image.Create(System.Get_NX(), System.Get_NY()) )
-	{
-		wxBitmap BMP(Image); wxMemoryDC dc(BMP);
-		Draw_Map(dc, 1., Image.GetSize(), LAYER_DRAW_FLAG_NOEDITS, SG_GET_RGB(MASK_R, MASK_G, MASK_B));
-		dc.SelectObject(wxNullBitmap);
-		Image = BMP.ConvertToImage();
-		Image.SetMaskColour(MASK_R, MASK_G, MASK_B);
-
-		return( true );
-	}
-
-	return( false );
-}
-
-//---------------------------------------------------------
-bool CWKSP_Map::Get_Image(wxImage &Image, CSG_Rect &rWorld)
-{
-	if( Image.GetWidth() > 0 && Image.GetHeight() > 0 )
-	{
-		wxBitmap BMP(Image); wxMemoryDC dc(BMP);
-		Draw_Map(dc, 1., Image.GetSize(), LAYER_DRAW_FLAG_NOEDITS, SG_GET_RGB(MASK_R, MASK_G, MASK_B));
-		dc.SelectObject(wxNullBitmap);
-		Image = BMP.ConvertToImage();
-		Image.SetMaskColour(MASK_R, MASK_G, MASK_B);
-
-		rWorld = Get_World(Image.GetSize());
-
-		return( true );
-	}
-
-	return( false );
-}
-
 //---------------------------------------------------------
 bool CWKSP_Map::SaveAs_Image(void)
 {
@@ -1667,8 +1644,9 @@ bool CWKSP_Map::SaveAs_Image_Clipboard(bool bLegend)
 			Get_Manager()->Get_Parameter("CLIP_NX"         )->asInt (),
 			Get_Manager()->Get_Parameter("CLIP_NY"         )->asInt (),
 			Get_Manager()->Get_Parameter("CLIP_FRAME_SHOW" )->asBool() ?
-			Get_Manager()->Get_Parameter("CLIP_FRAME_WIDTH")->asInt () : 0)
-		);
+			Get_Manager()->Get_Parameter("CLIP_FRAME_WIDTH")->asInt () : 0,
+			Get_Manager()->Get_Parameter("CLIP_ALPHA"      )->asBool()
+		));
 	}
 
 	//-----------------------------------------------------
@@ -1714,34 +1692,59 @@ bool CWKSP_Map::SaveAs_Image_Clipboard(bool bLegend)
 }
 
 //---------------------------------------------------------
-bool CWKSP_Map::SaveAs_Image_Clipboard(int nx, int ny, int frame)
+bool CWKSP_Map::SaveAs_Image_Clipboard(int nx, int ny, int frame, bool bAlpha)
 {
+	if( nx < 10 || ny < 10 )
+	{
+		return( false );
+	}
+
 	Set_Buisy_Cursor(true);
 
-	if( frame < 10 ) frame = 0; // if( frame <  0 ) frame = Get_Frame_Width();
+	if( frame < 10 || 2 * frame + 10 > nx || 2 * frame + 10 > ny ) { frame = 0; }
 
-	wxRect r = wxRect(0, 0, nx + 2 * frame, ny + 2 * frame);
+	wxRect rClient = wxRect(0, 0, nx + 2 * frame, ny + 2 * frame);
 
-	wxBitmap BMP(r.GetWidth(), r.GetHeight());
+	CSG_Rect rWorld = Get_World(rClient);
 
-	r.Deflate(frame);
+	if( frame )
+	{
+		rWorld.Inflate(frame * rWorld.Get_XRange() / nx, false);
+	}
 
-	wxMemoryDC dc(BMP); dc.SetBackground(*wxWHITE_BRUSH); dc.Clear();
+	//-----------------------------------------------------
+	int Background = Get_Color_asInt(Get_Background());
 
-	Draw_Map(dc, 1., r, LAYER_DRAW_FLAG_NOEDITS);
+	CSG_Map_DC dcMap(rWorld, rClient, 1., Background, bAlpha);
+
+	Draw_Map(dcMap, LAYER_DRAW_FLAG_NOEDITS);
 
 	if( frame > 0 )
 	{
-		Draw_Frame(dc, r, frame);
+		if( bAlpha )
+		{
+			dcMap.Draw_Layer_Begin();
+
+			dcMap.SetPen(*wxWHITE_PEN); dcMap.SetBrush(*wxWHITE_BRUSH);
+			dcMap.DrawRectangle(0, 0, rClient.width , frame); dcMap.DrawRectangle(rClient.width  - frame, 0, frame, rClient.height);
+			dcMap.DrawRectangle(0, 0, frame, rClient.height); dcMap.DrawRectangle(0, rClient.height - frame, rClient.width , frame);
+			dcMap.SetPen(*wxBLACK_PEN);
+		}
+
+		rClient.Deflate(frame); Draw_Frame(dcMap.Get_DC(), rClient, frame);
+
+		if( bAlpha )
+		{
+			dcMap.Draw_Layer_End();
+		}
 	}
 
-	dc.SelectObject(wxNullBitmap);
-
+	//-----------------------------------------------------
 	if( wxTheClipboard->Open() )
 	{
-		wxBitmapDataObject *pBMP = new wxBitmapDataObject;
-		pBMP->SetBitmap(BMP);
-		wxTheClipboard->SetData(pBMP);
+		wxBitmapDataObject *pBitmap = new wxBitmapDataObject;
+		pBitmap->SetBitmap(dcMap.Get_Bitmap());
+		wxTheClipboard->SetData(pBitmap);
 		wxTheClipboard->Close();
 	}
 
@@ -1776,38 +1779,44 @@ bool CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 		return( false );
 	}
 
+	//-----------------------------------------------------
+	static CSG_Parameters Parameters(_TL("Export Map to Google Earth")); static CSG_Parameters_Grid_Target Target;
+
+	if( !Parameters.Get_Count() )
+	{
+		Parameters.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
+
+		Parameters.Add_FilePath("",
+			"FILE", _TL("File"),
+			_TL("Omitting the target file will create it as temporary file."),
+			CSG_String::Format("%s|*.kmz|%s|*.*",
+				_TL("KMZ Files"),
+				_TL("All Files")
+			), NULL, true
+		);
+
+		Parameters.Add_Bool("",
+			"OPEN", _TL("Open"),
+			_TL("Try to open KMZ file after creation with the associated application (e.g. \'Google Earth\')."),
+			true
+		);
+
+		Target.Create(&Parameters, true, "", "TARGET_");
+	}
+
+	//-----------------------------------------------------
 	CSG_Grid_System System(Get_Extent().Get_XRange() / nx, Get_Extent());
 
-	CSG_Parameters P(_TL("Export Map to Google Earth"), SG_T(""), SG_T("SAVE_IMAGE"));
-	P.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
+	Target.Set_User_Defined(&Parameters, System);
 
-	P.Add_FilePath("",
-		"FILE", _TL("File"),
-		_TL("Omitting the target file will create it as temporary file."),
-		CSG_String::Format("%s|*.kmz|%s|*.*",
-			_TL("KMZ Files"),
-			_TL("All Files")
-		), NULL, true
-	);
-
-	P.Add_Bool("",
-		"OPEN", _TL("Open"),
-		_TL("Try to open KMZ file after creation with the associated application (e.g. \'Google Earth\')."),
-		true
-	);
-
-	CSG_Parameters_Grid_Target Target;
-	Target.Create(&P, true, "", "TARGET_");
-	Target.Set_User_Defined(&P, System);
-
-	if( !DLG_Parameters(&P) )
+	if( !DLG_Parameters(&Parameters) )
 	{
 		return( false );
 	}
 
 	System = Target.Get_System();
 
-	wxFileName FileName(P["FILE"].asString());
+	wxFileName FileName(Parameters["FILE"].asString());
 
 	if( !FileName.IsOk() )
 	{
@@ -1823,40 +1832,33 @@ bool CWKSP_Map::SaveAs_Image_To_KMZ(int nx, int ny)
 	}
 
 	//-----------------------------------------------------
-	wxImage Image;
+	CSG_Map_DC dcMap(System.Get_Extent(), wxSize(System.Get_NX(), System.Get_NY()), 1., -1, true);
 
-	if( !Get_Image(Image, System) )
+	Draw_Map(dcMap, LAYER_DRAW_FLAG_NOEDITS);
+
+	CSG_Grid Map;
+
+	if( !dcMap.Get_Image(Map) )
 	{
 		return( false );
 	}
 
-	//-----------------------------------------------------
-	CSG_Grid Map(System, SG_DATATYPE_Int);
-
 	Map.Set_Name(Get_Name().wx_str());
-	Map.Set_NoData_Value(SG_GET_RGB(MASK_R, MASK_G, MASK_B));
+	Map.Set_NoData_Value(0);
 	Map.Get_Projection().Create(m_Projection);
 
-	#pragma omp parallel for
-	for(int y=0; y<Map.Get_NY(); y++)
-	{
-		for(int x=0, yy=Map.Get_NY()-1-y; x<Map.Get_NX(); x++)
-		{
-			Map.Set_Value(x, y, SG_GET_RGB(Image.GetRed(x, yy), Image.GetGreen(x, yy), Image.GetBlue(x, yy)));
-		}
-	}
-
 	//-----------------------------------------------------
-	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("io_grid_image", 2, true);
+	CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("io_grid_image", 2, true); // Export Grid to KML
 
 	if(	pTool && pTool->Settings_Push()
 	&&  pTool->Set_Parameter("GRID"     , &Map)
 	&&  pTool->Set_Parameter("FILE"     , FileName.GetFullPath().wc_str())
 	&&  pTool->Set_Parameter("COLOURING", 4) // rgb coded values
-	&&  pTool->Set_Parameter("FORMAT"   , 0) // 0 = png, 1 = jpg (default)
+	&&  pTool->Set_Parameter("ALPHA"    , true)
+	&&  pTool->Set_Parameter("FORMAT"   , 0) // 0 = png (default), 1 = jpg
 	&&  pTool->Execute() )
 	{
-		if( P["OPEN"].asBool() )
+		if( Parameters["OPEN"].asBool() )
 		{
 			Open_Application(FileName.GetFullPath());
 		}
@@ -1879,93 +1881,98 @@ bool CWKSP_Map::SaveAs_Image_To_Memory(int nx, int ny)
 		return( false );
 	}
 
-	CSG_Grid_System System(Get_Extent().Get_XRange() / nx, Get_Extent());
-
-	CSG_Parameters P(_TL("Save To Memory Grid"), SG_T(""), SG_T("SAVE_IMAGE"));
-	P.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
-
-	CSG_Parameters_Grid_Target Target;
-	Target.Create(&P, true, "", "TARGET_");
-	Target.Set_User_Defined(&P, System);
-
-	if( !DLG_Parameters(&P) )
-	{
-		return( false );
-	}
-
-	System = Target.Get_System();
-
 	//-----------------------------------------------------
-	wxImage Image;
+	CSG_Rect Extent(Get_World(wxSize(nx, ny)));
+
+	CSG_Grid_System System(Extent.Get_XRange() / nx, Extent);
 
-	if( !Get_Image(Image, System) )
 	{
-		return( false );
-	}
+		CSG_Parameters Parameters(_TL("Save To Memory Grid"), SG_T(""), SG_T("SAVE_IMAGE"));
 
-	//-----------------------------------------------------
-	CSG_Grid *pMap = SG_Create_Grid(System, SG_DATATYPE_Int);
+		Parameters.Set_Callback_On_Parameter_Changed(_On_Parameter_Changed);
 
-	pMap->Set_Name(Get_Name().wx_str());
-	pMap->Set_NoData_Value(16711935);
-	pMap->Get_Projection().Create(m_Projection);
+		CSG_Parameters_Grid_Target Target;
+		Target.Create(&Parameters, true, "", "TARGET_");
+		Target.Set_User_Defined(&Parameters, System);
 
-	for(int y=0, yy=pMap->Get_NY()-1; y<pMap->Get_NY(); y++, yy--)
-	{
-		for(int x=0; x<pMap->Get_NX(); x++)
+		if( !DLG_Parameters(&Parameters) )
 		{
-			pMap->Set_Value(x, y, SG_GET_RGB(Image.GetRed(x, yy), Image.GetGreen(x, yy), Image.GetBlue(x, yy)));
+			return( false );
 		}
+
+		System = Target.Get_System();
 	}
 
 	//-----------------------------------------------------
-	g_pData->Add(pMap);
+	int Background = Get_Color_asInt(Get_Background());
 
-	g_pData->Get_Parameters(pMap, &P);
+	CSG_Map_DC dcMap(System.Get_Extent(), wxSize(System.Get_NX(), System.Get_NY()), 1., Background, true);
 
-	if( P("COLORS_TYPE") )
+	Draw_Map(dcMap, LAYER_DRAW_FLAG_NOEDITS);
+
+	CSG_Grid &Map = *SG_Create_Grid(System, SG_DATATYPE_Int);
+
+	if( !dcMap.Get_Image(Map) )
 	{
-		P("COLORS_TYPE")->Set_Value(5);	// Color Classification Type: RGB Coded Values
-
-		g_pData->Set_Parameters(pMap, &P);
+		return( false );
 	}
 
+	Map.Set_Name(Get_Name().wx_str());
+	Map.Set_NoData_Value(Background);
+	Map.Get_Projection().Create(m_Projection);
+
+	//-----------------------------------------------------
+	g_pData->Add(&Map); CSG_Parameters Parameters; g_pData->Get_Parameters(&Map, &Parameters);
+
+	Parameters.Set_Parameter("COLORS_TYPE", 5   ); // RGB Coded Values
+	Parameters.Set_Parameter("RGB_ALPHA"  , true); // Alpha Channel
+
+	g_pData->Set_Parameters(&Map, &Parameters);
+
 	return( true );
 }
 
 //---------------------------------------------------------
-bool CWKSP_Map::SaveAs_Image_To_Grid(CSG_Grid &Grid, int Size)
+bool CWKSP_Map::SaveAs_Image_To_Grid(CSG_Grid &Map, int maxSize)
 {
-	if( Size < 1 )
+	if( maxSize < 1 )
 	{
 		return( false );
 	}
 
-	CSG_Rect Extent(Get_Extent()); wxImage Image;
+	//-----------------------------------------------------
+	CSG_Rect Extent(Get_Extent()); wxSize Size;
 
 	if( Extent.Get_XRange() > Extent.Get_YRange() )
 	{
-		Image.Create(Size, Size * Extent.Get_YRange() / Extent.Get_XRange());
+		Size.Set(maxSize, maxSize * Extent.Get_YRange() / Extent.Get_XRange());
 	}
 	else
 	{
-		Image.Create(Size * Extent.Get_XRange() / Extent.Get_YRange(), Size);
+		Size.Set(maxSize * Extent.Get_XRange() / Extent.Get_YRange(), maxSize);
 	}
 
-	if( Get_Image(Image, Extent) )
+	if( Size.x < 1 || Size.y < 1 )
 	{
-		Grid.Create(SG_DATATYPE_Int, Image.GetWidth(), Image.GetHeight(), Extent.Get_XRange() / (double)Image.GetWidth(), Extent.Get_XMin(), Extent.Get_YMin());
-		Grid.Set_NoData_Value(SG_GET_RGB(MASK_R, MASK_G, MASK_B));
-
-		for(int y=0, yy=Grid.Get_NY()-1; y<Grid.Get_NY(); y++, yy--)
-		{
-			for(int x=0; x<Grid.Get_NX(); x++)
-			{
-				Grid.Set_Value(x, y, SG_GET_RGB(Image.GetRed(x, yy), Image.GetGreen(x, yy), Image.GetBlue(x, yy)));
-			}
-		}
+		return( false );
 	}
 
+	//-----------------------------------------------------
+	int Background = Get_Color_asInt(Get_Background());
+
+	CSG_Map_DC dcMap(Extent, Size, 1., Background, true);
+
+	Draw_Map(dcMap, LAYER_DRAW_FLAG_NOEDITS);
+
+	if( !dcMap.Get_Image(Map) )
+	{
+		return( false );
+	}
+
+	Map.Set_Name(Get_Name().wx_str());
+	Map.Set_NoData_Value(Background);
+	Map.Get_Projection().Create(m_Projection);
+
 	return( true );
 }
 
@@ -2182,15 +2189,15 @@ wxColour CWKSP_Map::Get_Background(void)
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-void CWKSP_Map::Draw_Map(wxDC &dc, double Zoom, const wxRect &rClient, int Flags, int Background)
+void CWKSP_Map::Draw_Map(wxDC &dc, double Zoom, const wxRect &rClient, int Flags, int Background, bool bAlpha)
 {
-	Draw_Map(dc, Get_World(rClient), Zoom, rClient, Flags, Background);
+	Draw_Map(dc, Get_World(rClient), Zoom, rClient, Flags, Background, bAlpha);
 }
 
 //---------------------------------------------------------
-void CWKSP_Map::Draw_Map(wxDC &dc, const CSG_Rect &rWorld, double Zoom, const wxRect &rClient, int Flags, int Background)
+void CWKSP_Map::Draw_Map(wxDC &dc, const CSG_Rect &rWorld, double Zoom, const wxRect &rClient, int Flags, int Background, bool bAlpha)
 {
-	CSG_Map_DC dc_Map(rWorld, rClient, Zoom, Background >= 0 ? Background : Get_Background().GetRGB());
+	CSG_Map_DC dc_Map(rWorld, rClient, Zoom, Background >= 0 ? Background : Get_Background().GetRGB(), bAlpha);
 
 	Draw_Map(dc_Map, Flags);
 
@@ -2247,9 +2254,13 @@ void CWKSP_Map::Draw_Map(CSG_Map_DC &dc, int Flags)
 	//-----------------------------------------------------
 	if( (Flags & LAYER_DRAW_FLAG_THUMBNAIL) == 0 )
 	{
+		if( dc.has_Alpha() ) { dc.Draw_Layer_Begin(); }
+
 		Draw_Extent     (dc);
 		Draw_ScaleBar   (dc);
 		Draw_North_Arrow(dc);
+
+		if( dc.has_Alpha() ) { dc.Draw_Layer_End  (); }
 	}
 }
 
