@@ -535,7 +535,7 @@ bool CWKSP_Map_BaseMap::Draw(CSG_Map_DC &dc_Map)
 		if( !Set_BaseMap(System) )
 		{
 			wxColour oldColour(dc_Map.Get_DC().GetTextForeground()); dc_Map.Get_DC().SetTextForeground(*wxLIGHT_GREY);
-			wxFont   oldFont  (dc_Map.Get_DC().GetFont          ()); dc_Map.Get_DC().SetFont          (wxFont()     );
+			wxFont   oldFont  (dc_Map.Get_DC().GetFont          ()); dc_Map.Get_DC().SetFont(wxSystemSettings::GetFont(wxSYS_SYSTEM_FONT));
 
 			wxString Text(_TL("Base Map Error")); wxSize Size(dc_Map.Get_DC().GetTextExtent(Text));
 
