@@ -47,6 +47,7 @@
 
 //---------------------------------------------------------
 #include "res_commands.h"
+#include "res_controls.h"
 #include "res_dialogs.h"
 
 #include "helper.h"
@@ -55,6 +56,7 @@
 #include "active_attributes.h"
 
 #include "wksp_map_control.h"
+#include "wksp_map.h"
 
 #include "wksp_layer_classify.h"
 
@@ -65,11 +67,13 @@
 #include "view_table.h"
 #include "view_scatterplot.h"
 
+#include "view_map.h"
+
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -87,8 +91,8 @@ CWKSP_PointCloud::CWKSP_PointCloud(CSG_PointCloud *pPointCloud)
 
 	DataObject_Changed();
 
-	m_Parameters("COLORS_TYPE"  )->Set_Value(CLASSIFY_GRADUATED);
-	m_Parameters("METRIC_ATTRIB")->Set_Value(2);
+	m_Parameters("COLORS_TYPE" )->Set_Value(CLASSIFY_GRADUATED);
+	m_Parameters("METRIC_FIELD")->Set_Value(2);
 
 	CSG_Table &LUT = *m_Parameters("LUT")->asTable(); LUT.Del_Records();
 
@@ -109,7 +113,7 @@ CWKSP_PointCloud::CWKSP_PointCloud(CSG_PointCloud *pPointCloud)
 	ADD_CLASS(16711680, "Water"                       ,  9);
 	ADD_CLASS(16711935, "Overlap Points"              , 12);
 
-	On_Parameter_Changed(&m_Parameters, m_Parameters("METRIC_ATTRIB"), PARAMETER_CHECK_ALL);
+	On_Parameter_Changed(&m_Parameters, m_Parameters("METRIC_FIELD"), PARAMETER_CHECK_ALL);
 
 	Parameters_Changed();
 }
@@ -122,7 +126,7 @@ CWKSP_PointCloud::~CWKSP_PointCloud(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -133,6 +137,8 @@ wxString CWKSP_PointCloud::Get_Description(void)
 	//-----------------------------------------------------
 	s += wxString::Format("<h4>%s</h4>", _TL("Point Cloud"));
 
+	s += g_pData->Set_Description_Image(this);
+
 	s += "<table border=\"0\">";
 
 	DESC_ADD_STR  (_TL("Name"            ), m_pObject->Get_Name());
@@ -162,6 +168,13 @@ wxString CWKSP_PointCloud::Get_Description(void)
 	DESC_ADD_LONG(_TL("Number of Points" ), Get_PointCloud()->Get_Count());
 	DESC_ADD_LONG(_TL("Selected"         ), Get_PointCloud()->Get_Selection_Count());
 
+	double Samples = 100. * (double)m_pObject->Get_Max_Samples() / (double)m_pObject->asPointCloud()->Get_Count();
+
+	if( Samples < 100. )
+	{
+		DESC_ADD_STR(_TL("Sample Size"   ), wxString::Format("%lld (%.02f%%)", m_pObject->Get_Max_Samples(), Samples));
+	}
+
 	s += "</table>";
 
 	s += wxString::Format("<hr><h4>%s</h4>", _TL("Coordinate System Details"));
@@ -178,19 +191,53 @@ wxString CWKSP_PointCloud::Get_Description(void)
 	return( s );
 }
 
+
+///////////////////////////////////////////////////////////
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+wxToolBarBase * CWKSP_PointCloud::Get_ToolBar(void)
+{
+	static wxToolBarBase *static_pToolBar = NULL;
+
+	if( !static_pToolBar )
+	{
+		static_pToolBar = CMD_ToolBar_Create(ID_TB_DATA_POINTCLOUD);
+
+		Add_ToolBar_Defaults(static_pToolBar);
+		CMD_ToolBar_Add_Item(static_pToolBar,  true, ID_CMD_TABLE_SHOW);
+		CMD_ToolBar_Add_Item(static_pToolBar, false, ID_CMD_POINTCLOUD_3DVIEW);
+
+		CMD_ToolBar_Add(static_pToolBar, _TL("Point Cloud"));
+	}
+
+	return( static_pToolBar );
+}
+
 //---------------------------------------------------------
 wxMenu * CWKSP_PointCloud::Get_Menu(void)
 {
 	wxMenu *pSubMenu, *pMenu = new wxMenu(m_pObject->Get_Name());
 
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_WKSP_ITEM_CLOSE);
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SHOW_MAP);
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_ADD_TO_MAP);
+
 	if( MDI_Get_Active_Map() )
 	{
-		CMD_Menu_Add_Item(pMenu, false, ID_CMD_MAP_ZOOM_ACTIVE);
-		CMD_Menu_Add_Item(pMenu, false, ID_CMD_MAP_PAN_ACTIVE);
+		pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Active Map"), pSubMenu = new wxMenu());
+
+		if( Can_Add_To_Active_Map() )
+		{
+			CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_ADD_TO_MAP_ACTIVE, _TL("Add Layer"));
+		}
+
+		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_ZOOM_LAYER_ACTIVE);
+		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_PAN_LAYER_ACTIVE);
 	}
 
+	CMD_Menu_Add_Item(pMenu, false, ID_CMD_POINTCLOUD_3DVIEW);
+
 	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVE);
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_SAVEAS);
@@ -201,11 +248,11 @@ wxMenu * CWKSP_PointCloud::Get_Menu(void)
 	if( m_pObject->is_File_Native() )
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_DEL_FILES);
 
-	pMenu->AppendSeparator();
-	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_PROJECTION);
-
 	if( m_pObject->Get_MetaData().Get_Children_Count() > 0 )
+	{
+		pMenu->AppendSeparator();
 		CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_METADATA);
+	}
 
 	pMenu->AppendSeparator();
 	CMD_Menu_Add_Item(pMenu, false, ID_CMD_DATA_CLASSIFY);
@@ -232,20 +279,25 @@ wxMenu * CWKSP_PointCloud::Get_Menu(void)
 	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Selection" ), pSubMenu = new wxMenu());
 	if( MDI_Get_Active_Map() && Get_PointCloud()->Get_Selection_Count() > 0 )
 	{
-		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_ZOOM_SELECTION);
-		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_PAN_SELECTION);
+		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_ZOOM_LAYER_SELECTION);
+		CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_MAP_PAN_LAYER_SELECTION);
 	}
-	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_DATA_SELECTION_CLEAR);
-	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_SELECTION_INVERT);
-	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_TABLE_SELECT_NUMERIC);
-	CMD_Menu_Add_Item(pSubMenu,  true, ID_CMD_TABLE_SELECT_STRING);
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_SELECTION_CLEAR  );
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_SELECTION_INVERT );
+	pSubMenu->AppendSeparator();
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_TABLE_SELECT_NUMERIC  );
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_TABLE_SELECT_STRING   );
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_SHAPES_SELECT_LOCATION);
+
+	pMenu->Append(ID_CMD_WKSP_FIRST, _TL("Advanced"  ), pSubMenu = new wxMenu());
+	CMD_Menu_Add_Item(pSubMenu, false, ID_CMD_DATA_PROJECTION);
 
 	return( pMenu );
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -256,31 +308,28 @@ bool CWKSP_PointCloud::On_Command(int Cmd_ID)
 	default:
 		return( CWKSP_Layer::On_Command(Cmd_ID) );
 
+	//-----------------------------------------------------
 	case ID_CMD_POINTCLOUD_LAST:
 		break;
 
-	case ID_CMD_DATA_CLASSIFY:
-		_LUT_Create();
-		break;
-
 	case ID_CMD_DATA_RANGE_MINMAX:
-		Set_Color_Range(
-			Get_PointCloud()->Get_Minimum(m_fValue),
-			Get_PointCloud()->Get_Maximum(m_fValue)
+		Set_Stretch_Range(
+			Get_PointCloud()->Get_Minimum(m_Stretch.Value),
+			Get_PointCloud()->Get_Maximum(m_Stretch.Value)
 		);
 		break;
 
 	case ID_CMD_DATA_RANGE_STDDEV150:
-		Set_Color_Range(
-			Get_PointCloud()->Get_Mean(m_fValue) - 1.5 * Get_PointCloud()->Get_StdDev(m_fValue),
-			Get_PointCloud()->Get_Mean(m_fValue) + 1.5 * Get_PointCloud()->Get_StdDev(m_fValue)
+		Set_Stretch_Range(
+			Get_PointCloud()->Get_Mean(m_Stretch.Value) - 1.5 * Get_PointCloud()->Get_StdDev(m_Stretch.Value),
+			Get_PointCloud()->Get_Mean(m_Stretch.Value) + 1.5 * Get_PointCloud()->Get_StdDev(m_Stretch.Value)
 		);
 		break;
 
 	case ID_CMD_DATA_RANGE_STDDEV200:
-		Set_Color_Range(
-			Get_PointCloud()->Get_Mean(m_fValue) - 2. * Get_PointCloud()->Get_StdDev(m_fValue),
-			Get_PointCloud()->Get_Mean(m_fValue) + 2. * Get_PointCloud()->Get_StdDev(m_fValue)
+		Set_Stretch_Range(
+			Get_PointCloud()->Get_Mean(m_Stretch.Value) - 2. * Get_PointCloud()->Get_StdDev(m_Stretch.Value),
+			Get_PointCloud()->Get_Mean(m_Stretch.Value) + 2. * Get_PointCloud()->Get_StdDev(m_Stretch.Value)
 		);
 		break;
 
@@ -289,19 +338,42 @@ bool CWKSP_PointCloud::On_Command(int Cmd_ID)
 	case ID_CMD_DATA_SELECTION_INVERT: Get_PointCloud()->Inv_Selection(); Update_Views(); break;
 
 	case ID_CMD_TABLE_SELECT_NUMERIC : { CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Get_Tool("shapes_tools", 3);
-			if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("SHAPES", m_pObject) && DLG_Parameters(pTool->Get_Parameters()) )
-			{
-				pTool->Execute();
-			}
+		if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("SHAPES", m_pObject) && DLG_Parameters(pTool->Get_Parameters()) )
+		{
+			pTool->Execute();
+		}
  		break; }
 
 	case ID_CMD_TABLE_SELECT_STRING  : { CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Get_Tool("shapes_tools", 4);
-			if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("SHAPES", m_pObject) && DLG_Parameters(pTool->Get_Parameters()) )
-			{
-				pTool->Execute();
-			}
+		if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("SHAPES", m_pObject) && DLG_Parameters(pTool->Get_Parameters()) )
+		{
+			pTool->Execute();
+		}
  		break; }
 
+	case ID_CMD_SHAPES_SELECT_LOCATION: { CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Get_Tool("shapes_tools", 5);
+		if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("SHAPES", m_pObject) && DLG_Parameters(pTool->Get_Parameters()) )
+		{
+			pTool->Execute();
+		}
+		break; }
+
+	//-----------------------------------------------------
+	case ID_CMD_POINTCLOUD_3DVIEW    : { CSG_Tool *pTool = SG_Get_Tool_Library_Manager().Create_Tool("vis_3d_viewer", 1);
+		if(	pTool && pTool->On_Before_Execution() && pTool->Set_Parameter("POINTS", m_pObject) )
+		{
+			if( MDI_Get_Active_Map() && ((CVIEW_Map *)MDI_Get_Active_Map())->Get_Map()->Get_Map_Layer(this) )
+			{
+				CSG_Rect Extent(((CVIEW_Map *)MDI_Get_Active_Map())->Get_Map()->Get_Extent());
+
+				pTool->Set_Parameter("XRANGE.MIN", Extent.xMin); pTool->Set_Parameter("XRANGE.MAX", Extent.xMax);
+				pTool->Set_Parameter("YRANGE.MIN", Extent.yMin); pTool->Set_Parameter("YRANGE.MAX", Extent.yMax);
+			}
+
+			pTool->Execute();
+		}
+		break; }
+
 	//-----------------------------------------------------
 	case ID_CMD_SHAPES_EDIT_DEL_SHAPE:
 		if( Get_PointCloud()->Get_Selection_Count() > 0 && DLG_Message_Confirm(_TL("Delete selected point(s)."), _TL("Edit Point Cloud")) )
@@ -333,7 +405,6 @@ bool CWKSP_PointCloud::On_Command(int Cmd_ID)
 	//-----------------------------------------------------
 	case ID_CMD_TABLE_SHOW    : m_pTable->Toggle_View   (); break;
 	case ID_CMD_DATA_DIAGRAM  : m_pTable->Toggle_Diagram(); break;
-	case ID_CMD_DATA_HISTOGRAM: Histogram_Toggle        (); break;
 	}
 
 	return( true );
@@ -347,28 +418,14 @@ bool CWKSP_PointCloud::On_Command_UI(wxUpdateUIEvent &event)
 	default:
 		return( CWKSP_Layer::On_Command_UI(event) );
 
-    case ID_CMD_DATA_SELECTION_CLEAR:
-        event.Enable(Get_PointCloud()->Get_Selection_Count() > 0);
-        break;
+	//-----------------------------------------------------
+    case ID_CMD_DATA_SELECTION_CLEAR : event.Enable(Get_PointCloud()->Get_Selection_Count() > 0); break;
+    case ID_CMD_DATA_SELECTION_INVERT: event.Enable(Get_PointCloud()->Get_Selection_Count() > 0); break;
+    case ID_CMD_SHAPES_EDIT_DEL_SHAPE: event.Enable(Get_PointCloud()->Get_Selection_Count() > 0); break;
+    case ID_CMD_SHAPES_EDIT_SEL_COPY : event.Enable(Get_PointCloud()->Get_Selection_Count() > 0); break;
 
-    case ID_CMD_DATA_SELECTION_INVERT:
-        event.Enable(true);
-        break;
-
-    case ID_CMD_SHAPES_EDIT_DEL_SHAPE:
-        event.Enable(Get_PointCloud()->Get_Selection_Count() > 0);
-        break;
-
-    case ID_CMD_SHAPES_EDIT_SEL_COPY:
-        event.Enable(Get_PointCloud()->Get_Selection_Count() > 0);
-        break;
-
-	case ID_CMD_POINTCLOUD_LAST:
-		break;
-
-	case ID_CMD_TABLE_SHOW    : event.Check(m_pTable->Get_View   () != NULL); break;
-	case ID_CMD_DATA_DIAGRAM  : event.Check(m_pTable->Get_Diagram() != NULL); break;
-	case ID_CMD_DATA_HISTOGRAM: event.Check(m_pHistogram            != NULL); break;
+	case ID_CMD_TABLE_SHOW           : event.Check(m_pTable->Get_View   () != NULL); break;
+	case ID_CMD_DATA_DIAGRAM         : event.Check(m_pTable->Get_Diagram() != NULL); break;
 	}
 
 	return( true );
@@ -376,7 +433,7 @@ bool CWKSP_PointCloud::On_Command_UI(wxUpdateUIEvent &event)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -385,66 +442,44 @@ void CWKSP_PointCloud::On_Create_Parameters(void)
 	CWKSP_Layer::On_Create_Parameters();
 
 	//-----------------------------------------------------
+	// Values...
+
+	m_Parameters.Add_Choice("NODE_VALUES", "DISPLAY_VALUE_AGGREGATE", _TL("Aggregation"), _TL(""), CSG_String::Format("%s|%s|%s|%s",
+		_TL("first"  ),
+		_TL("last"   ),
+		_TL("lowest" ),
+		_TL("highest")), 3
+	);
+
+	//-----------------------------------------------------
+	// Display...
+
 	m_Parameters.Add_Int("NODE_DISPLAY",
-		"DISPLAY_SIZE"	, _TL("Point Size"),
-		_TL("Radius given as number of pixels."),
+		"DISPLAY_SIZE"           , _TL("Point Size"),
+		_TL("Point radius size as number of pixels."),
 		1, 1, true, 10, true
 	);
 
-	m_Parameters.Add_Choice("NODE_DISPLAY",
-		"DISPLAY_VALUE_AGGREGATE", _TL("Value Aggregation"),
-		_TL(""),
-		CSG_String::Format("%s|%s|%s|%s",
-			_TL("first value"),
-			_TL("last value" ),
-			_TL("lowest z"   ),
-			_TL("highest z"  )
-		), 3
-	);
-
 	//-----------------------------------------------------
-	m_Parameters.Add_Node("NODE_COLORS", "NODE_RGB", _TL("RGB"), _TL(""));
+	// Colors...
 
-	m_Parameters.Add_Choice("NODE_RGB",
-		"RGB_ATTRIB"	, _TL("Attribute"),
-		_TL(""),
-		_TL("<default>")
-	);
-
-	//-----------------------------------------------------
-	// Memory...
-
-	m_Parameters.Add_Double("NODE_GENERAL",
-		"MAX_SAMPLES"	, _TL("Maximum Samples"),
-		_TL("Maximum number of samples used to build statistics and histograms expressed as percent of the total number of cells."),
-		100. * m_pObject->Get_Max_Samples() / (double)Get_PointCloud()->Get_Count(), 0., true, 100., true
-	);
+	m_Parameters.Add_Node  ("NODE_COLORS", "NODE_RGB" , _TL("RGB"  ), _TL(""));
+	m_Parameters.Add_Choice("NODE_RGB"   , "RGB_FIELD", _TL("Field"), _TL(""), CSG_String::Format("<%s>", _TL("not set")));
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 void CWKSP_PointCloud::On_DataObject_Changed(void)
 {
-	if( m_fValue >= Get_PointCloud()->Get_Field_Count() )
-	{
-		m_fValue = Get_PointCloud()->Get_Field_Count() - 1;
-	}
-
-	double m = Get_PointCloud()->Get_Mean(m_fValue), s = 2. * Get_PointCloud()->Get_StdDev(m_fValue);
-
-	m_Parameters("METRIC_ZRANGE")->asRange()->Set_Range(m - s, m + s);
-
-	//-----------------------------------------------------
-	_AttributeList_Set(m_Parameters("LUT_ATTRIB"   ), false);
-	_AttributeList_Set(m_Parameters("METRIC_ATTRIB"), false);
-	_AttributeList_Set(m_Parameters("RGB_ATTRIB"   ), false);
-
-	//-----------------------------------------------------
-	m_Parameters.Set_Parameter("MAX_SAMPLES", 100. * m_pObject->Get_Max_Samples() / (double)Get_PointCloud()->Get_Count());
+	Set_Fields_Choice(m_Parameters("LUT_FIELD"    ), false, false);
+	Set_Fields_Choice(m_Parameters("LUT_NORMAL"   ),  true,  true);
+	Set_Fields_Choice(m_Parameters("METRIC_FIELD" ),  true, false);
+	Set_Fields_Choice(m_Parameters("METRIC_NORMAL"),  true,  true);
+	Set_Fields_Choice(m_Parameters("RGB_FIELD"    ),  true, false);
 
 	//-----------------------------------------------------
 	CWKSP_Layer::On_DataObject_Changed();
@@ -455,36 +490,44 @@ void CWKSP_PointCloud::On_DataObject_Changed(void)
 //---------------------------------------------------------
 void CWKSP_PointCloud::On_Parameters_Changed(void)
 {
-	CWKSP_Layer::On_Parameters_Changed();
-
-	//-----------------------------------------------------
 	switch( m_Parameters("COLORS_TYPE")->asInt() )
 	{
-	default: m_fValue = -1                                    ; break; // CLASSIFY_SINGLE
-	case  1: m_fValue = m_Parameters("LUT_ATTRIB"   )->asInt(); break; // CLASSIFY_LUT
-	case  2: m_fValue = m_Parameters("METRIC_ATTRIB")->asInt(); break; // CLASSIFY_DISCRETE
-	case  3: m_fValue = m_Parameters("METRIC_ATTRIB")->asInt(); break; // CLASSIFY_GRADUATED
-	case  4: m_fValue = m_Parameters("RGB_ATTRIB"   )->asInt(); break; // CLASSIFY_RGB
+	default: // CLASSIFY_SINGLE
+		m_Stretch.Value  = -1;
+		break;
+
+	case  1: // CLASSIFY_LUT
+		m_Stretch.Value  = Get_Fields_Choice(m_Parameters("LUT_FIELD"    ));
+		m_Stretch.Normal = Get_Fields_Choice(m_Parameters("LUT_NORMAL"   ));
+		m_Stretch.Scale  = 1.;
+		break;
+
+	case  2: // CLASSIFY_DISCRETE
+	case  3: // CLASSIFY_GRADUATED
+		m_Stretch.Value  = Get_Fields_Choice(m_Parameters("METRIC_FIELD" ));
+		m_Stretch.Normal = Get_Fields_Choice(m_Parameters("METRIC_NORMAL"));
+		m_Stretch.Scale  = m_Parameters("METRIC_NORFMT")->asInt() == 0 ? 1. : 100.;
+		break;
+
+	case  4: // CLASSIFY_RGB
+		m_Stretch.Value  = Get_Fields_Choice(m_Parameters("RGB_FIELD"    ));
+		m_Stretch.Normal = -1; m_pClassify->Set_Mode(CLASSIFY_RGB);
+		break;
 	}
 
-	if( m_fValue < 0 || m_fValue >= Get_PointCloud()->Get_Field_Count() )
+	if( m_Stretch.Value < 0 )
 	{
-		m_fValue = -1;
-
 		m_pClassify->Set_Mode(CLASSIFY_SINGLE);
 	}
-	else if( m_Parameters("COLORS_TYPE")->asInt() == 4 ) // CLASSIFY_RGB
-	{
-		m_pClassify->Set_Mode(CLASSIFY_RGB);
-	}
 
 	//-----------------------------------------------------
-	m_pObject->Set_Max_Samples(Get_PointCloud()->Get_Count() * (m_Parameters("MAX_SAMPLES")->asDouble() / 100.) );
-
 	long DefColor = m_Parameters("SINGLE_COLOR")->asColor();
 	m_Color_Pen   = wxColour(SG_GET_R(DefColor), SG_GET_G(DefColor), SG_GET_B(DefColor));
 
 	m_PointSize   = m_Parameters("DISPLAY_SIZE")->asInt() - 1;
+
+	//-----------------------------------------------------
+	CWKSP_Layer::On_Parameters_Changed();
 }
 
 //---------------------------------------------------------
@@ -495,54 +538,21 @@ void CWKSP_PointCloud::On_Update_Views(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 int CWKSP_PointCloud::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter, int Flags)
 {
-	//-----------------------------------------------------
 	if( Flags & PARAMETER_CHECK_VALUES )
 	{
-		if(	pParameter->Cmp_Identifier("METRIC_ATTRIB") )
-		{
-			int zField = pParameter->asInt(); double m = Get_PointCloud()->Get_Mean(zField), s = 2. * Get_PointCloud()->Get_StdDev(zField);
-
-			double min = m - s;	if( min < Get_PointCloud()->Get_Minimum(zField) ) { min = Get_PointCloud()->Get_Minimum(zField); }
-			double max = m + s;	if( max > Get_PointCloud()->Get_Maximum(zField) ) { max = Get_PointCloud()->Get_Maximum(zField); }
-
-			pParameters->Get_Parameter("METRIC_ZRANGE")->asRange()->Set_Range(min, max);
-		}
-
-		if(	pParameter->Cmp_Identifier("LUT_ATTRIB") || (pParameter->Cmp_Identifier("COLORS_TYPE") && pParameter->asInt() == 1) ) // CLASSIFY_LUT
-		{
-			int Field = (*pParameters)("LUT_ATTRIB")->asInt();
-
-			if(	Field >= 0 && Field < Get_PointCloud()->Get_Field_Count() )
-			{
-				TSG_Data_Type Type = SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(Field))
-					? SG_DATATYPE_Double : SG_DATATYPE_String;
-		
-				(*pParameters)("LUT")->asTable()->Set_Field_Type(LUT_MIN, Type);
-				(*pParameters)("LUT")->asTable()->Set_Field_Type(LUT_MAX, Type);
-			}
-		}
+		// nop
 	}
 
 	//-----------------------------------------------------
 	if( Flags & PARAMETER_CHECK_ENABLE )
 	{
-		if(	pParameter->Cmp_Identifier("COLORS_TYPE") )
-		{
-			int Value = pParameter->asInt();
-
-			pParameters->Set_Enabled("NODE_SINGLE", Value == CLASSIFY_SINGLE);
-			pParameters->Set_Enabled("NODE_LUT"   , Value == CLASSIFY_LUT);
-			pParameters->Set_Enabled("NODE_METRIC", Value == CLASSIFY_DISCRETE || Value == CLASSIFY_GRADUATED);
-			pParameters->Set_Enabled("NODE_RGB"	  , Value == 4);
-
-			return( 1 );
-		}
+		// nop
 	}
 
 	//-----------------------------------------------------
@@ -551,303 +561,22 @@ int CWKSP_PointCloud::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Para
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-wxString CWKSP_PointCloud::Get_Name_Attribute(void)
-{
-	return(	m_fValue < 0 || m_pClassify->Get_Mode() == CLASSIFY_SINGLE ? SG_T("") : Get_PointCloud()->Get_Field_Name(m_fValue) );
-}
-
-//---------------------------------------------------------
-void CWKSP_PointCloud::_AttributeList_Set(CSG_Parameter *pFields, bool bAddNoField)
-{
-	if( pFields && pFields->Get_Type() == PARAMETER_TYPE_Choice )
-	{
-		CSG_String s;
-
-		for(int i=0; i<Get_PointCloud()->Get_Field_Count(); i++)
-		{
-			s += Get_PointCloud()->Get_Field_Name(i); s += "|";
-		}
-
-		if( bAddNoField )
-		{
-			s += _TL("<none>");
-
-			pFields->asChoice()->Set_Items(s);
-
-			pFields->Set_Value(Get_PointCloud()->Get_Field_Count());
-		}
-		else
-		{
-			pFields->asChoice()->Set_Items(s);
-		}
-	}
-}
-
-
-///////////////////////////////////////////////////////////
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-void CWKSP_PointCloud::_LUT_Create(void)
-{
-	if( Get_PointCloud()->Get_Field_Count() <= 0 || Get_PointCloud()->Get_Count() < 1 )
-	{
-		DLG_Message_Show(_TL("Function failed because no attributes are available"), _TL("Classify"));
-
-		return;
-	}
-
-	//-----------------------------------------------------
-	static CSG_Parameters Parameters;
-
-	if( Parameters.Get_Count() == 0 )
-	{
-		Parameters.Create(_TL("Classify"));
-		Parameters.Add_Choice("", "FIELD" , _TL("Attribute"     ), _TL(""), "");
-		Parameters.Add_Colors("", "COLOR" , _TL("Colors"        ), _TL(""))->asColors()->Set_Count(11);
-		Parameters.Add_Choice("", "METHOD", _TL("Classification"), _TL(""),
-			CSG_String::Format("%s|%s|%s|%s",
-				_TL("unique values"),
-				_TL("equal intervals"),
-				_TL("quantiles"),
-				_TL("natural breaks")
-			), 0
-		);
-	}
-
-	{
-		CSG_String Fields;
-
-		for(int i=0; i<Get_PointCloud()->Get_Field_Count(); i++)
-		{
-			Fields += CSG_String(Get_PointCloud()->Get_Field_Name(i)) + "|";
-		}
-
-		Parameters("FIELD")->asChoice()->Set_Items(Fields);
-	}
-
-	if( !DLG_Parameters(&Parameters) )
-	{
-		return;
-	}
-
-	//-----------------------------------------------------
-	DataObject_Changed();
-
-	int Field = Parameters("FIELD")->asInt();
-
-	CSG_Colors Colors(*Parameters("COLOR")->asColors());
-
-	CSG_Table Classes(m_Parameters("LUT")->asTable());
-
-	switch( SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(Field)) ? Parameters("METHOD")->asInt() : 0 )
-	{
-	//-----------------------------------------------------
-	case 0:	// unique values
-		{
-			TSG_Data_Type Type = SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(Field))
-				? SG_DATATYPE_Double
-				: SG_DATATYPE_String;
-
-			Classes.Set_Field_Type(LUT_MIN, Type);
-			Classes.Set_Field_Type(LUT_MAX, Type);
-
-			CSG_Unique_String_Statistics Values; CSG_String Value;
-
-			#define MAX_CLASSES	1024
-
-			for(sLong iPoint=0; iPoint<Get_PointCloud()->Get_Count() && Values.Get_Count()<MAX_CLASSES; iPoint++)
-			{
-				if( Get_PointCloud()->Get_Value(iPoint, Field, Value) )
-				{
-					Values += Value;
-				}
-			}
-
-			Colors.Set_Count(Values.Get_Count());
-
-			for(int iClass=0; iClass<Values.Get_Count(); iClass++)
-			{
-				CSG_Table_Record &Class = *Classes.Add_Record(); Value = Values.Get_Value(iClass);
-
-				Class.Set_Value(0, Colors[iClass]); // Color
-				Class.Set_Value(1, Value         ); // Name
-				Class.Set_Value(2, ""            ); // Description
-				Class.Set_Value(3, Value         ); // Minimum
-				Class.Set_Value(4, Value         ); // Maximum
-			}
-		}
-		break;
-
-	//-----------------------------------------------------
-	case 1:	// equal intervals
-		{
-			double Interval = Get_PointCloud()->Get_Range  (Field) / (double)Colors.Get_Count();
-			double Minimum  = Get_PointCloud()->Get_Minimum(Field);
-
-			Classes.Set_Field_Type(LUT_MIN, SG_DATATYPE_Double);
-			Classes.Set_Field_Type(LUT_MAX, SG_DATATYPE_Double);
-
-			for(int iClass=0; iClass<Colors.Get_Count(); iClass++, Minimum+=Interval)
-			{
-				CSG_Table_Record &Class = *Classes.Add_Record();
-
-				double Maximum = iClass < Colors.Get_Count() - 1 ? Minimum + Interval : Get_PointCloud()->Get_Maximum(Field) + 1.;
-
-				CSG_String Name(SG_Get_String(Minimum, -2) + " - " + SG_Get_String(Maximum, -2));
-
-				Class.Set_Value(0, Colors[iClass]); // Color
-				Class.Set_Value(1, Name          ); // Name
-				Class.Set_Value(2, ""            ); // Description
-				Class.Set_Value(3, Minimum       ); // Minimum
-				Class.Set_Value(4, Maximum       ); // Maximum
-			}
-		}
-		break;
-
-	//-----------------------------------------------------
-	case 2:	// quantiles
-		{
-			CSG_Index Index; Get_PointCloud()->Set_Index(Index, Field, true);
-
-			Classes.Set_Field_Type(LUT_MIN, SG_DATATYPE_Double);
-			Classes.Set_Field_Type(LUT_MAX, SG_DATATYPE_Double);
-
-			if( Get_PointCloud()->Get_Count() < Colors.Get_Count() )
-			{
-				Colors.Set_Count(Get_PointCloud()->Get_Count());
-			}
-
-			double Maximum = Get_PointCloud()->Get_Minimum(Field);
-
-			double Step	= Get_PointCloud()->Get_Count() / (double)Colors.Get_Count();
-
-			for(int iClass=0; iClass<Colors.Get_Count(); iClass++)
-			{
-				CSG_Table_Record &Class = *Classes.Add_Record();
-
-				sLong i = (sLong)(Step * (1 + iClass));
-
-				double Minimum = Maximum; Maximum = i < Get_PointCloud()->Get_Count()
-					? Get_PointCloud()->Get_Value(Index[i], Field)
-					: Get_PointCloud()->Get_Maximum(Field) + 1.;
-
-				CSG_String Name(SG_Get_String(Minimum, -2) + " - " + SG_Get_String(Maximum, -2));
-
-				Class.Set_Value(0, Colors[iClass]); // Color
-				Class.Set_Value(1, Name          ); // Name
-				Class.Set_Value(2, ""            ); // Description
-				Class.Set_Value(3, Minimum       ); // Minimum
-				Class.Set_Value(4, Maximum       ); // Maximum
-			}
-		}
-		break;
-
-	//-----------------------------------------------------
-	case 3:	// natural breaks
-		{
-			CSG_Natural_Breaks Breaks(Get_PointCloud(), Field, Colors.Get_Count(), Get_PointCloud()->Get_Count() > 4096 ? 256 : 0);
-
-			if( Breaks.Get_Count() <= Colors.Get_Count() ) return;
-
-			Classes.Set_Field_Type(LUT_MIN, SG_DATATYPE_Double);
-			Classes.Set_Field_Type(LUT_MAX, SG_DATATYPE_Double);
-
-			for(int iClass=0; iClass<Colors.Get_Count(); iClass++)
-			{
-				CSG_Table_Record &Class = *Classes.Add_Record();
-
-				double Minimum = Breaks[iClass    ];
-				double Maximum = Breaks[iClass + 1];
-
-				CSG_String Name(SG_Get_String(Minimum, -2) + " - " + SG_Get_String(Maximum, -2));
-
-				Class.Set_Value(0, Colors[iClass]); // Color
-				Class.Set_Value(1, Name          ); // Name
-				Class.Set_Value(2, ""            ); // Description
-				Class.Set_Value(3, Minimum       ); // Minimum
-				Class.Set_Value(4, Maximum       ); // Maximum
-			}
-		}
-		break;
-	}
-
-	//-----------------------------------------------------
-	if( Classes.Get_Count() > 0 )
-	{
-		m_Parameters("LUT")->asTable()->Assign(&Classes);
-
-		m_Parameters("COLORS_TYPE")->Set_Value(CLASSIFY_LUT); // Lookup Table
-		m_Parameters("LUT_ATTRIB" )->Set_Value(Field);
-
-		Parameters_Changed();
-	}
-}
-
-
-///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 wxString CWKSP_PointCloud::Get_Value(CSG_Point ptWorld, double Epsilon)
 {
-	wxString Value; sLong Index = Get_PointCloud()->Get_Point(ptWorld, Epsilon);
+	wxString s;
 
-	if( Index >= 0 )
-	{
-		switch( m_fValue < 0 ? -1 : m_pClassify->Get_Mode() )
-		{
-		case -1               :
-			Value.Printf("%s: %lld", _TL("Index"), Index + 1);
-			break;
+	CWKSP_Layer::Get_Field_Value(Get_PointCloud()->Get_Point(ptWorld, Epsilon), m_Stretch.Value, m_Stretch.Normal, m_Stretch.Scale, s);
 
-		default               :
-			Value.Printf("%f", Get_PointCloud()->Get_Value(Index, m_fValue));
-			break;
-
-		case CLASSIFY_RGB     : { int color = (int)Get_PointCloud()->Get_Value(Index, m_fValue);
-			Value.Printf("R%03d G%03d B%03d", SG_GET_R(color), SG_GET_G(color), SG_GET_B(color));
-			break; }
-
-		case CLASSIFY_LUT     :
-			if( SG_Data_Type_is_Numeric(Get_PointCloud()->Get_Field_Type(m_fValue)) )
-			{
-				Value = m_pClassify->Get_Class_Name_byValue(Get_PointCloud()->Get_Value(Index, m_fValue));
-			}
-			else
-			{
-				CSG_String s; Get_PointCloud()->Get_Value(Index, m_fValue, s);
-
-				Value = m_pClassify->Get_Class_Name_byValue(s.c_str());
-			}
-			break;
-		}
-	}
-
-	return( Value );
+	return( s );
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-///////////////////////////////////////////////////////////
-
-//---------------------------------------------------------
-double CWKSP_PointCloud::Get_Value_Minimum(void)	{	return( m_fValue < 0 ? 0. : Get_PointCloud()->Get_Minimum(m_fValue) );	}
-double CWKSP_PointCloud::Get_Value_Maximum(void)	{	return( m_fValue < 0 ? 0. : Get_PointCloud()->Get_Maximum(m_fValue) );	}
-double CWKSP_PointCloud::Get_Value_Range  (void)	{	return( m_fValue < 0 ? 0. : Get_PointCloud()->Get_Range  (m_fValue) );	}
-double CWKSP_PointCloud::Get_Value_Mean   (void)	{	return( m_fValue < 0 ? 0. : Get_PointCloud()->Get_Mean   (m_fValue) );	}
-double CWKSP_PointCloud::Get_Value_StdDev (void)	{	return( m_fValue < 0 ? 0. : Get_PointCloud()->Get_StdDev (m_fValue) );	}
-
-
-///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -858,7 +587,7 @@ bool CWKSP_PointCloud::asImage(CSG_Grid *pImage)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -968,6 +697,12 @@ bool CWKSP_PointCloud::Edit_Set_Index(int Index)
 	return( true );
 }
 
+//---------------------------------------------------------
+int CWKSP_PointCloud::Edit_Get_Index(void)
+{
+	return( m_Edit_Index );
+}
+
 //---------------------------------------------------------
 bool CWKSP_PointCloud::Edit_Set_Attributes(void)
 {
@@ -990,7 +725,7 @@ bool CWKSP_PointCloud::Edit_Set_Attributes(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1013,13 +748,13 @@ void CWKSP_PointCloud::On_Draw(CSG_Map_DC &dc_Map, int Flags)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 inline void CWKSP_PointCloud::_Draw_Point(CSG_Map_DC &dc_Map, int x, int y, double z, int Color)
 {
-	if( m_Aggregation == 1 )	// last value
+	if( m_Aggregation == 1 ) // last value
 	{
 		dc_Map.Draw_Image_Pixel(x, y, Color);
 	}
@@ -1037,16 +772,14 @@ inline void CWKSP_PointCloud::_Draw_Point(CSG_Map_DC &dc_Map, int x, int y, doub
 		case 2:	// lowest z
 			if( m_N.asInt(x, y) == 0 || z < m_Z.asDouble(x, y) )
 			{
-				dc_Map.Draw_Image_Pixel(x, y, Color);
-				m_Z.Set_Value(x, y, z);
+				dc_Map.Draw_Image_Pixel(x, y, Color); m_Z.Set_Value(x, y, z);
 			}
 			break;
 
 		case 3:	// highest z
 			if( m_N.asInt(x, y) == 0 || z > m_Z.asDouble(x, y) )
 			{
-				dc_Map.Draw_Image_Pixel(x, y, Color);
-				m_Z.Set_Value(x, y, z);
+				dc_Map.Draw_Image_Pixel(x, y, Color); m_Z.Set_Value(x, y, z);
 			}
 			break;
 		}
@@ -1087,17 +820,16 @@ void CWKSP_PointCloud::_Draw_Points(CSG_Map_DC &dc_Map)
 	}
 
 	//-----------------------------------------------------
-	CSG_PointCloud *pPoints = Get_PointCloud();
+	CSG_PointCloud *pPoints = Get_PointCloud(); bool bNumeric = SG_Data_Type_is_Numeric(pPoints->Get_Field_Type(m_Stretch.Value));
 
 	sLong Selection = pPoints->Get_Selection_Count() > 0 ? pPoints->Get_Selection_Index(m_Edit_Index) : -1;
 
+	#pragma omp parallel for
 	for(sLong i=0; i<pPoints->Get_Count(); i++)
 	{
-		pPoints->Set_Cursor(i);
-
-		if( !pPoints->is_NoData(m_fValue) )
+		if( !pPoints->is_NoData(i, m_Stretch.Value) && (m_Stretch.Normal < 0 || (!pPoints->is_NoData(i, m_Stretch.Normal) && pPoints->Get_Value(i, m_Stretch.Normal) != 0.)) )
 		{
-			TSG_Point_3D Point = pPoints->Get_Point();
+			TSG_Point_3D Point = pPoints->Get_Point(i);
 
 			if( dc_Map.rWorld().Contains(Point.x, Point.y) )
 			{
@@ -1113,20 +845,23 @@ void CWKSP_PointCloud::_Draw_Points(CSG_Map_DC &dc_Map)
 				}
 				else
 				{
-					int Color;
-					
-					if( m_pClassify->Get_Mode() == CLASSIFY_LUT && !SG_Data_Type_is_Numeric(pPoints->Get_Field_Type(m_fValue)) )
+					if( m_pClassify->Get_Mode() == CLASSIFY_LUT && !bNumeric ) // string
 					{
-						CSG_String Value;
+						int Color; CSG_String Value;
 
-						if( pPoints->Get_Value(m_fValue, Value) && m_pClassify->Get_Class_Color_byValue(Value, Color) )
+						if( pPoints->Get_Value(i, m_Stretch.Value, Value) && m_pClassify->Get_Class_Color_byValue(Value, Color) )
 						{
 							_Draw_Point(dc_Map, x, y, Point.z, Color, m_PointSize);
 						}
 					}
-					else if( m_pClassify->Get_Class_Color_byValue(pPoints->Get_Value(m_fValue), Color) )
+					else // number
 					{
-						_Draw_Point(dc_Map, x, y, Point.z, Color, m_PointSize);
+						int Color; double Value = pPoints->Get_Value(i, m_Stretch.Value); if( m_Stretch.Normal >= 0 ) { Value *= m_Stretch.Normal / pPoints->Get_Value(i, m_Stretch.Normal); }
+						
+						if( m_pClassify->Get_Class_Color_byValue(Value, Color) )
+						{
+							_Draw_Point(dc_Map, x, y, Point.z, Color, m_PointSize);
+						}
 					}
 				}
 			}
@@ -1141,20 +876,19 @@ void CWKSP_PointCloud::_Draw_Thumbnail(CSG_Map_DC &dc_Map)
 
 	sLong n = 1 + (sLong)(pPoints->Get_Count() / (2 * dc_Map.rDC().GetWidth() * dc_Map.rDC().GetHeight()));
 
+//	#pragma omp parallel for
 	for(sLong i=0; i<pPoints->Get_Count(); i+=n)
 	{
-		pPoints->Set_Cursor(i);
-
-		if( !pPoints->is_NoData(m_fValue) )
+		if( !pPoints->is_NoData(i, m_Stretch.Value) && (m_Stretch.Normal < 0 || (!pPoints->is_NoData(i, m_Stretch.Normal) && pPoints->Get_Value(i, m_Stretch.Normal) != 0.)) )
 		{
-			TSG_Point_3D Point = pPoints->Get_Point();
+			TSG_Point_3D Point = pPoints->Get_Point(i);
 
 			int x = (int)dc_Map.xWorld2DC(Point.x);
 			int y = (int)dc_Map.yWorld2DC(Point.y);
 
-			int Color;
+			int Color; double Value = pPoints->Get_Value(i, m_Stretch.Value); if( m_Stretch.Normal >= 0 ) { Value *= m_Stretch.Scale / pPoints->Get_Value(i, m_Stretch.Normal); }
 
-			if( m_pClassify->Get_Class_Color_byValue(pPoints->Get_Value(m_fValue), Color) )
+			if( m_pClassify->Get_Class_Color_byValue(Value, Color) )
 			{
 				dc_Map.Draw_Image_Pixel(x, y, Color);
 			}
@@ -1164,9 +898,9 @@ void CWKSP_PointCloud::_Draw_Thumbnail(CSG_Map_DC &dc_Map)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
