@@ -64,9 +64,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -74,9 +74,9 @@ CWKSP_Tool *g_pTool = NULL;
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -108,7 +108,7 @@ CWKSP_Tool::~CWKSP_Tool(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -188,7 +188,7 @@ CSG_Parameters * CWKSP_Tool::Get_Parameters(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -234,7 +234,7 @@ bool CWKSP_Tool::On_Command(int Cmd_ID)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -245,7 +245,7 @@ void CWKSP_Tool::Set_Menu_ID(int Menu_ID)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -262,7 +262,7 @@ bool CWKSP_Tool::is_Executing(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -278,7 +278,7 @@ bool CWKSP_Tool::Set_Projection(const CSG_Projection &Projection)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -320,12 +320,12 @@ bool CWKSP_Tool::Execute(bool bDialog)
 
 	if( m_pTool->On_Before_Execution() )
 	{
-		#ifndef __WXMAC__
-		g_pTools->Set_Recently_Used(this);
-		#endif
-
 		if( !bDialog || DLG_Parameters(m_pTool->Get_Parameters(), "", m_pTool->Get_Summary(false).c_str()) )
 		{
+			#ifndef __WXMAC__
+			g_pTools->Add_Recent(this);
+			#endif
+
 			MSG_General_Add_Line(); MSG_Execution_Add_Line(); STATUSBAR_Set_Text(m_pTool->Get_Name().w_str());
 
 			bResult	= m_pTool->Execute(true);
@@ -341,6 +341,10 @@ bool CWKSP_Tool::Execute(bool bDialog)
 		m_pTool->On_After_Execution();
 
 		g_pActive->Get_Parameters()->Update_Parameters(m_pTool->Get_Parameters(), false);
+
+		#ifdef __WXMAC__
+		g_pTools->Add_Recent(this);
+		#endif
 	}
 
 	if( !bResult || !m_pTool->is_Interactive() )
@@ -384,9 +388,9 @@ bool CWKSP_Tool::Finish(bool bDialog, bool bCloseGUI)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -414,7 +418,7 @@ void CWKSP_Tool::Save_to_Script(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -425,11 +429,11 @@ int CWKSP_Tool::On_Clipboard_Format_Changed(CSG_Parameter *pParameter, int Flags
 		CSG_Parameters &Parameters = *pParameter->Get_Parameters();
 
 		Parameters.Set_Enabled("CMD"    , Parameters["TYPE"].asInt() == 0);
-		Parameters.Set_Enabled("ARGS"   , Parameters["TYPE"].asInt() != 2 || (Parameters["TYPE"].asInt() == 2 && Parameters["PYTHON"].asInt() != 1));
+		Parameters.Set_Enabled("ARGS"   , (Parameters["TYPE"].asInt() != 2 && Parameters["TYPE"].asInt() != 3) || (Parameters["TYPE"].asInt() == 2 && Parameters["PYTHON"].asInt() != 1));
 		Parameters.Set_Enabled("WRAP"   , Parameters["TYPE"].asInt() == 0 || (Parameters["TYPE"].asInt() == 2 && Parameters["PYTHON"].asInt() == 0));
 		Parameters.Set_Enabled("PYTHON" , Parameters["TYPE"].asInt() == 2);
 		Parameters.Set_Enabled("PY_FUNC",                                    (Parameters["TYPE"].asInt() == 2 && Parameters["PYTHON"].asInt() != 2));
-
+		Parameters.Set_Enabled("MD"     , Parameters["TYPE"].asInt() == 3);
 		CSG_String Arguments;
 
 		if( Parameters["TYPE"].asInt() == 2 && Parameters["PYTHON"].asInt() == 0 )
@@ -462,10 +466,11 @@ void CWKSP_Tool::Save_to_Clipboard(void)
 		Options.Add_Choice("",
 			"TYPE"   , _TL("Script Type"),
 			_TL(""),
-			CSG_String::Format("%s|%s|%s",
+			CSG_String::Format("%s|%s|%s|%s",
 				_TL("Command Line"),
 				_TL("Tool Chain"),
-				_TL("Python")
+				_TL("Python"),
+				_TL("Markdown Table")
 			)
 		);
 
@@ -501,6 +506,16 @@ void CWKSP_Tool::Save_to_Clipboard(void)
 			), 0
 		);
 
+		Options.Add_Choice("TYPE",
+			"MD", _TL("Parameters"),
+			_TL(""),
+			CSG_String::Format("%s|%s|%s",
+				_TL("Command Line"),
+				_TL("GUI"),
+				_TL("all")
+			), 1
+		);
+
 		Options.Add_Bool("",
 			"HEADER" , _TL("Header"),
 			_TL(""),
@@ -571,6 +586,13 @@ void CWKSP_Tool::Save_to_Clipboard(void)
 				);
 				break;
 			}
+			break;
+
+		case  3: // Markdown table
+			Script = m_pTool->Get_Script(CSG_Tool::Script_Format::Markdown_Table,
+				Options["HEADER"].asBool(), Options["MD"].asInt(), false
+			);
+			break;
 		}
 
 		if( !Script.is_Empty() && wxTheClipboard->Open() )
@@ -584,9 +606,9 @@ void CWKSP_Tool::Save_to_Clipboard(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
