@@ -50,9 +50,9 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -93,13 +93,13 @@ CFilter_Gauss::CFilter_Gauss(void)
 	Parameters.Add_Double("",
 		"SIGMA"			, _TL("Standard Deviation"),
 		_TL("The standard deviation as percentage of the kernel radius."),
-		50., 1., true
+		30., 1., true
 	);
 }
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -117,21 +117,36 @@ bool CFilter_Gauss::On_Execute(void)
 	}
 
 	//-----------------------------------------------------
-	double Bandwidth = Radius * Parameters("SIGMA")->asDouble() / 100.;
+	double Sigma = Radius * Parameters("SIGMA")->asDouble() / 100.; double Bandwidth = 2. * Sigma*Sigma;
 
 	for(int i=0; i<Kernel.Get_NY(); i++)
 	{
 		for(int j=0; j<Kernel.Get_NX(); j++)
 		{
-			double d = SG_Get_Distance(i, j, Radius, Radius) / Bandwidth;
+			double d = SG_Get_Distance(i, j, Radius, Radius);
 
-			Kernel[i][j] = exp(-0.5 * d*d);
+			Kernel[i][j] = exp(-d*d / Bandwidth) / (M_PI * Bandwidth);
+
+		//	double d = SG_Get_Distance(i, j, Radius, Radius) / Sigma; Kernel[i][j] = exp(-0.5 * d*d); // gives identical weighting pattern with center cell's weight = 1.0
 		}
 	}
 
-	//Message_Fmt("\n\n\n");
-	//Message_Fmt(Kernel.to_String(-1, 2).c_str());
-	//Message_Fmt("\n\n\n");
+	if( true ) // print kernel
+	{
+		Message_Fmt("\n\nSigma = %f, %dx%d %s:\n", Sigma, Kernel.Get_NX(), Kernel.Get_NY(), _TL("Kernel"));
+
+		for(int i=0; i<Kernel.Get_NY(); i++) for(int j=0; j<Kernel.Get_NX(); j++)
+		{
+			Message_Fmt("%.6f%c", Kernel[i][j], j < Kernel.Get_NX() - 1 ? '\t' : '\n');
+		}
+
+		Message_Fmt("\nSigma = %f, %dx%d %s (%s):\n", Sigma, Kernel.Get_NX(), Kernel.Get_NY(), _TL("Kernel"), _TL("normalized"));
+
+		for(int i=0; i<Kernel.Get_NY(); i++) for(int j=0; j<Kernel.Get_NX(); j++)
+		{
+			Message_Fmt("%.6f%c", Kernel[i][j] / Kernel[Radius][Radius], j < Kernel.Get_NX() - 1 ? '\t' : '\n');
+		}
+	}
 
 	//-----------------------------------------------------
 	CSG_Grid Input, *pInput = Parameters("INPUT")->asGrid();
@@ -195,9 +210,9 @@ bool CFilter_Gauss::On_Execute(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
