@@ -361,7 +361,7 @@ bool CGDAL_Import_WMS::Get_WMS_System(CSG_Grid_System &System, CSG_Projection &P
 	{
 		CSG_Rect r(-180., -90., 180., 90.);
 		
-		if( SG_Get_Projected(CSG_Projection::Get_GCS_WGS84(), pTarget->Get_Projection(), r) )
+		if( SG_Get_Projected(CSG_Projection::Get_GCS_WGS84(), pTarget->Get_Projection(), r) && r.Get_Area() > 0. )
 		{
 			Extent.Intersect(r);
 		}
