@@ -88,7 +88,7 @@ CSG_String SG_Get_OGR_WKB_Type_Choice_Name(int Type)
 {
 	switch( Type )
 	{
-	default                           :	return( _TL("Automatic"                 ) );
+	default                           :	return( _TL("auto-detect"               ) );
 	case OGR_WKB_POINT                :	return( _TL("Point"                     ) );
 	case OGR_WKB_POINT25D             :	return( _TL("Point (2.5D)"              ) );
 	case OGR_WKB_MULTIPOINT           :	return( _TL("Multi-Point"               ) );
@@ -285,7 +285,7 @@ CSG_String CSG_OGR_Drivers::Get_Description(int Index) const
 
 	s += OGR_Dr_TestCapability(pDriver, OLCSequentialWrite  ) ? SG_T("\n[x]") : SG_T("\n[ ]");
 	s += _TL("sequential write");
-/**/
+*/
 
 	return( s );
 }
@@ -463,9 +463,12 @@ CSG_OGR_DataSet::CSG_OGR_DataSet(void)
 
 CSG_OGR_DataSet::CSG_OGR_DataSet(const CSG_String &File)
 {
-	m_pDataSet = NULL;
+	m_pDataSet = NULL; Create(File);
+}
 
-	Create(File);
+CSG_OGR_DataSet::CSG_OGR_DataSet(const CSG_String &File, const CSG_String &DriverName, const CSG_String &Options)
+{
+	m_pDataSet = NULL; Create(File, DriverName, Options);
 }
 
 //---------------------------------------------------------
@@ -482,7 +485,14 @@ bool CSG_OGR_DataSet::Create(const CSG_String &File)
 
 	m_pDataSet = GDALOpenEx(File.to_UTF8().Get_Data(), GDAL_OF_VECTOR, NULL, NULL, NULL);
 
-	return( m_pDataSet != NULL );
+	if( m_pDataSet != NULL )
+	{
+		m_File = File;
+
+		return( true );
+	}
+
+	return( false );
 }
 
 bool CSG_OGR_DataSet::Create(const CSG_String &File, const CSG_String &DriverName, const CSG_String &Options)
@@ -512,7 +522,14 @@ bool CSG_OGR_DataSet::Create(const CSG_String &File, const CSG_String &DriverNam
         CSLDestroy(pOptions);
 	}
 
-	return( m_pDataSet != NULL );
+	if( m_pDataSet != NULL )
+	{
+		m_File = File;
+
+		return( true );
+	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -525,6 +542,8 @@ bool CSG_OGR_DataSet::Destroy(void)
 		m_pDataSet = NULL;
 	}
 
+	m_File.Clear();
+
 	return( true );
 }
 
@@ -579,7 +598,14 @@ bool CSG_OGR_DataSet::Create(const CSG_String &File)
 
 	m_pDataSet = OGROpen(File, 0, 0);
 
-	return( m_pDataSet != NULL );
+	if( m_pDataSet != NULL )
+	{
+		m_File = File;
+
+		return( true );
+	}
+
+	return( false );
 }
 
 bool CSG_OGR_DataSet::Create(const CSG_String &File, const CSG_String &DriverName, const CSG_String &Options)
@@ -598,7 +624,14 @@ bool CSG_OGR_DataSet::Create(const CSG_String &File, const CSG_String &DriverNam
         CSLDestroy(pOptions);
 	}
 
-	return( m_pDataSet != NULL );
+	if( m_pDataSet != NULL )
+	{
+		m_File = File;
+
+		return( true );
+	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -654,37 +687,37 @@ OGRLayerH CSG_OGR_DataSet::Get_Layer(int iLayer) const
 }
 
 //---------------------------------------------------------
-TSG_Shape_Type CSG_OGR_DataSet::Get_Type(int iLayer) const
+OGRwkbGeometryType CSG_OGR_DataSet::Get_Geomety_Type(int iLayer, bool bForce) const
 {
 	OGRLayerH pLayer = Get_Layer(iLayer);
 
-	if( !pLayer )
+	if( pLayer )
 	{
-		return( SHAPE_TYPE_Undefined );
-	}
+		OGRwkbGeometryType Type = OGR_FD_GetGeomType(OGR_L_GetLayerDefn(pLayer));
 
-	OGRwkbGeometryType Type = OGR_FD_GetGeomType(OGR_L_GetLayerDefn(pLayer));
-
-	if( Type == wkbNone || Type == wkbUnknown )
-	{
-		OGR_L_ResetReading(pLayer);
-
-		OGRFeatureH pFeature;
-
-		while( (Type == wkbNone || Type == wkbUnknown) && (pFeature = OGR_L_GetNextFeature(pLayer)) != NULL )
+		if( (Type == wkbNone || Type == wkbUnknown) && bForce )
 		{
-			if( OGR_F_GetGeometryRef(pFeature) )
+			OGR_L_ResetReading(pLayer);
+
+			OGRFeatureH pFeature;
+
+			while( (Type == wkbNone || Type == wkbUnknown) && (pFeature = OGR_L_GetNextFeature(pLayer)) != NULL )
 			{
-				Type = OGR_G_GetGeometryType(OGR_F_GetGeometryRef(pFeature));
+				if( OGR_F_GetGeometryRef(pFeature) )
+				{
+					Type = OGR_G_GetGeometryType(OGR_F_GetGeometryRef(pFeature));
+				}
+
+				OGR_F_Destroy(pFeature);
 			}
 
-			OGR_F_Destroy(pFeature);
+			OGR_L_ResetReading(pLayer);
 		}
 
-		OGR_L_ResetReading(pLayer);
+		return( Type );
 	}
 
-	return( CSG_OGR_Drivers::Get_Shape_Type(Type) );
+	return( wkbUnknown );
 }
 
 //---------------------------------------------------------
@@ -711,7 +744,7 @@ CSG_Projection CSG_OGR_DataSet::Get_Projection(int iLayer)	const
 		{
 			Projection.Create(WKT);
 
-			OGRFree(WKT);
+			CPLFree(WKT);
 		}
 	}
 
@@ -726,7 +759,7 @@ CSG_Projection CSG_OGR_DataSet::Get_Projection(int iLayer)	const
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-CSG_Shapes * CSG_OGR_DataSet::Read(int iLayer, int iGeomTypeChoice)
+CSG_Shapes * CSG_OGR_DataSet::Read(int iLayer, int ForceToOGRType)
 {
 	OGRLayerH pLayer = Get_Layer(iLayer);
 
@@ -735,43 +768,38 @@ CSG_Shapes * CSG_OGR_DataSet::Read(int iLayer, int iGeomTypeChoice)
 		return( NULL );
 	}
 
-	OGRFeatureDefnH pDefn = OGR_L_GetLayerDefn(pLayer);
+	OGRwkbGeometryType Type = ForceToOGRType ? SG_Get_OGR_WKB_Type_Choice_Key(ForceToOGRType) : Get_Geomety_Type(iLayer, true);
 
-	if( iGeomTypeChoice != 0 )
-	{
-		OGR_FD_SetGeomType(pDefn, (OGRwkbGeometryType)SG_Get_OGR_WKB_Type_Choice_Key(iGeomTypeChoice));
-	}
-
-	if( Get_Type(iLayer) == SHAPE_TYPE_Undefined )
+	if( CSG_OGR_Drivers::Get_Shape_Type(Type) == SHAPE_TYPE_Undefined )
 	{
 		return( NULL );
 	}
 
 	//-----------------------------------------------------
 #ifdef GDAL_V1_8_OR_OLDER
-	CSG_Shapes *pShapes = SG_Create_Shapes(Get_Type(iLayer), CSG_String(OGR_FD_GetName(OGR_L_GetLayerDefn(pLayer))), NULL, Get_Coordinate_Type(iLayer));
+	CSG_Shapes *pShapes = SG_Create_Shapes(CSG_OGR_Drivers::Get_Shape_Type(Type), CSG_String(OGR_FD_GetName(OGR_L_GetLayerDefn(pLayer))), NULL, Get_Coordinate_Type(iLayer));
 #else
-	CSG_Shapes *pShapes = SG_Create_Shapes(Get_Type(iLayer), CSG_String(OGR_L_GetName(pLayer)), NULL, Get_Coordinate_Type(iLayer));
+	CSG_Shapes *pShapes = SG_Create_Shapes(CSG_OGR_Drivers::Get_Shape_Type(Type), CSG_String(OGR_L_GetName(pLayer)), NULL, Get_Coordinate_Type(iLayer));
 #endif
 
 	pShapes->Get_Projection() = Get_Projection(iLayer);
 
 	//-----------------------------------------------------
-	{
-		for(int iField=0; iField<OGR_FD_GetFieldCount(pDefn); iField++)
-		{
-			OGRFieldDefnH pDefnField = OGR_FD_GetFieldDefn(pDefn, iField);
+	OGRFeatureDefnH pDefn = OGR_L_GetLayerDefn(pLayer);
 
-			pShapes->Add_Field(OGR_Fld_GetNameRef(pDefnField), CSG_OGR_Drivers::Get_Data_Type(OGR_Fld_GetType(pDefnField)));
-		}
+	for(int iField=0; iField<OGR_FD_GetFieldCount(pDefn); iField++)
+	{
+		OGRFieldDefnH pDefnField = OGR_FD_GetFieldDefn(pDefn, iField);
+
+		pShapes->Add_Field(OGR_Fld_GetNameRef(pDefnField), CSG_OGR_Drivers::Get_Data_Type(OGR_Fld_GetType(pDefnField)));
 	}
 
 	//-----------------------------------------------------
 	OGR_L_ResetReading(pLayer);
 
-	OGRFeatureH pFeature;
+	OGRFeatureH pFeature; sLong nFeatures = OGR_L_GetFeatureCount(pLayer, 1);
 
-	while( (pFeature = OGR_L_GetNextFeature(pLayer)) != NULL && SG_UI_Process_Get_Okay(false) )
+	while( (pFeature = OGR_L_GetNextFeature(pLayer)) != NULL && (nFeatures < 0 ? SG_UI_Process_Get_Okay(true) : SG_UI_Process_Set_Progress(pShapes->Get_Count(), nFeatures)) )
 	{
 		OGRGeometryH pGeometry = OGR_F_GetGeometryRef(pFeature);
 
@@ -796,7 +824,7 @@ CSG_Shapes * CSG_OGR_DataSet::Read(int iLayer, int iGeomTypeChoice)
 			}
 
 			//---------------------------------------------
-			if( _Read_Geometry(pShape, pGeometry) == false || !pShape->is_Valid() )
+			if( _Read_Geometry(pShape, pGeometry) == false )
 			{
 				pShapes->Del_Shape(pShape);
 			}
@@ -819,53 +847,45 @@ CSG_Shapes * CSG_OGR_DataSet::Read(int iLayer, int iGeomTypeChoice)
 //---------------------------------------------------------
 bool CSG_OGR_DataSet::_Read_Geometry(CSG_Shape *pShape, OGRGeometryH pGeometry)
 {
-	if( !pShape || !pGeometry )
+	if( pShape && pGeometry )
 	{
-		return( false );
-	}
-
-	switch( OGR_G_GetGeometryType(pGeometry) )
-	{
-	//-----------------------------------------------------
-	case wkbPoint             : // 0-dimensional geometric object, standard WKB
-	case wkbPoint25D          : // 2.5D extension as per 99-402
-		pShape->Add_Point(OGR_G_GetX(pGeometry, 0), OGR_G_GetY(pGeometry, 0));
-		pShape->Set_Z(OGR_G_GetZ(pGeometry, 0), 0);
-		return( true );
-
-	//-----------------------------------------------------
-	case wkbLineString        : // 1-dimensional geometric object with linear interpolation between Points, standard WKB
-	case wkbLineString25D     : // 2.5D extension as per 99-402
-		return( _Read_Line(pShape, pGeometry) );
-
-	//-----------------------------------------------------
-	case wkbPolygon           : // planar 2-dimensional geometric object defined by 1 exterior boundary and 0 or more interior boundaries, standard WKB
-	case wkbPolygon25D        : // 2.5D extension as per 99-402
-		return( _Read_Polygon(pShape, pGeometry) );
-
-	//-----------------------------------------------------
-	case wkbMultiPoint        : // GeometryCollection of Points, standard WKB
-	case wkbMultiPoint25D     : // 2.5D extension as per 99-402
-	case wkbMultiLineString   : // GeometryCollection of LineStrings, standard WKB
-	case wkbMultiLineString25D: // 2.5D extension as per 99-402
-	case wkbMultiPolygon      : // GeometryCollection of Polygons, standard WKB
-	case wkbMultiPolygon25D   : // 2.5D extension as per 99-402
+		switch( OGR_G_GetGeometryType(pGeometry) )
 		{
+		//-------------------------------------------------
+		case wkbPoint             : // 0-dimensional geometric object, standard WKB
+		case wkbPoint25D          : // 2.5D extension as per 99-402
+			pShape->Set_Point(OGR_G_GetX(pGeometry, 0), OGR_G_GetY(pGeometry, 0)); pShape->Set_Z(OGR_G_GetZ(pGeometry, 0));
+			return( true );
+
+		//-------------------------------------------------
+		case wkbLineString        : // 1-dimensional geometric object with linear interpolation between Points, standard WKB
+		case wkbLineString25D     : // 2.5D extension as per 99-402
+			return( _Read_Line(pShape, pGeometry) );
+
+		//-------------------------------------------------
+		case wkbPolygon           : // planar 2-dimensional geometric object defined by 1 exterior boundary and 0 or more interior boundaries, standard WKB
+		case wkbPolygon25D        : // 2.5D extension as per 99-402
+			return( _Read_Polygon(pShape, pGeometry) );
+
+		//-------------------------------------------------
+		case wkbMultiPoint        : // GeometryCollection of Points, standard WKB
+		case wkbMultiPoint25D     : // 2.5D extension as per 99-402
+		case wkbMultiLineString   : // GeometryCollection of LineStrings, standard WKB
+		case wkbMultiLineString25D: // 2.5D extension as per 99-402
+		case wkbMultiPolygon      : // GeometryCollection of Polygons, standard WKB
+		case wkbMultiPolygon25D   : // 2.5D extension as per 99-402
 			for(int i=0; i<OGR_G_GetGeometryCount(pGeometry); i++)
 			{
-				if( _Read_Geometry(pShape, OGR_G_GetGeometryRef(pGeometry, i)) == false )
-				{
-					return( false );
-				}
+				_Read_Geometry(pShape, OGR_G_GetGeometryRef(pGeometry, i));
 			}
+			return( pShape->is_Valid() );
+
+		//-------------------------------------------------
+		default: break;
 		}
-
-		return( true );
-
-	//-----------------------------------------------------
-	default:
-		return( false );
 	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
@@ -1035,7 +1055,10 @@ bool CSG_OGR_DataSet::Write(CSG_Shapes *pShapes, const CSG_String &CreationOptio
 				}
 			}
 
-			OGR_L_CreateFeature(pLayer, pFeature);
+			if( OGR_L_CreateFeature(pLayer, pFeature) != OGRERR_NONE )
+			{
+				// nop
+			}
 		}
 
 		OGR_F_Destroy(pFeature);
