@@ -118,8 +118,12 @@ CTiles_Provider::CTiles_Provider(bool bLogin)
 	Parameters.Add_Double("EXTENT", "XMAX", _TL("East"   ), _TL(""),  931320.);
 	Parameters.Add_Double("EXTENT", "YMIN", _TL("South"  ), _TL(""), 5225850.);
 	Parameters.Add_Double("EXTENT", "YMAX", _TL("North"  ), _TL(""), 6111540.);
-	Parameters.Add_Int   ("EXTENT", "NX"  , _TL("Columns"), _TL(""),    7345, 1, true);
-	Parameters.Add_Int   ("EXTENT", "NY"  , _TL("Rows"   ), _TL(""),    9842, 1, true);
+
+	if( has_GUI() )
+	{
+		Parameters.Add_Int("EXTENT", "NX" , _TL("Columns"), _TL(""), 7345, 1, true)->Set_UseInCMD(false);
+		Parameters.Add_Int("EXTENT", "NY" , _TL("Rows"   ), _TL(""), 9842, 1, true)->Set_UseInCMD(false);
+	}
 
 	Parameters.Add_Double("",
 		"BUFFER"     , _TL("Buffer"),
@@ -160,7 +164,7 @@ bool CTiles_Provider::On_After_Execution(void)
 //---------------------------------------------------------
 int CTiles_Provider::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if( pParameter->Cmp_Identifier("CELLSIZE") || (pParameter->Get_Parent() && pParameter->Get_Parent()->Cmp_Identifier("EXTENT")) )
+	if( has_GUI() && (pParameter->Cmp_Identifier("CELLSIZE") || (pParameter->Get_Parent() && pParameter->Get_Parent()->Cmp_Identifier("EXTENT"))) )
 	{
 		double Cellsize = (*pParameters)("CELLSIZE")->asDouble();
 		double xMin     = (*pParameters)("XMIN"    )->asDouble();
@@ -388,7 +392,14 @@ bool CTiles_Provider::On_Execute(void)
 	{
 		pGrid->Set_Name(m_Grid_Name);
 
-		Parameters.Set_Parameter("RESULT", pGrid);
+		if( Parameters["RESULT"].asGrid() )
+		{
+			Parameters["RESULT"].asGrid()->Create(*pGrid); delete(pGrid);
+		}
+		else
+		{
+			Parameters.Set_Parameter("RESULT", pGrid);
+		}
 
 		return( true );
 	}
@@ -424,7 +435,14 @@ bool CTiles_Provider::On_Execute(void)
 	//--------------------------------------------------------
 	pGrid->Set_Name(m_Grid_Name);
 
-	Parameters.Set_Parameter("RESULT", pGrid);
+	if( Parameters["RESULT"].asGrid() )
+	{
+		Parameters["RESULT"].asGrid()->Create(*pGrid); delete(pGrid);
+	}
+	else
+	{
+		Parameters.Set_Parameter("RESULT", pGrid);
+	}
 
 	return( true );
 }
