@@ -65,10 +65,16 @@ COpenData_DGM1::COpenData_DGM1(void)
 
 	Parameters.Add_FilePath("",
 		"TILES"      , _TL("Local Tiles Directory"),
-		_TL("Download location for tiles. If requested tile is already present download will be skipped."),
+		_TL("Download location for tiles. If requested tile is already present download will be skipped. Takes user's temporary directory if empty."),
 		NULL, NULL, true, true
 	);
 
+	//-----------------------------------------------------
+	if( has_GUI() )
+	{
+		Parameters.Add_Bool("", "SHOW_ERROR_DLG", _TL("Show Error Dialog"), _TL("Show error message dialog if a single tile download or preparation fails."));
+	}
+
 	//-----------------------------------------------------
 	Parameters.Add_Choice("",
 		"EXTENT"     , _TL("Extent"),
@@ -450,7 +456,34 @@ bool COpenData_DGM1::On_Execute(void)
 	//--------------------------------------------------------
 	pGrid->Set_Name(m_Name);
 
-	Parameters.Set_Parameter("RESULT", pGrid);
+	if( Parameters["RESULT"].asGrid() )
+	{
+		Parameters["RESULT"].asGrid()->Create(*pGrid); delete(pGrid);
+	}
+	else
+	{
+		Parameters.Set_Parameter("RESULT", pGrid);
+	}
+
+	return( true );
+}
+
+
+///////////////////////////////////////////////////////////
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+bool COpenData_DGM1::Show_Error(const CSG_String &Message)
+{
+	if( !Parameters("SHOW_ERROR_DLG") || Parameters("SHOW_ERROR_DLG")->asBool() )
+	{
+		Error_Set(Message);
+	}
+	else
+	{
+		Message_Add("\n" + Message);
+	}
 
 	return( true );
 }
@@ -542,7 +575,7 @@ int COpenData_DGM1::Provide_Tile(const CSG_String &Directory, const CSG_String &
 
 		Message_Fmt(_TL("failed"));
 
-		Error_Fmt("%s:\n\n%s/%s", _TL("failed to request file from server"), m_ServerPath.c_str(), Request.c_str());
+		Show_Error(CSG_String::Format("%s:\n\n%s/%s", _TL("failed to request file from server"), m_ServerPath.c_str(), Request.c_str()));
 
 		return( -1 );
 	}
@@ -558,7 +591,7 @@ int COpenData_DGM1::Provide_Tile(const CSG_String &Directory, const CSG_String &
 
 		Message_Fmt(_TL("failed"));
 
-		Error_Fmt("%s:\n\n%s/%s", _TL("failed to request file from server"), m_ServerPath.c_str(), Request.c_str());
+		Show_Error(CSG_String::Format("%s:\n\n%s/%s", _TL("failed to request file from server"), m_ServerPath.c_str(), Request.c_str()));
 
 		return( -1 );
 	}
@@ -601,7 +634,7 @@ int COpenData_DGM1::Provide_Tile(const CSG_String &Directory, const CSG_String &
 
 			Message_Fmt(_TL("failed"));
 
-			Error_Fmt("%s (\"%s\")\n", _TL("conversion failed"), File.c_str());
+			Show_Error(CSG_String::Format("%s (\"%s\")\n", _TL("conversion failed"), File.c_str()));
 
 			return( -1 );
 		}
@@ -611,6 +644,14 @@ int COpenData_DGM1::Provide_Tile(const CSG_String &Directory, const CSG_String &
 		SG_UI_ProgressAndMsg_Lock(false);
 	}
 
+	//-----------------------------------------------------
+	if( !On_Provide_Tile(File) )
+	{
+		Message_Fmt(_TL("failed"));
+
+		return( -1 );
+	}
+
 	//-----------------------------------------------------
 	Message_Fmt(_TL("okay"));
 
@@ -1362,6 +1403,34 @@ CSG_String COpenData_DGM1_Brandenburg::Get_Tile_Name(int Col, int Row)
 	);
 }
 
+//---------------------------------------------------------
+bool COpenData_DGM1_Brandenburg::On_Provide_Tile(const CSG_String &File)
+{
+	// GeoTIFF's CRS for some tiles is defined in WKT1 for others
+	// in WKT2, what let's the VRT creation fail on completion
+	// (The GDAL does not recognize both definitions as identical!).
+
+	static CSG_Projection CRS(25833);
+
+	if( true )
+	{
+		SG_UI_ProgressAndMsg_Lock(true);
+
+		CSG_Grid Grid(File);
+
+		if( Grid.is_Valid() )
+		{
+			Grid.Get_Projection().Create(CRS);
+
+			Grid.Save(File);
+		}
+
+		SG_UI_ProgressAndMsg_Lock(false);
+	}
+
+	return( true );
+}
+
 
 ///////////////////////////////////////////////////////////
 //                                                       //
