@@ -92,6 +92,8 @@ protected:
 
 	virtual bool				On_Execute				(void);
 
+	bool						Show_Error				(const CSG_String &Message);
+
 	bool						Set_Data_CRS			(int EPSG);
 	bool						Set_Default_Extent		(double xMin, double yMin, double xMax, double yMax, double Cellsize = 1.);
 
@@ -101,6 +103,8 @@ protected:
 	virtual int					Provide_Tile			(const CSG_String &Directory, int Col, int Row);
 	virtual int					Provide_Tile			(const CSG_String &Directory, const CSG_String &Tile, const CSG_String &Request);
 
+	virtual bool				On_Provide_Tile			(const CSG_String &File) { return( true ); }
+
 
 private:
 
@@ -303,6 +307,8 @@ protected:
 
 	virtual CSG_String			Get_Tile_Name			(int Col, int Row);
 
+	virtual bool				On_Provide_Tile			(const CSG_String &File);
+
 };
 
 
