@@ -160,8 +160,12 @@ COpenTopography::COpenTopography(bool bLogin)
 	Parameters.Add_Double("EXTENT", "XMAX", _TL("East"   ), _TL(""),  598020.);
 	Parameters.Add_Double("EXTENT", "YMIN", _TL("South"  ), _TL(""), 5906910.);
 	Parameters.Add_Double("EXTENT", "YMAX", _TL("North"  ), _TL(""), 5965170.);
-	Parameters.Add_Int   ("EXTENT", "NX"  , _TL("Columns"), _TL(""),    1990, 1, true);
-	Parameters.Add_Int   ("EXTENT", "NY"  , _TL("Rows"   ), _TL(""),    1943, 1, true);
+
+	if( has_GUI() )
+	{
+		Parameters.Add_Int("EXTENT", "NX" , _TL("Columns"), _TL(""), 7345, 1, true)->Set_UseInCMD(false);
+		Parameters.Add_Int("EXTENT", "NY" , _TL("Rows"   ), _TL(""), 9842, 1, true)->Set_UseInCMD(false);
+	}
 
 	Parameters.Add_Double("",
 		"BUFFER"     , _TL("Buffer"),
@@ -211,7 +215,7 @@ bool COpenTopography::On_After_Execution(void)
 //---------------------------------------------------------
 int COpenTopography::On_Parameter_Changed(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
 {
-	if( pParameter->Cmp_Identifier("CELLSIZE") || (pParameter->Get_Parent() && pParameter->Get_Parent()->Cmp_Identifier("EXTENT")) )
+	if( has_GUI() && (pParameter->Cmp_Identifier("CELLSIZE") || (pParameter->Get_Parent() && pParameter->Get_Parent()->Cmp_Identifier("EXTENT"))) )
 	{
 		double Cellsize = (*pParameters)("CELLSIZEDEF")->asInt() == 1
 			? (*pParameters)("CELLSIZE")->asDouble() // user defined
@@ -439,7 +443,7 @@ bool COpenTopography::On_Execute(void)
 		}
 	}
 
-	CSG_Grid *pGrid = Parameters("RESULT")->asGrid(); if( !pGrid ) { Parameters("pGrid")->Set_Value(pGrid = SG_Create_Grid()); }
+	CSG_Grid *pGrid = Parameters["RESULT"].asGrid(); if( !pGrid ) { Parameters["RESULT"].Set_Value(pGrid = SG_Create_Grid()); }
 
 	//--------------------------------------------------------
 	if( Grid.Get_Projection() == Projection || !Grid.Get_Projection().is_Okay() )
