@@ -52,20 +52,27 @@
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 CTWI::CTWI(void)
 {
-	Set_Name		(_TL("Topographic Wetness Index (TWI)"));
+	Set_Name		(_TL("Topographic Wetness Index"));
 
 	Set_Author		("O.Conrad (c) 2003");
 
 	Set_Description	(_TW(
-		"Calculation of the slope and specific catchment area (SCA) based Topographic Wetness Index (TWI)."
+		"Calculates the Topographic Wetness Index (TWI) using the supplied rasters for "
+		"slope (Beta), specific catchment area (SCA), and optional transmissivity (T) "
+		"when the soil profile is saturated:\n\n"
+		"TWI = ln(SCA / tan(Beta))\n"
+		"\n"
+		"or with transmissivity\n"
+		"\n"
+		"TWI = ln(SCA / (T * tan(Beta)))"
 	));
 
 	Add_Reference(
@@ -133,88 +140,63 @@ CTWI::CTWI(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CTWI::On_Execute(void)
 {
-	bool		bTopmodel, bConvert;
-	int			x, y;
-	double		Area, Slope, Kf, Mean_Kf, Mean_TWI;
-	CSG_Grid	*pArea, *pSlope, *pKf, *pTWI;
+	CSG_Grid *pSlope = Parameters("SLOPE" )->asGrid();
+	CSG_Grid *pArea  = Parameters("AREA"  )->asGrid();
+	CSG_Grid *pKf    = Parameters("TRANS" )->asGrid();
+	CSG_Grid *pTWI   = Parameters("TWI"   )->asGrid();
+
+	bool bConvert    = Parameters("CONV"  )->asInt() == 1;
+	bool bTopmodel   = Parameters("METHOD")->asInt() == 1;
+
+	DataObject_Set_Colors(pTWI, 0, SG_COLORS_RED_GREY_BLUE);
 
 	//-----------------------------------------------------
-	pSlope		= Parameters("SLOPE" )->asGrid();
-	pArea		= Parameters("AREA"  )->asGrid();
-	pKf			= Parameters("TRANS" )->asGrid();
-	pTWI		= Parameters("TWI"   )->asGrid();
-
-	bConvert	= Parameters("CONV"  )->asInt() == 1;
-	bTopmodel	= Parameters("METHOD")->asInt() == 1;
-
-	DataObject_Set_Colors(pTWI, 11, SG_COLORS_RED_GREY_BLUE);
-
-	//-----------------------------------------------------
-	Kf			= 1.;
+	double Mean_Kf = 0., Mean_TWI = 0.;
 
 	if( bTopmodel )
 	{
-		Mean_TWI	= 0.;
-		Mean_Kf		= 0.;
-		Area		= 0.;
+		sLong Area = 0;
 
-		for(y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
+		for(int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
 		{
-			for(x=0; x<Get_NX(); x++)
+			for(int x=0; x<Get_NX(); x++)
 			{
 				if( !pArea->is_NoData(x, y) && !pSlope->is_NoData(x, y) )
 				{
-					if( pKf )
+					double Kf = pKf ? 0. : 1.;
+
+					if( pKf && !pKf->is_NoData(x, y) )
 					{
-						if( !pKf->is_NoData(x, y) )
-						{
-							Kf	= pKf->asDouble(x, y);
-
-							if( Kf	< M_ALMOST_ZERO )
-								Kf	= M_ALMOST_ZERO;
-
-							Kf	= log(Kf);
-						}
-						else
-						{
-							Kf	= 0.;
-						}
+						Kf = pKf->asDouble(x, y); if( Kf < M_FLT_EPSILON ) { Kf = M_FLT_EPSILON; }
+						Kf = log(Kf);
 					}
 
-					Slope	= tan(pSlope->asDouble(x, y));
+					double Slope = tan(pSlope->asDouble(x, y)); if( Slope < M_FLT_EPSILON ) { Slope = M_FLT_EPSILON; }
 
-					if( Slope	< M_ALMOST_ZERO )
-						Slope	= M_ALMOST_ZERO;
-
-					Area		++;
-					Mean_TWI	+= log((pArea->asDouble(x, y) / (bConvert ? Get_Cellsize() : 1.)) / Slope);
-					Mean_Kf		+= Kf;
+					Area++; Mean_Kf += Kf; Mean_TWI += log((pArea->asDouble(x, y) / (bConvert ? Get_Cellsize() : 1.)) / Slope);
 				}
 			}
 		}
 
-		if( Area > 0. )
-		{
-			Area		*= pArea->Get_Cellarea();
-			Mean_TWI	/= Area;
-			Mean_Kf		/= Area;
-		}
-		else
+		if( Area < 1 )
 		{
 			return( false );
 		}
+
+		Mean_Kf /= Area * pArea->Get_Cellarea(); Mean_TWI /= Area * pArea->Get_Cellarea();
 	}
 
 	//-----------------------------------------------------
-	for(y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
+	for(int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
 	{
-		for(x=0; x<Get_NX(); x++)
+		#pragma omp parallel for
+		for(int x=0; x<Get_NX(); x++)
 		{
 			if( pArea->is_NoData(x, y) || pSlope->is_NoData(x, y) || (pKf && pKf->is_NoData(x, y)) )
 			{
@@ -222,23 +204,9 @@ bool CTWI::On_Execute(void)
 			}
 			else
 			{
-				Area	= pArea->asDouble(x, y);
-
-				if( bConvert )
-					Area	/= Get_Cellsize();
-
-				Slope	= tan(pSlope->asDouble(x, y));
-
-				if( Slope	< M_ALMOST_ZERO )
-					Slope	= M_ALMOST_ZERO;
-
-				if( pKf )
-				{
-					Kf	= pKf->asDouble(x, y);
-
-					if( Kf	< M_ALMOST_ZERO )
-						Kf	= M_ALMOST_ZERO;
-				}
+				double Area  =      pArea->asDouble(x, y)     ; if( bConvert ) { Area /= Get_Cellsize(); }
+				double Slope = tan(pSlope->asDouble(x, y))    ; if( Slope < M_FLT_EPSILON ) { Slope	= M_FLT_EPSILON; }
+				double Kf    =  pKf ? pKf->asDouble(x, y) : 1.; if( Kf    < M_FLT_EPSILON ) { Kf    = M_FLT_EPSILON; }
 
 				pTWI->Set_Value(x, y, bTopmodel
 					? (log(Area / (     Slope)) - Mean_TWI) - (log(Kf) - Mean_Kf)
@@ -253,9 +221,9 @@ bool CTWI::On_Execute(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -307,7 +275,7 @@ CStream_Power::CStream_Power(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -319,7 +287,7 @@ bool CStream_Power::On_Execute(void)
 
 	bool	bConvert	= Parameters("CONV" )->asInt() == 1;
 
-	DataObject_Set_Colors(pSPI, 11, SG_COLORS_RED_GREY_GREEN, true);
+	DataObject_Set_Colors(pSPI, 0, SG_COLORS_RED_GREY_GREEN, true);
 
 	//-----------------------------------------------------
 	for(int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
@@ -343,9 +311,9 @@ bool CStream_Power::On_Execute(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -405,7 +373,7 @@ CCIT::CCIT(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -417,7 +385,7 @@ bool CCIT::On_Execute(void)
 
     bool	bConvert	= Parameters("CONV" )->asInt() == 1;
 
-    DataObject_Set_Colors(pCIT, 11, SG_COLORS_RED_GREY_GREEN, true);
+    DataObject_Set_Colors(pCIT, 0, SG_COLORS_RED_GREY_GREEN, true);
 
     //-----------------------------------------------------
     for(int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
@@ -441,9 +409,9 @@ bool CCIT::On_Execute(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -491,26 +459,35 @@ CLS_Factor::CLS_Factor(void)
 
 	//-----------------------------------------------------
 	Parameters.Add_Grid("",
-		"SLOPE"		, _TL("Slope"),
+		"SLOPE"     , _TL("Slope"),
 		_TL(""),
 		PARAMETER_INPUT
 	);
 
+	Parameters.Add_Choice("SLOPE",
+		"SLOPE_UNIT", _TL("Unit"),
+		_TL(""),
+		CSG_String::Format("%s|%s",
+			_TL("radians"),
+			_TL("degree")
+		), 0
+	);
+
 	Parameters.Add_Grid("",
-		"AREA"		, _TL("Catchment Area"),
+		"AREA"      , _TL("Catchment Area"),
 		_TL(""),
 		PARAMETER_INPUT
 	);
 
 	//-----------------------------------------------------
 	Parameters.Add_Grid("",
-		"LS"		, _TL("LS Factor"),
+		"LS"        , _TL("LS Factor"),
 		_TL(""),
 		PARAMETER_OUTPUT
 	);
 
 	Parameters.Add_Choice("",
-		"CONV"		, _TL("Area to Length Conversion"),
+		"CONV"      , _TL("Area to Length Conversion"),
 		_TL("Derivation of slope lengths from catchment areas. These are rough approximations! Applies not to Desmet & Govers' method."),
 		CSG_String::Format("%s|%s|%s",
 			_TL("no conversion (areas already given as specific catchment area)"),
@@ -520,14 +497,14 @@ CLS_Factor::CLS_Factor(void)
 	);
 
 	Parameters.Add_Bool("",
-		"FEET"		, _TL("Feet Adjustment"),
+		"FEET"      , _TL("Feet Adjustment"),
 		_TL("Needed if area and lengths come from coordinates measured in feet."),
 		false
 	);
 
 	//-----------------------------------------------------
 	Parameters.Add_Choice("",
-		"METHOD"	, _TL("Method (LS)"),
+		"METHOD"    , _TL("Method (LS)"),
 		_TL(""),
 		CSG_String::Format("%s|%s|%s",
 			_TL("Moore et al. 1991"),
@@ -537,18 +514,18 @@ CLS_Factor::CLS_Factor(void)
 	);
 
 	Parameters.Add_Node("",
-		"NODE_DG"	, _TL("Desmet & Govers"),
+		"NODE_DG"   , _TL("Desmet & Govers"),
 		_TL("")
 	);
 
 	Parameters.Add_Double("NODE_DG",
-		"EROSIVITY"	, _TL("Rill/Interrill Erosivity"),
+		"EROSIVITY" , _TL("Rill/Interrill Erosivity"),
 		_TL(""),
 		1., 0., true
 	);
 
 	Parameters.Add_Choice("NODE_DG",
-		"STABILITY"	, _TL("Stability"),
+		"STABILITY" , _TL("Stability"),
 		_TL(""),
 		CSG_String::Format("%s|%s",
 			_TL("stable"),
@@ -559,25 +536,42 @@ CLS_Factor::CLS_Factor(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+int CLS_Factor::On_Parameters_Enable(CSG_Parameters *pParameters, CSG_Parameter *pParameter)
+{
+	if( pParameter->Cmp_Identifier("METHOD") )
+	{
+		pParameters->Set_Enabled("NODE_DG", pParameter->asInt() == 1); // Desmet & Govers
+	}
+
+	return( CSG_Tool_Grid::On_Parameters_Enable(pParameters, pParameter) );
+}
+
+
+///////////////////////////////////////////////////////////
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
 bool CLS_Factor::On_Execute(void)
 {
-	CSG_Grid	*pSlope	= Parameters("SLOPE")->asGrid();
-	CSG_Grid	*pArea	= Parameters("AREA" )->asGrid();
-	CSG_Grid	*pLS	= Parameters("LS"   )->asGrid();
+	CSG_Grid *pSlope = Parameters("SLOPE"     )->asGrid();
+	CSG_Grid  *pArea = Parameters("AREA"      )->asGrid();
+	CSG_Grid    *pLS = Parameters("LS"        )->asGrid();
 
-	int	Conversion	= Parameters("CONV")->asInt();
+	int   Conversion = Parameters("CONV"      )->asInt();
 
-	bool	bFeet	= Parameters("FEET")->asBool();
+	double    dSlope = Parameters("SLOPE_UNIT")->asInt() == 0 ? 1. : M_DEG_TO_RAD;
+	bool       bFeet = Parameters("FEET"      )->asBool();
 
-	m_Method	= Parameters("METHOD"   )->asInt();
-	m_Erosivity	= Parameters("EROSIVITY")->asInt();
-	m_Stability	= Parameters("STABILITY")->asInt();
+	m_Method         = Parameters("METHOD"    )->asInt();
+	m_Erosivity      = Parameters("EROSIVITY" )->asInt();
+	m_Stability      = Parameters("STABILITY" )->asInt();
 
-	DataObject_Set_Colors(pLS, 11, SG_COLORS_RED_GREY_GREEN, true);
+	DataObject_Set_Colors(pLS, 0, SG_COLORS_RED_GREY_GREEN, true);
 
 	//-----------------------------------------------------
 	for(int y=0; y<Get_NY() && Set_Progress_Rows(y); y++)
@@ -591,8 +585,8 @@ bool CLS_Factor::On_Execute(void)
 			}
 			else
 			{
-				double	Slope	= pSlope->asDouble(x, y);
-				double	Area	= pArea ->asDouble(x, y);
+				double Slope = pSlope->asDouble(x, y) * dSlope;
+				double Area  = pArea ->asDouble(x, y);
 
 				switch( Conversion )
 				{
@@ -613,52 +607,52 @@ inline double CLS_Factor::Get_LS(double Slope, double SCA, bool bFeet)
 {
 	if( bFeet )
 	{
-		SCA	*= 0.3048;
+		SCA *= 0.3048;
 	}
 
-	double	LS, sinSlope = sin(Slope);
+	double LS, sinSlope = sin(Slope);
 
 	switch( m_Method )
 	{
 	//-----------------------------------------------------
 	default: {
-		LS		= (0.4 + 1) * pow(SCA / 22.13, 0.4) * pow(sinSlope / 0.0896, 1.3);
+		LS     = (0.4 + 1) * pow(SCA / 22.13, 0.4) * pow(sinSlope / 0.0896, 1.3);
 		break; }
 
 	//-----------------------------------------------------
 	case  1: {
-		double	L, S, m, x, d;
+		double L, S, m, x, d;
 
-		d		= (bFeet ? 0.3048 : 1.) * Get_Cellsize();
+		d      = (bFeet ? 0.3048 : 1.) * Get_Cellsize();
 
-		m		= m_Erosivity * (sinSlope / 0.0896) / (3. * pow(sinSlope, 0.8) + 0.56);
-		m		= m / (1. + m);
+		m      = m_Erosivity * (sinSlope / 0.0896) / (3. * pow(sinSlope, 0.8) + 0.56);
+		m      = m / (1. + m);
 
-		x		= 1.;
+		x      = 1.;
 
 		// x : coefficient that adjusts for width of flow at the center of the cell.
 		// It has a value of 1. when the flow is toward a side and sqrt(2.) when the flow is toward a corner.
 		// (Peter I. A. Kinnell: 'ALTERNATIVE APPROACHES FOR DETERMINING THE USLE-M SLOPE LENGTH FACTOR FOR GRID CELLS'
 		// https://www.soils.org/publications/sssaj/abstracts/69/3/0674)
 
-		L		= (pow(SCA + d*d, m + 1.) - pow(SCA, m + 1.))
-				/ (pow(d, m + 2.) * pow(22.13, m) * pow(x, m));
+		L      = (pow(SCA + d*d, m + 1.) - pow(SCA, m + 1.))
+		       / (pow(d, m + 2.) * pow(22.13, m) * pow(x, m));
 
 		//-------------------------------------------------
 		if( Slope < 0.08975817419 )		// < 9% Steigung := atan(0.09), ca. 5 Degree
 		{
-			S	= 10.8 * sinSlope + 0.03;	
+			S  = 10.8 * sinSlope + 0.03;	
 		}
 		else if( m_Stability == 0 )		// >= 9% Steigung, stabil
 		{
-			S	= 16.8 * sinSlope - 0.5;
+			S  = 16.8 * sinSlope - 0.5;
 		}
 		else							// >= 9% Steigung, tauend u. instabil
 		{
-			S	= pow(sinSlope / 0.896, 0.6);
+			S  = pow(sinSlope / 0.896, 0.6);
 		}
 
-		LS		= L * S;
+		LS     = L * S;
 
 		break; }
 
@@ -666,13 +660,13 @@ inline double CLS_Factor::Get_LS(double Slope, double SCA, bool bFeet)
 	case  2: {
 		if( Slope > 0.0505 )	// >  ca. 3 Degree
 		{
-			LS	= sqrt(SCA / 22.13)
-				* (65.41 * sinSlope * sinSlope + 4.56 * sinSlope + 0.065);
+			LS = sqrt(SCA / 22.13)
+			   * (65.41 * sinSlope * sinSlope + 4.56 * sinSlope + 0.065);
 		}
 		else					// <= ca. 3 Degree
 		{
-			LS	= pow (SCA / 22.13, 3. * pow(Slope, 0.6))
-				* (65.41 * sinSlope * sinSlope + 4.56 * sinSlope + 0.065);
+			LS = pow (SCA / 22.13, 3. * pow(Slope, 0.6))
+			   * (65.41 * sinSlope * sinSlope + 4.56 * sinSlope + 0.065);
 		}
 		break; }
 	}
@@ -682,9 +676,9 @@ inline double CLS_Factor::Get_LS(double Slope, double SCA, bool bFeet)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -729,7 +723,7 @@ CTCI_Low::CTCI_Low(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -740,7 +734,7 @@ bool CTCI_Low::On_Execute(void)
 	CSG_Grid	*pTWI		= Parameters("TWI"     )->asGrid();
 	CSG_Grid	*pTCI_Low	= Parameters("TCILOW"  )->asGrid();
 
-	DataObject_Set_Colors(pTCI_Low, 11, SG_COLORS_RED_GREY_BLUE, false);
+	DataObject_Set_Colors(pTCI_Low, 0, SG_COLORS_RED_GREY_BLUE, false);
 
 	double	dMax	= pDistance->Get_Max();
 	double	dRange	= pDistance->Get_Range();
@@ -773,9 +767,9 @@ bool CTCI_Low::On_Execute(void)
 
 
 ///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
+//                                                       //
+//                                                       //
+//                                                       //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
